﻿using System;
using System.Collections.Generic;
using System.Linq;

namespace GlobalCF
{
	public static class RSR2_BMPKR_Helper
	{
		public static GKDriver Create()
		{
			var driver = new GKDriver()
			{
				DriverTypeNo = 0xF9,
				DriverType = GKDriverType.RSR2_BMPKR,
				UID = new Guid("D34219D9-3B49-4B12-866C-F75CB2B93845"),
				Name = "Блок модульного пожаротушения",
				ShortName = "БМП",
				IsPlaceable = true,
				HasLogic = true,
				DriverClassification = GKDriver.DriverClassifications.ActuatingDevice,
				GroupDeviceChildrenCount = 4,
				HasUsers = true,
				IsControlDevice = true
			};

			driver.AutoCreateChildren.Add(GKDriverType.RSR2_BMPK);
			driver.AutoCreateChildren.Add(GKDriverType.RSR2_BMPK);
			driver.AutoCreateChildren.Add(GKDriverType.RSR2_BMPUI);
			driver.AutoCreateChildren.Add(GKDriverType.RSR2_BMPP);
			driver.AutoCreateChildren.Add(GKDriverType.RSR2_BMPKR_Part);
			driver.AutoCreateChildren.Add(GKDriverType.RSR2_BMPKR_Part);

			GKDriversHelper.AddControlAvailableStates(driver);
			driver.AvailableStateBits.Add(GKStateBit.Off);
			driver.AvailableStateBits.Add(GKStateBit.Failure);
			driver.AvailableStateBits.Add(GKStateBit.TurningOn);
			driver.AvailableStateBits.Add(GKStateBit.TurningOff);
			driver.AvailableStateBits.Add(GKStateBit.Test);

			GKDriversHelper.AddAvailableStateClasses(driver, XStateClass.On);
			GKDriversHelper.AddAvailableStateClasses(driver, XStateClass.Off);
			GKDriversHelper.AddAvailableStateClasses(driver, XStateClass.TurningOn);
			GKDriversHelper.AddAvailableStateClasses(driver, XStateClass.TurningOff);
			GKDriversHelper.AddAvailableStateClasses(driver, XStateClass.AutoOff);
			GKDriversHelper.AddAvailableStateClasses(driver, XStateClass.Test);

			GKDriversHelper.AddCommandBit(driver, GKStateBit.TurnOn_InManual, 0, "Пуск");
			GKDriversHelper.AddCommandBit(driver, GKStateBit.TurnOff_InManual, 0, "Стоп");
			GKDriversHelper.AddCommandBit(driver, GKStateBit.SetRegime_Manual);
			GKDriversHelper.AddCommandBit(driver, GKStateBit.SetRegime_Automatic);

			GKDriversHelper.AddIntProprety(driver, 0, "Длина АЛС 3", 0, 0, 195).CanNotEdit = true;
			GKDriversHelper.AddIntProprety(driver, 1, "Длина АЛС 4", 0, 0, 195).CanNotEdit = true;
			GKDriversHelper.AddIntProprety(driver, 2, "Задержка 1, c", 0, 0, 65535);
			GKDriversHelper.AddIntProprety(driver, 3, "Задержка 2, c", 0, 0, 65535);
			GKDriversHelper.AddIntProprety(driver, 4, "Задержка 3, c", 30, 1, 65535);

			var propertyOne = new GKDriverProperty()
			{
				No = 5,
				Name = "Логика датчиков Окна-Двери",
				Caption = "Логика датчиков Окна-Двери",
				Default = 0,
				IsLowByte = true
			};
			GKDriversHelper.AddPropertyParameter(propertyOne, "Ожидание закрытия всех О-Д, повторный отсчёт ЗАДЕРЖКИ 3, проверка, подрыв", 0);
			GKDriversHelper.AddPropertyParameter(propertyOne, "Ожидание закрытия всех О-Д, подрыв", 1);
			GKDriversHelper.AddPropertyParameter(propertyOne, "Подрыв", 2);
			driver.Properties.Add(propertyOne);

			var propertyTwo = new GKDriverProperty()
			{
				No = 5,
				Name = "Условие пуска",
				Caption = "Условие пуска",
				Default = 3,
				Min = 1,
				Max = 200,
				IsHieghByte = true,
				DriverPropertyType = GKDriverPropertyTypeEnum.IntType
			};
			driver.Properties.Add(propertyTwo);

			var devicePropertyNo = 1;
			var deviceTypes = Enum.GetValues(typeof(BMPDeviceType)).OfType<BMPDeviceType>().Select(x => x.ToString()).ToList();
			for (byte propertyNo = 6; propertyNo <= 55; propertyNo++)
			{

				var property1 = new GKDriverProperty()
				{
					No = propertyNo,
					Name = $"Устройство { devicePropertyNo}",
					Caption = $"Устройство { devicePropertyNo++}",
					Default = 0,
					IsLowByte = true,
					Mask = 0xF,
					CanNotEdit = true
				};

				var property2 = new GKDriverProperty()
				{
					No = propertyNo,
					Name = $"Устройство { devicePropertyNo}",
					Caption = $"Устройство { devicePropertyNo++}",
					Default = 0,
					IsLowByte = true,
					Mask = 0xF0,
					CanNotEdit = true
				};

				var property3 = new GKDriverProperty()
				{
					No = propertyNo,
					Name = $"Устройства { devicePropertyNo}",
					Caption = $"Устройство { devicePropertyNo++}",
					Default = 0,
					IsHieghByte = true,
					Mask = 0xF,
					CanNotEdit = true
				};

				var property4 = new GKDriverProperty()
				{
					No = propertyNo,
					Name = $"Устройство { devicePropertyNo}",
					Caption = $"Устройство { devicePropertyNo++}",
					Default = 0,
					IsHieghByte = true,
					Mask = 0xF0,
					CanNotEdit = true
				};

				foreach (BMPDeviceType deviceType in Enum.GetValues(typeof(BMPDeviceType)))
				{
					GKDriversHelper.AddPropertyParameter(property1, deviceType.ToString(), (int)deviceType);
					GKDriversHelper.AddPropertyParameter(property2, deviceType.ToString(), (int)deviceType * 16);
					GKDriversHelper.AddPropertyParameter(property3, deviceType.ToString(), (int)deviceType);
					GKDriversHelper.AddPropertyParameter(property4, deviceType.ToString(), (int)deviceType * 16);
				}

				driver.Properties.Add(property1);
				driver.Properties.Add(property2);
				driver.Properties.Add(property3);
				driver.Properties.Add(property4);
			}

			driver.MeasureParameters.Add(new GKMeasureParameter { No = 1, Name = "Задержка 1", IsDelay = true, IsNotVisible = true });
			driver.MeasureParameters.Add(new GKMeasureParameter { No = 2, Name = "Задержка 2", IsDelay = true, IsNotVisible = true });
			driver.MeasureParameters.Add(new GKMeasureParameter { No = 3, Name = "Задержка 3", IsDelay = true, IsNotVisible = true });

			return driver;
		}
	}
}