﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.ComponentModel;

namespace GlobalCF
{
	public class GKDriver
	{
		public GKDriver()
		{
			Children = new List<GKDriverType>();
			Properties = new List<GKDriverProperty>();
			MeasureParameters = new List<GKMeasureParameter>();
			AutoCreateChildren = new List<GKDriverType>();
			AvailableStateBits = new List<GKStateBit>();
			AvailableStateClasses = new List<XStateClass>();
			AvailableCommandBits = new List<CommandBit>();
			CanEditAddress = false;
			HasAddress = true;
			IsDeviceOnShleif = true;
			IsReal = true;
			IsPlaceable = false;
			AutoCreateAddresses = new List<byte>();
		}
		public uint RSspeed { set; get; }
		public ushort RSparity { set; get; }
		public ushort RSdatabit { set; get; }
		public ushort RSstopbit { set; get; }
		public ushort RSReadtimeout { set; get; }
		public ushort RSWritetimeout { set; get; }
		public string IPaddress1 { get; set; }
		public string IPaddress2 { get; set; }
		public string Mask1 { get; set; }
		public string Mask2 { get; set; }
		public string Mac1 { get; set; }
		public string Mac2 { get; set; }
		public int MDBethernetPortNum { get; set; }
		public byte KAUlogicalNum { get; set; }
		
		public GKDriverType DriverType { get; set; }
		public ushort DriverTypeNo { get; set; }
		public Guid UID { get; set; }
		public string Name { get; set; }
		public string ShortName { get; set; }
		public string DeviceClassName { get; set; }
		public List<GKDriverState> XStates { get; set; }

		public List<GKDriverProperty> Properties { get; set; }
		public List<GKStateBit> AvailableStateBits { get; set; }
		public List<XStateClass> AvailableStateClasses { get; set; }
		public List<CommandBit> AvailableCommandBits { get; set; }
		public List<GKMeasureParameter> MeasureParameters { get; set; }

		public List<GKDriverType> Children { get; set; }
		public List<GKDriverType> AutoCreateChildren { get; set; }

		public bool HasAddress { get; set; }
		public bool CanEditAddress { get; set; }
		public bool IsRangeEnabled { get; set; }
		public bool IsAutoCreate { get; set; }
		public byte MinAddress { get; set; }
		public int MaxAddress { get; set; }
		public byte MinAddress2 { get; set; }
		public byte MaxAddress2 { get; set; }
		public bool IsSeparateAutoCreateAddresses { get; set; }
		public List<byte> AutoCreateAddresses { get; set; }
		public byte MaxAddressOnShleif { get; set; }
		public bool IsDeviceOnShleif { get; set; }
		public bool IsReal { get; set; }
		public bool HasLogic { get; set; }
		public bool IgnoreHasLogic { get; set; }
		public bool HasZone { get; set; }
		public bool HasMirror { get; set; }
		public bool IsEditMirror { get; set; }
		public bool HasGuardZone { get; set; }
		public bool IsControlDevice { get; set; }
		public bool IsPlaceable { get; set; }
		public bool IsGroupDevice { get; set; }
		public GKDriverType GroupDeviceChildType { get; set; }
		public byte GroupDeviceChildrenCount { get; set; }
		public bool IsLine { get; set; }
		public bool HasUsers { get; set; }
		public bool IsKau
		{
			get { return DriverType == GKDriverType.RSR2_KAU; }
		}
		public bool IsKauOrTPU
		{
			get { return DriverType == GKDriverType.RSR2_KAU || DriverType == GKDriverType.GKMirror; }
		}

		public bool IsAm
		{
			get
			{
				return DriverType == GKDriverType.RSR2_AM_1 || DriverType == GKDriverType.RSR2_AM_2 || DriverType == GKDriverType.RSR2_AM_4
					|| DriverType == GKDriverType.RSR2_MAP4_Group || DriverType == GKDriverType.RSR2_MAP4
					|| DriverType == GKDriverType.RSR2_MAV_1 || DriverType == GKDriverType.RSR2_MAV_2 || DriverType == GKDriverType.RSR2_MAV_4;
			}
		}

		public bool IsPump
		{
			get
			{
				return DriverType == GKDriverType.RSR2_Bush_Drenazh || DriverType == GKDriverType.RSR2_Bush_Jokey || DriverType == GKDriverType.RSR2_Bush_Fire;
			}
		}

		public bool IsCardReaderOrCodeReader
		{
			get
			{
				return DriverType == GKDriverType.RSR2_CardReader || DriverType == GKDriverType.RSR2_CodeCardReader;
			}
		}
		public string ImageSource
		{
			get { return "GKModule/Images/" + DriverType + ".png"; }
		}
		public override bool Equals(object obj)
		{
			if (obj == null || !(obj is GKDriver))
				return false;
			return ((GKDriver)obj).UID == this.UID;
		}

		public DriverClassifications DriverClassification { get; set; }

		public enum DriverClassifications
		{
			[Description("Извещатели пожарные")]
			FireDetector = 1,

			[Description("Извещатели охранные")]
			IntruderDetector = 2,

			[Description("Исполнительные устройства")]
			ActuatingDevice = 3,

			[Description("Шкафы управления")]
			ControlCabinet = 4,

			[Description("Оповещатели")]
			Announcers = 5,

			[Description("Прочие")]
			Other = 7,

			[Description("Контроль доступа")]
			AccessControl = 8,

			[Description("Устройства взрывозащиты")]
			ExplosionProtection = 9,
		}
	}

	public class CommandBit
	{
		public GKStateBit GKStateBit { get; set; }
		public byte AdditionalByte { get; set; }
		public string Name { get; set; }
	}

	public enum GKStateBit
	{
		[DescriptionAttribute("Автоматика включена")]
		Norm = 0,

		[DescriptionAttribute("Внимание")]
		Attention = 1,

		[DescriptionAttribute("Сработка 1")]
		Fire1 = 2,

		[DescriptionAttribute("Сработка 2")]
		Fire2 = 3,

		[DescriptionAttribute("Тест")]
		Test = 4,

		[DescriptionAttribute("Неисправность")]
		Failure = 5,

		[DescriptionAttribute("Отключено")]
		Ignore = 6,

		[DescriptionAttribute("Включено")]
		On = 7,

		[DescriptionAttribute("Выключено")]
		Off = 8,

		[DescriptionAttribute("Включается")]
		TurningOn = 9,

		[DescriptionAttribute("Выключается")]
		TurningOff = 10,

		[DescriptionAttribute("Включить для режима Автоматика")]
		TurnOn_InAutomatic = 11,

		[DescriptionAttribute("Резерв")]
		Reserve1 = 12,

		[DescriptionAttribute("Выключить для режима Автоматика")]
		TurnOff_InAutomatic = 13,

		[DescriptionAttribute("Остановить")]
		Stop_InManual = 14,

		[DescriptionAttribute("Запретить пуск")]
		ForbidStart_InManual = 15,

		[DescriptionAttribute("Включить немедленно для режима Автоматика")]
		TurnOnNow_InAutomatic = 16,

		[DescriptionAttribute("Выключить немедленно для режима Автоматика")]
		TurnOffNow_InAutomatic = 17,

		[DescriptionAttribute("Перевести в режим Автоматика")]
		SetRegime_Automatic = 18,

		[DescriptionAttribute("Перевести в режим Ручное")]
		SetRegime_Manual = 19,

		[DescriptionAttribute("Включить")]
		TurnOn_InManual = 20,

		[DescriptionAttribute("Выключить")]
		TurnOff_InManual = 21,

		[DescriptionAttribute("Включить немедленно")]
		TurnOnNow_InManual = 22,

		[DescriptionAttribute("Выключить немедленно")]
		TurnOffNow_InManual = 23,

		[DescriptionAttribute("Перевести в режим Отключение")]
		SetRegime_Off = 24,

		[DescriptionAttribute("Сброс")]
		Reset = 25,

		[DescriptionAttribute("Резерв")]
		Reserve2 = 26,

		[DescriptionAttribute("Сохранение состояния")]
		Save = 31,

		[DescriptionAttribute("Нет")]
		No = 32,
	}

	public enum XStateClass
	{
		[DescriptionAttribute("Потеря связи с сервером")]
		ConnectionServerLost = -1,

		[DescriptionAttribute("Отсутствует лицензия")]
		HasNoLicense = 0,

		[DescriptionAttribute("База данных прибора не соответствует базе данных ПК")]
		DBMissmatch = 1,

		[DescriptionAttribute("Контроллер в технологическом режиме")]
		TechnologicalRegime = 2,

		[DescriptionAttribute("Потеря связи")]
		ConnectionLost = 3,

		[DescriptionAttribute("Пожар 2")]
		Fire2 = 4,

		[DescriptionAttribute("Пожар 1")]
		Fire1 = 5,

		[DescriptionAttribute("Внимание")]
		Attention = 6,

		[DescriptionAttribute("Отключено")]
		Ignore = 7,

		[DescriptionAttribute("Неисправность")]
		Failure = 8,

		[DescriptionAttribute("Включено")]
		On = 9,

		[DescriptionAttribute("Включается")]
		TurningOn = 10,

		[DescriptionAttribute("Выключается")]
		TurningOff = 11,

		[DescriptionAttribute("Автоматика отключена")]
		AutoOff = 12,

		[DescriptionAttribute("Требуется обслуживание")]
		Service = 13,

		[DescriptionAttribute("Тест")]
		Test = 14,

		[DescriptionAttribute("Информация")]
		Info = 15,

		[DescriptionAttribute("Выключено")]
		Off = 16,

		[DescriptionAttribute("Неизвестно")]
		Unknown = 17,

		[DescriptionAttribute("Норма")]
		Norm = 18,

		[DescriptionAttribute("Нет")]
		No = 19
	}

	public class GKMeasureParameter
	{
		public byte No { get; set; }
		public string Name { get; set; }
		public bool IsDelay { get; set; }
		public string InternalName { get; set; }
		public double? Multiplier { get; set; }
		public bool HasNegativeValue { get; set; }
		public bool IsNotVisible { get; set; }
	}

	public class GKDriverState
	{
		public string Code { get; set; }
		public string Name { get; set; }
		public List<XStateClass> XStateClass { get; set; }
	}

}
