﻿using System;
using System.Collections.Generic;
using System.Linq;

namespace GlobalCF
{
	public static class IEnumerableExtensions
	{
		public static void ForEach<T>(this IEnumerable<T> source, Action<T> action)
		{
			if (source != null)
				foreach (T item in source)
					action(item);
		}
		public static bool IsEmpty<T>(this IEnumerable<T> source)
		{
			return source == null || !source.Any();
		}
		public static bool IsEmpty(this string source)
		{
			return string.IsNullOrWhiteSpace(source);
		}

		public static IEnumerable<T> Flatten<T>(this IEnumerable<T> e, Func<T, IEnumerable<T>> f)
		{
			return e.SelectMany(c => f(c).Flatten(f)).Concat(e);
		}

		public static bool IsNullOrEmpty<T>(this IEnumerable<T> enumerable)
		{
			switch (enumerable)
			{
				case null:
					return true;
				case ICollection<T> collection:
					return collection.Count < 1;
				default:
					return !enumerable.Any();
			}
		}
	}
}
