﻿using System;

namespace GlobalCF
{
	public static class KD_KDTD_Helper
	{
		public static GKDriver Create()
		{
			var driver = new GKDriver()
			{
				DriverTypeNo = 0xF7,
				DriverType = GKDriverType.KD_KDTD,
				UID = new Guid("EB21B6CE-6AE8-40C5-92C7-5111D8418163"),
				Name = "ТД КД",
				ShortName = "КДТД",
				IsControlDevice = true,
				HasLogic = true,
				IsPlaceable = true,
				IsAutoCreate = true,
				MinAddress = 9,
				MaxAddress = 16,
			};

			GKDriversHelper.AddControlAvailableStates(driver);
			GKDriversHelper.AddAvailableStateBits(driver, GKStateBit.Fire1);
			driver.AvailableStateBits.Add(GKStateBit.Off);
			GKDriversHelper.AddAvailableStateClasses(driver, XStateClass.AutoOff);
			GKDriversHelper.AddAvailableStateClasses(driver, XStateClass.On);
			GKDriversHelper.AddAvailableStateClasses(driver, XStateClass.Off);
			GKDriversHelper.AddAvailableStateClasses(driver, XStateClass.TurningOff);
			GKDriversHelper.AddAvailableStateClasses(driver, XStateClass.Test);
			GKDriversHelper.AddAvailableStateClasses(driver, XStateClass.Fire1);

			GKDriversHelper.AddCommandBit(driver, GKStateBit.Test);
			GKDriversHelper.AddCommandBit(driver, GKStateBit.Fire2);
			GKDriversHelper.AddCommandBit(driver, GKStateBit.TurnOn_InManual);
			GKDriversHelper.AddCommandBit(driver, GKStateBit.TurnOff_InManual);
			GKDriversHelper.AddCommandBit(driver, GKStateBit.TurnOffNow_InManual);

			GKDriversHelper.AddIntProprety(driver, 0, "Задержка на тревогу, с", 3, 0, 65535);
			GKDriversHelper.AddIntProprety(driver, 1, "Время удержания, с", 30, 0, 65535);

			var property1 = new GKDriverProperty()
			{
				No = 2,
				Name = "Конфигурация",
				Caption = "Конфигурация",
				Default = 0,
				IsLowByte = true,
				Mask = 0x07
			};

			GKDriversHelper.AddPropertyParameter(property1, "Дверь 1 (вход - КВ, выход - кнопка)", 0);
			GKDriversHelper.AddPropertyParameter(property1, "Дверь 2 (вход - КВ, выход - КВ)", 1);
			GKDriversHelper.AddPropertyParameter(property1, "Турникет", 2);
			GKDriversHelper.AddPropertyParameter(property1, "Шлагбаум 1 (вход - КВ, выход - кнопка, без обнаружения)", 3);
			GKDriversHelper.AddPropertyParameter(property1, "Шлагбаум 2 (вход - КВ, выход - КВ, без обнаружения)", 4);
			GKDriversHelper.AddPropertyParameter(property1, "Шлагбаум 3 (вход - КВ, выход - кнопка, с обнаружением)", 5);
			GKDriversHelper.AddPropertyParameter(property1, "Шлагбаум 4 (вход - КВ, выход - КВ, с обнаружением)", 6);
			driver.Properties.Add(property1);

			GKDriversHelper.AddIntProprety(driver, 3, KDDeviceType.Lock.ToString(), 0, 0, 49, isLowByte: true, cannotEdit: true);
			GKDriversHelper.AddIntProprety(driver, 4, KDDeviceType.Enter.ToString(), 0, 0, 49, isLowByte: true, cannotEdit: true);
			GKDriversHelper.AddIntProprety(driver, 5, KDDeviceType.Exit.ToString(), 0, 0, 49, isLowByte: true, cannotEdit: true);
			GKDriversHelper.AddIntProprety(driver, 6, KDDeviceType.Sensor.ToString(), 0, 0, 49, isLowByte: true, cannotEdit: true);
			var enterZonePoperty = GKDriversHelper.AddIntProprety(driver, 3, "Зона на вход(ЗПП)", 0, 0, 15);
			enterZonePoperty.IsHieghByte = true;
			enterZonePoperty.Mask = 15;
			var exitZonePoperty = GKDriversHelper.AddIntProprety(driver, 3, "Зона на выход(ЗПП)", 0, 0, 240);
			exitZonePoperty.Multiplier = 16;
			exitZonePoperty.IsHieghByte = true;
			exitZonePoperty.Mask = 240;
			
			GKDriversHelper.AddIntProprety(driver, 4, KDDeviceType.Button1.ToString(), 0, 0, 49, isHighByte: true, cannotEdit: true);
			GKDriversHelper.AddIntProprety(driver, 5, KDDeviceType.Button2.ToString(), 0, 0, 49, isHighByte: true, cannotEdit: true);
			GKDriversHelper.AddIntProprety(driver, 6, KDDeviceType.Button3.ToString(), 0, 0, 49, isHighByte: true, cannotEdit: true);
			GKDriversHelper.AddIntProprety(driver, 2, KDDeviceType.Button4.ToString(), 0, 0, 49, isHighByte: true, cannotEdit: true);
			return driver;
		}
	}
}
