﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Modbus;
using Modbus.Device;
using System.Net.Sockets;
using System.Threading;
using System.Diagnostics;
using System.IO.Ports;
using System.Reflection;
using System.Collections;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Runtime.CompilerServices;
using System.Runtime.InteropServices;

namespace GlobalCF
{
    public class ModBusTCPServerCS : INotifyPropertyChanged
    {
        /// <summary>
        /// Номер функции для чтения (0 - Coils, 1 - Discreet Inputs, 2 - Holding, 3 - Inputs)
        /// </summary>
        public int FunctionID = 0;
        public ushort StartAddress = 0;
        public ushort NumberOfRegisters = 1;
        public int ThreadSleepValue = 100;
        public bool SendCommandFlag = false;
        public ushort SendValue = 0;
        public ushort SendAddress = 0;

        /// <summary>
        /// IP-адрес сервера
        /// </summary>
        public string IPaddress;
        /// <summary>
        /// Номер порта
        /// </summary>
        public int PortNumber = 502;
        /// <summary>
        /// Клиент TCP/IP
        /// </summary>
        public TcpClient Client;
        /// <summary>
        /// Подключение к устройству
        /// </summary>
        public ModbusIpMaster IPMaster;
        /// <summary>
        /// Количество пропусков попыток связи
        /// </summary>
        public int RetriesCounter = 0;
        public Thread reInitThread;
        public int VeracitiesCounter = 0;
        public bool WorkerFlag = false;
        public ulong CyclesCounter = 0;
        public ulong PrevCycleCounter = 0;

        public int ClientNullCounter = 0;
        public int ClientNotConnectedCounter = 0;
        public int NoAnswerCounter = 0;

        public byte Address = 1;
        string _ErrorString = "";
        public string ErrorString
        {
            set
            {
                if (this._ErrorString != value)
                {
                    this._ErrorString = value;
                    this.OnPropertyChanged();
                }
            }
            get
            {
                return this._ErrorString;
            }
        }
        public MDBValue ServiceReg0 = new MDBValue();
        public MDBValue ServiceReg1 = new MDBValue();

        public bool ServerChangigFlag = false;
        public double WorkTime = 0;

        /// <summary>
        /// Счетчик тактов когда сервис вообще не ведет обмен. По превышению его надо рестартовать.
        /// </summary>
        public int HaltCounter = 0;
        /// <summary>
        /// Запретить изменение счетчика циклов
        /// </summary>
        public bool DebugFlag = false;

        public ObservableCollection<MDBValue> Values = new ObservableCollection<MDBValue>();

        public event PropertyChangedEventHandler PropertyChanged = delegate { };
        public void OnPropertyChanged([CallerMemberName] string propertyName = null)
        {
            this.PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
        }
        void InitClient()
        {
            try
            {
                this.Client = new TcpClient(this.IPaddress, this.PortNumber);
                this.IPMaster = ModbusIpMaster.CreateIp(this.Client);
                this.Client.ReceiveTimeout = 2000;
                this.Client.SendTimeout = 2000;
            }
            catch(Exception ex) { this.ErrorString = ex.ToString(); }
        }
        public void TCPworker(object tsk)
        {
            bool[] BoolVals;
            ushort[] UshortVals;

            long Freq = Stopwatch.Frequency;
            double Cycle = 1.0D / (double)Freq;
            Stopwatch sw = new Stopwatch();
            Stopwatch sw1 = new Stopwatch();
            double WorkTime = 0.0D;

            while (this.WorkerFlag)
            {
                sw.Reset();
                sw.Start();
                try
                {
                    if (this.Client == null)
                    {
                        if (!this.ServerChangigFlag)
                        {
                            this.ClientNullCounter++;
                            this.InitClient();
                        }
                    }
                    else
                    {
                        if (!this.Client.Connected)
                        {
                            if (!this.ServerChangigFlag)
                            {
                                this.ClientNotConnectedCounter++;
                                this.InitClient();
                            }
                        }
                        else
                        {

                            bool ClientReady = true;
                            #region Опрос ReadCoils
                            if (this.FunctionID == 0)
                            {
                                try
                                {
                                    if (ClientReady)
                                    {
                                        //foreach (ModBusReadCoilsCS cMBTag in this.ReadCoilsList)
                                        {
                                            try
                                            {
                                                #region Проверка связи и соединения
                                                if (this.Client != null)
                                                {
                                                    if (!this.Client.Connected)
                                                    {
                                                        ClientReady = false;
                                                        continue;
                                                    }
                                                }
                                                else
                                                {
                                                    ClientReady = false;
                                                    continue;
                                                }
                                                #endregion
                                                sw1.Restart();
                                                BoolVals = this.IPMaster.ReadCoils(this.Address, this.StartAddress, this.NumberOfRegisters);
                                                sw1.Stop();
                                                WorkTime = Math.Round(1000000.0D * (double)(Cycle * sw1.ElapsedTicks), 3); // микросекунды
                                                /*if (WorkTime < cMBTag.MinRequestTime)
                                                    cMBTag.MinRequestTime = WorkTime;
                                                if (WorkTime > cMBTag.MaxRequestTime)
                                                    cMBTag.MaxRequestTime = WorkTime;
                                                cMBTag.TotalRequestCounter++;
                                                cMBTag.TotalRequestTime += WorkTime;
                                                cMBTag.AvrRequestTime = (double)(cMBTag.TotalRequestTime / cMBTag.TotalRequestCounter);*/
                                                for (int i = 0; i < BoolVals.Length; i++)
                                                {
                                                    this.Values[i].Value = BoolVals[i] ? (ushort)1 : (ushort)0;
                                                }
                                            }
                                            catch (Exception ex)
                                            {
                                                this.ErrorString = DateTime.Now.ToString() + "." + DateTime.Now.Millisecond.ToString() + ": " + "Ошибка запроса: " + ex.ToString();
                                                this.VeracitiesCounter++;
                                            }
                                        }
                                    }
                                }
                                catch { }
                            }
                            #endregion
                            #region Опрос ReadHoldingRegisters
                            if (this.FunctionID == 2)
                            {
                                try
                                {
                                    if (ClientReady)
                                    {
                                        //foreach (ModBusReadHoldingRegistersCS cMBTag in this.ReadHoldingRegistersList)
                                        {
                                            try
                                            {
                                                #region Проверка связи и соединения
                                                if (this.Client != null)
                                                {
                                                    if (!this.Client.Connected)
                                                    {
                                                        ClientReady = false;
                                                        continue;
                                                    }
                                                }
                                                else
                                                {
                                                    ClientReady = false;
                                                    continue;
                                                }
                                                #endregion
                                                sw1.Restart();
                                                UshortVals = this.IPMaster.ReadHoldingRegisters(this.Address, this.StartAddress, this.NumberOfRegisters);
                                                sw1.Stop();
                                                WorkTime = Math.Round(1000000.0D * (double)(Cycle * sw1.ElapsedTicks), 3); // микросекунды
                                                /*if (WorkTime < cMBTag.MinRequestTime)
                                                    cMBTag.MinRequestTime = WorkTime;
                                                if (WorkTime > cMBTag.MaxRequestTime)
                                                    cMBTag.MaxRequestTime = WorkTime;
                                                cMBTag.TotalRequestCounter++;
                                                cMBTag.TotalRequestTime += WorkTime;
                                                cMBTag.AvrRequestTime = (double)(cMBTag.TotalRequestTime / cMBTag.TotalRequestCounter);*/
                                                for(int i=0; i<UshortVals.Length; i++)
                                                {
                                                    this.Values[i].Value = UshortVals[i];
                                                }
                                            }
                                            catch (Exception ex)
                                            {
                                                this.ErrorString = DateTime.Now.ToString() + "." + DateTime.Now.Millisecond.ToString() + ": " + "Ошибка запроса: " + ex.ToString();
                                                this.VeracitiesCounter++;
                                            }
                                        }
                                    }
                                }
                                catch { }
                            }
                            #endregion
                            #region Опрос ReadInputRegisters
                            if (this.FunctionID == 3)
                            {
                                try
                                {
                                    if (ClientReady)
                                    {
                                        //foreach (ModBusReadInputRegistersCS cMBTag in this.ReadInputRegistersList)
                                        {
                                            try
                                            {
                                                #region Проверка связи и соединения
                                                if (this.Client != null)
                                                {
                                                    if (!this.Client.Connected)
                                                    {
                                                        ClientReady = false;
                                                        continue;
                                                    }
                                                }
                                                else
                                                {
                                                    ClientReady = false;
                                                    continue;
                                                }
                                                #endregion
                                                sw1.Restart();
                                                UshortVals = this.IPMaster.ReadInputRegisters(this.Address, this.StartAddress, this.NumberOfRegisters);
                                                sw1.Stop();
                                                WorkTime = Math.Round(1000000.0D * (double)(Cycle * sw1.ElapsedTicks), 3); // микросекунды
                                                /*if (WorkTime < cMBTag.MinRequestTime)
                                                    cMBTag.MinRequestTime = WorkTime;
                                                if (WorkTime > cMBTag.MaxRequestTime)
                                                    cMBTag.MaxRequestTime = WorkTime;
                                                cMBTag.TotalRequestCounter++;
                                                cMBTag.TotalRequestTime += WorkTime;
                                                cMBTag.AvrRequestTime = (double)(cMBTag.TotalRequestTime / cMBTag.TotalRequestCounter);*/
                                                for (int i = 0; i < UshortVals.Length; i++)
                                                {
                                                    this.Values[i].Value = UshortVals[i];
                                                }
                                            }
                                            catch (Exception ex)
                                            {
                                                this.ErrorString = DateTime.Now.ToString() + "." + DateTime.Now.Millisecond.ToString() + ": " + "Ошибка запроса: " + ex.ToString();
                                                this.VeracitiesCounter++;
                                            }
                                        }
                                    }
                                }
                                catch { }
                            }
                            #endregion;
                            #region Опрос ReadInputs
                            if (this.FunctionID == 1)
                            {
                                try
                                {
                                    if (ClientReady)
                                    {
                                        //foreach (ModBusReadInputsCS cMBTag in this.ReadInputsList)
                                        {
                                            try
                                            {
                                                #region Проверка связи и соединения
                                                if (this.Client != null)
                                                {
                                                    if (!this.Client.Connected)
                                                    {
                                                        ClientReady = false;
                                                        continue;
                                                    }
                                                }
                                                else
                                                {
                                                    ClientReady = false;
                                                    continue;
                                                }
                                                #endregion
                                                sw1.Restart();
                                                BoolVals = this.IPMaster.ReadInputs(this.Address, this.StartAddress, this.NumberOfRegisters);
                                                sw1.Stop();
                                                WorkTime = Math.Round(1000000.0D * (double)(Cycle * sw1.ElapsedTicks), 3); // микросекунды
                                                /*if (WorkTime < cMBTag.MinRequestTime)
                                                    cMBTag.MinRequestTime = WorkTime;
                                                if (WorkTime > cMBTag.MaxRequestTime)
                                                    cMBTag.MaxRequestTime = WorkTime;
                                                cMBTag.TotalRequestCounter++;
                                                cMBTag.TotalRequestTime += WorkTime;
                                                cMBTag.AvrRequestTime = (double)(cMBTag.TotalRequestTime / cMBTag.TotalRequestCounter);*/
                                                for (int i = 0; i < BoolVals.Length; i++)
                                                {
                                                    this.Values[i].Value = BoolVals[i] ? (ushort)1 : (ushort)0;
                                                }
                                            }
                                            catch (Exception ex)
                                            {
                                                this.ErrorString = DateTime.Now.ToString() + "." + DateTime.Now.Millisecond.ToString() + ": " + "Ошибка запроса: " + ex.ToString();
                                                this.VeracitiesCounter++;
                                            }
                                        }
                                    }
                                }
                                catch { }
                            }
                            #endregion
                            #region Передача WriteCoilList
                            /*
                            lock (this.WriteCoilList)
                            {
                                try
                                {
                                    if (ClientReady)
                                    {
                                        foreach (ModBusWriteCoilCS cMBTag in this.WriteCoilList)
                                        {
                                            if (cMBTag.Status)
                                                continue;
                                            try
                                            {
                                                if (cMBTag.ChangedFlag)
                                                {
                                                    cMBTag.InAction = true;
                                                    #region Проверка связи и соединения
                                                    if (this.Client != null)
                                                    {
                                                        if (!this.Client.Connected)
                                                        {
                                                            //недостоверность
                                                            cMBTag.Veracity = cMBTag.Veracity | 0x1;
                                                            ClientReady = false;
                                                            continue;
                                                        }
                                                    }
                                                    else
                                                    {
                                                        //недостоверность
                                                        cMBTag.Veracity = cMBTag.Veracity | 0x1;
                                                        ClientReady = false;
                                                        continue;
                                                    }
                                                    #endregion
                                                    sw1.Restart();
                                                    if (cMBTag.TimeoutBeforeRequest != 0)
                                                        Thread.Sleep(cMBTag.TimeoutBeforeRequest);
                                                    this.IPMaster.WriteSingleCoil(cMBTag.Address, cMBTag.BeginAddress, Convert.ToBoolean(((IAttributes)cMBTag).Get_AttributeValue("DO_OUT0")));
                                                    sw1.Stop();
                                                    WorkTime = Math.Round(1000000.0D * (double)(Cycle * sw1.ElapsedTicks), 3); // микросекунды
                                                    if (WorkTime < cMBTag.MinRequestTime)
                                                        cMBTag.MinRequestTime = WorkTime;
                                                    if (WorkTime > cMBTag.MaxRequestTime)
                                                        cMBTag.MaxRequestTime = WorkTime;
                                                    cMBTag.TotalRequestCounter++;
                                                    cMBTag.TotalRequestTime += WorkTime;
                                                    cMBTag.AvrRequestTime = (double)(cMBTag.TotalRequestTime / cMBTag.TotalRequestCounter);

                                                    cMBTag.Veracity = cMBTag.Veracity & 0xFE;
                                                    cMBTag.ChangedFlag = false;
                                                    cMBTag.InAction = false;
                                                }
                                            }
                                            catch (Exception ex)
                                            {
                                                cMBTag.InAction = false;
                                                //недостоверность
                                                cMBTag.Veracity = cMBTag.Veracity | 0x1;

                                                this.ChangeServer();
                                                this.ErrorString = DateTime.Now.ToString() + "." + DateTime.Now.Millisecond.ToString() + ": " + "Ошибка запроса: " + ex.ToString();
                                                cMBTag.Veracity = 1;
                                                this.VeracitiesCounter++;
                                            }
                                        }
                                    }
                                }
                                catch { }
                            } */
                            #endregion
                            #region Передача WriteRegister
                            
                            if (this.SendCommandFlag)
                            {
                                try
                                {
                                    if (ClientReady)
                                    {
                                        try
                                        {
                                            //if (cMBTag.ChangedFlag)
                                            {
                                                #region Проверка связи и соединения
                                                if (this.Client != null)
                                                {
                                                    if (!this.Client.Connected)
                                                    {
                                                        ClientReady = false;
                                                        continue;
                                                    }
                                                }
                                                else
                                                {
                                                    ClientReady = false;
                                                    continue;
                                                }
                                                #endregion
                                                sw1.Restart();
                                                this.IPMaster.WriteSingleRegister(this.Address, this.SendAddress, this.SendValue);
                                                sw1.Stop();
                                                WorkTime = Math.Round(1000000.0D * (double)(Cycle * sw1.ElapsedTicks), 3); // микросекунды
                                                /*if (WorkTime < cMBTag.MinRequestTime)
                                                    cMBTag.MinRequestTime = WorkTime;
                                                if (WorkTime > cMBTag.MaxRequestTime)
                                                    cMBTag.MaxRequestTime = WorkTime;
                                                cMBTag.TotalRequestCounter++;
                                                cMBTag.TotalRequestTime += WorkTime;
                                                cMBTag.AvrRequestTime = (double)(cMBTag.TotalRequestTime / cMBTag.TotalRequestCounter);*/

                                            }
                                        }
                                        catch (Exception ex)
                                        {
                                            this.ErrorString = DateTime.Now.ToString() + "." + DateTime.Now.Millisecond.ToString() + ": " + "Ошибка запроса: " + ex.ToString();
                                            this.VeracitiesCounter++;
                                        }

                                    }
                                }
                                catch { }
                            }
                            #endregion
                            #region Передача WriteMultipleCoils
                            /*
                            lock (this.WriteMultipleCoilsList)
                            {
                                try
                                {
                                    if (ClientReady)
                                    {
                                        foreach (ModBusWriteMultipleCoilsCS cMBTag in this.WriteMultipleCoilsList)
                                        {
                                            if (cMBTag.Status)
                                                continue;
                                            try
                                            {
                                                if (cMBTag.ChangedFlag)
                                                {
                                                    cMBTag.InAction = true;
                                                    object vals = ((IAttributes)cMBTag).Get_AttributeValue("DO_OUT0");
                                                    bool[] bool_vals = new bool[8];
                                                    #region Значение в массив бит
                                                    if (vals.GetType() == typeof(sbyte))
                                                    {
                                                        sbyte sbyte_v = Convert.ToSByte(vals);
                                                        bool_vals = new bool[8];
                                                        bool_vals[0] = (sbyte_v & 0x1) == 0x1;
                                                        bool_vals[1] = (sbyte_v & 0x2) == 0x2;
                                                        bool_vals[2] = (sbyte_v & 0x4) == 0x4;
                                                        bool_vals[3] = (sbyte_v & 0x8) == 0x8;
                                                        bool_vals[4] = (sbyte_v & 0x10) == 0x10;
                                                        bool_vals[5] = (sbyte_v & 0x20) == 0x20;
                                                        bool_vals[6] = (sbyte_v & 0x40) == 0x40;
                                                        bool_vals[7] = (sbyte_v & 0x80) == 0x80;
                                                    }
                                                    if (vals.GetType() == typeof(byte))
                                                    {
                                                        byte byte_v = Convert.ToByte(vals);
                                                        bool_vals = new bool[8];
                                                        bool_vals[0] = (byte_v & 0x1) == 0x1;
                                                        bool_vals[1] = (byte_v & 0x2) == 0x2;
                                                        bool_vals[2] = (byte_v & 0x4) == 0x4;
                                                        bool_vals[3] = (byte_v & 0x8) == 0x8;
                                                        bool_vals[4] = (byte_v & 0x10) == 0x10;
                                                        bool_vals[5] = (byte_v & 0x20) == 0x20;
                                                        bool_vals[6] = (byte_v & 0x40) == 0x40;
                                                        bool_vals[7] = (byte_v & 0x80) == 0x80;
                                                    }
                                                    if (vals.GetType() == typeof(short))
                                                    {
                                                        short short_v = Convert.ToInt16(vals);
                                                        bool_vals = new bool[16];
                                                        bool_vals[0] = (short_v & 0x1) == 0x1;
                                                        bool_vals[1] = (short_v & 0x2) == 0x2;
                                                        bool_vals[2] = (short_v & 0x4) == 0x4;
                                                        bool_vals[3] = (short_v & 0x8) == 0x8;
                                                        bool_vals[4] = (short_v & 0x10) == 0x10;
                                                        bool_vals[5] = (short_v & 0x20) == 0x20;
                                                        bool_vals[6] = (short_v & 0x40) == 0x40;
                                                        bool_vals[7] = (short_v & 0x80) == 0x80;
                                                        bool_vals[8] = (short_v & 0x100) == 0x100;
                                                        bool_vals[9] = (short_v & 0x200) == 0x200;
                                                        bool_vals[10] = (short_v & 0x400) == 0x400;
                                                        bool_vals[11] = (short_v & 0x800) == 0x800;
                                                        bool_vals[12] = (short_v & 0x1000) == 0x1000;
                                                        bool_vals[13] = (short_v & 0x2000) == 0x2000;
                                                        bool_vals[14] = (short_v & 0x4000) == 0x4000;
                                                        bool_vals[15] = (short_v & 0x8000) == 0x8000;
                                                    }
                                                    if (vals.GetType() == typeof(ushort))
                                                    {
                                                        ushort ushort_v = Convert.ToUInt16(vals);
                                                        bool_vals = new bool[16];
                                                        bool_vals[0] = (ushort_v & 0x1) == 0x1;
                                                        bool_vals[1] = (ushort_v & 0x2) == 0x2;
                                                        bool_vals[2] = (ushort_v & 0x4) == 0x4;
                                                        bool_vals[3] = (ushort_v & 0x8) == 0x8;
                                                        bool_vals[4] = (ushort_v & 0x10) == 0x10;
                                                        bool_vals[5] = (ushort_v & 0x20) == 0x20;
                                                        bool_vals[6] = (ushort_v & 0x40) == 0x40;
                                                        bool_vals[7] = (ushort_v & 0x80) == 0x80;
                                                        bool_vals[8] = (ushort_v & 0x100) == 0x100;
                                                        bool_vals[9] = (ushort_v & 0x200) == 0x200;
                                                        bool_vals[10] = (ushort_v & 0x400) == 0x400;
                                                        bool_vals[11] = (ushort_v & 0x800) == 0x800;
                                                        bool_vals[12] = (ushort_v & 0x1000) == 0x1000;
                                                        bool_vals[13] = (ushort_v & 0x2000) == 0x2000;
                                                        bool_vals[14] = (ushort_v & 0x4000) == 0x4000;
                                                        bool_vals[15] = (ushort_v & 0x8000) == 0x8000;
                                                    }
                                                    if (vals.GetType() == typeof(int))
                                                    {
                                                        int int_v = Convert.ToInt32(vals);
                                                        bool_vals = new bool[32];
                                                        bool_vals[0] = (int_v & 0x1) == 0x1;
                                                        bool_vals[1] = (int_v & 0x2) == 0x2;
                                                        bool_vals[2] = (int_v & 0x4) == 0x4;
                                                        bool_vals[3] = (int_v & 0x8) == 0x8;
                                                        bool_vals[4] = (int_v & 0x10) == 0x10;
                                                        bool_vals[5] = (int_v & 0x20) == 0x20;
                                                        bool_vals[6] = (int_v & 0x40) == 0x40;
                                                        bool_vals[7] = (int_v & 0x80) == 0x80;
                                                        bool_vals[8] = (int_v & 0x100) == 0x100;
                                                        bool_vals[9] = (int_v & 0x200) == 0x200;
                                                        bool_vals[10] = (int_v & 0x400) == 0x400;
                                                        bool_vals[11] = (int_v & 0x800) == 0x800;
                                                        bool_vals[12] = (int_v & 0x1000) == 0x1000;
                                                        bool_vals[13] = (int_v & 0x2000) == 0x2000;
                                                        bool_vals[14] = (int_v & 0x4000) == 0x4000;
                                                        bool_vals[15] = (int_v & 0x8000) == 0x8000;
                                                        bool_vals[16] = (int_v & 0x10000) == 0x10000;
                                                        bool_vals[17] = (int_v & 0x20000) == 0x20000;
                                                        bool_vals[18] = (int_v & 0x40000) == 0x40000;
                                                        bool_vals[19] = (int_v & 0x80000) == 0x80000;
                                                        bool_vals[20] = (int_v & 0x100000) == 0x100000;
                                                        bool_vals[21] = (int_v & 0x200000) == 0x200000;
                                                        bool_vals[22] = (int_v & 0x400000) == 0x400000;
                                                        bool_vals[23] = (int_v & 0x800000) == 0x800000;
                                                        bool_vals[24] = (int_v & 0x1000000) == 0x1000000;
                                                        bool_vals[25] = (int_v & 0x2000000) == 0x2000000;
                                                        bool_vals[26] = (int_v & 0x4000000) == 0x4000000;
                                                        bool_vals[27] = (int_v & 0x8000000) == 0x8000000;
                                                        bool_vals[28] = (int_v & 0x10000000) == 0x10000000;
                                                        bool_vals[29] = (int_v & 0x20000000) == 0x20000000;
                                                        bool_vals[30] = (int_v & 0x40000000) == 0x40000000;
                                                        bool_vals[31] = (int_v & 0x80000000) == 0x80000000;
                                                    }
                                                    if (vals.GetType() == typeof(uint))
                                                    {
                                                        uint uint_v = Convert.ToUInt32(vals);
                                                        bool_vals = new bool[32];
                                                        bool_vals[0] = (uint_v & 0x1) == 0x1;
                                                        bool_vals[1] = (uint_v & 0x2) == 0x2;
                                                        bool_vals[2] = (uint_v & 0x4) == 0x4;
                                                        bool_vals[3] = (uint_v & 0x8) == 0x8;
                                                        bool_vals[4] = (uint_v & 0x10) == 0x10;
                                                        bool_vals[5] = (uint_v & 0x20) == 0x20;
                                                        bool_vals[6] = (uint_v & 0x40) == 0x40;
                                                        bool_vals[7] = (uint_v & 0x80) == 0x80;
                                                        bool_vals[8] = (uint_v & 0x100) == 0x100;
                                                        bool_vals[9] = (uint_v & 0x200) == 0x200;
                                                        bool_vals[10] = (uint_v & 0x400) == 0x400;
                                                        bool_vals[11] = (uint_v & 0x800) == 0x800;
                                                        bool_vals[12] = (uint_v & 0x1000) == 0x1000;
                                                        bool_vals[13] = (uint_v & 0x2000) == 0x2000;
                                                        bool_vals[14] = (uint_v & 0x4000) == 0x4000;
                                                        bool_vals[15] = (uint_v & 0x8000) == 0x8000;
                                                        bool_vals[16] = (uint_v & 0x10000) == 0x10000;
                                                        bool_vals[17] = (uint_v & 0x20000) == 0x20000;
                                                        bool_vals[18] = (uint_v & 0x40000) == 0x40000;
                                                        bool_vals[19] = (uint_v & 0x80000) == 0x80000;
                                                        bool_vals[20] = (uint_v & 0x100000) == 0x100000;
                                                        bool_vals[21] = (uint_v & 0x200000) == 0x200000;
                                                        bool_vals[22] = (uint_v & 0x400000) == 0x400000;
                                                        bool_vals[23] = (uint_v & 0x800000) == 0x800000;
                                                        bool_vals[24] = (uint_v & 0x1000000) == 0x1000000;
                                                        bool_vals[25] = (uint_v & 0x2000000) == 0x2000000;
                                                        bool_vals[26] = (uint_v & 0x4000000) == 0x4000000;
                                                        bool_vals[27] = (uint_v & 0x8000000) == 0x8000000;
                                                        bool_vals[28] = (uint_v & 0x10000000) == 0x10000000;
                                                        bool_vals[29] = (uint_v & 0x20000000) == 0x20000000;
                                                        bool_vals[30] = (uint_v & 0x40000000) == 0x40000000;
                                                        bool_vals[31] = (uint_v & 0x80000000) == 0x80000000;
                                                    }
                                                    if (vals.GetType() == typeof(long))
                                                    {
                                                        long long_v = Convert.ToInt64(vals);
                                                        bool_vals = new bool[64];
                                                        bool_vals[0] = (long_v & 0x1) == 0x1;
                                                        bool_vals[1] = (long_v & 0x2) == 0x2;
                                                        bool_vals[2] = (long_v & 0x4) == 0x4;
                                                        bool_vals[3] = (long_v & 0x8) == 0x8;
                                                        bool_vals[4] = (long_v & 0x10) == 0x10;
                                                        bool_vals[5] = (long_v & 0x20) == 0x20;
                                                        bool_vals[6] = (long_v & 0x40) == 0x40;
                                                        bool_vals[7] = (long_v & 0x80) == 0x80;
                                                        bool_vals[8] = (long_v & 0x100) == 0x100;
                                                        bool_vals[9] = (long_v & 0x200) == 0x200;
                                                        bool_vals[10] = (long_v & 0x400) == 0x400;
                                                        bool_vals[11] = (long_v & 0x800) == 0x800;
                                                        bool_vals[12] = (long_v & 0x1000) == 0x1000;
                                                        bool_vals[13] = (long_v & 0x2000) == 0x2000;
                                                        bool_vals[14] = (long_v & 0x4000) == 0x4000;
                                                        bool_vals[15] = (long_v & 0x8000) == 0x8000;
                                                        bool_vals[16] = (long_v & 0x10000) == 0x10000;
                                                        bool_vals[17] = (long_v & 0x20000) == 0x20000;
                                                        bool_vals[18] = (long_v & 0x40000) == 0x40000;
                                                        bool_vals[19] = (long_v & 0x80000) == 0x80000;
                                                        bool_vals[20] = (long_v & 0x100000) == 0x100000;
                                                        bool_vals[21] = (long_v & 0x200000) == 0x200000;
                                                        bool_vals[22] = (long_v & 0x400000) == 0x400000;
                                                        bool_vals[23] = (long_v & 0x800000) == 0x800000;
                                                        bool_vals[24] = (long_v & 0x1000000) == 0x1000000;
                                                        bool_vals[25] = (long_v & 0x2000000) == 0x2000000;
                                                        bool_vals[26] = (long_v & 0x4000000) == 0x4000000;
                                                        bool_vals[27] = (long_v & 0x8000000) == 0x8000000;
                                                        bool_vals[28] = (long_v & 0x10000000) == 0x10000000;
                                                        bool_vals[29] = (long_v & 0x20000000) == 0x20000000;
                                                        bool_vals[30] = (long_v & 0x40000000) == 0x40000000;
                                                        bool_vals[31] = (long_v & 0x80000000) == 0x80000000;
                                                        bool_vals[32] = (long_v & 0x100000000) == 0x100000000;
                                                        bool_vals[33] = (long_v & 0x200000000) == 0x200000000;
                                                        bool_vals[34] = (long_v & 0x400000000) == 0x400000000;
                                                        bool_vals[35] = (long_v & 0x800000000) == 0x800000000;
                                                        bool_vals[36] = (long_v & 0x1000000000) == 0x1000000000;
                                                        bool_vals[37] = (long_v & 0x2000000000) == 0x2000000000;
                                                        bool_vals[38] = (long_v & 0x4000000000) == 0x4000000000;
                                                        bool_vals[39] = (long_v & 0x8000000000) == 0x8000000000;
                                                        bool_vals[40] = (long_v & 0x10000000000) == 0x10000000000;
                                                        bool_vals[41] = (long_v & 0x20000000000) == 0x20000000000;
                                                        bool_vals[42] = (long_v & 0x40000000000) == 0x40000000000;
                                                        bool_vals[43] = (long_v & 0x80000000000) == 0x80000000000;
                                                        bool_vals[44] = (long_v & 0x100000000000) == 0x100000000000;
                                                        bool_vals[45] = (long_v & 0x200000000000) == 0x200000000000;
                                                        bool_vals[46] = (long_v & 0x400000000000) == 0x400000000000;
                                                        bool_vals[47] = (long_v & 0x800000000000) == 0x800000000000;
                                                        bool_vals[48] = (long_v & 0x1000000000000) == 0x1000000000000;
                                                        bool_vals[49] = (long_v & 0x2000000000000) == 0x2000000000000;
                                                        bool_vals[50] = (long_v & 0x4000000000000) == 0x4000000000000;
                                                        bool_vals[51] = (long_v & 0x8000000000000) == 0x8000000000000;
                                                        bool_vals[52] = (long_v & 0x10000000000000) == 0x10000000000000;
                                                        bool_vals[53] = (long_v & 0x20000000000000) == 0x20000000000000;
                                                        bool_vals[54] = (long_v & 0x40000000000000) == 0x40000000000000;
                                                        bool_vals[55] = (long_v & 0x80000000000000) == 0x80000000000000;
                                                        bool_vals[56] = (long_v & 0x100000000000000) == 0x100000000000000;
                                                        bool_vals[57] = (long_v & 0x200000000000000) == 0x200000000000000;
                                                        bool_vals[58] = (long_v & 0x400000000000000) == 0x400000000000000;
                                                        bool_vals[59] = (long_v & 0x800000000000000) == 0x800000000000000;
                                                        bool_vals[60] = (long_v & 0x1000000000000000) == 0x1000000000000000;
                                                        bool_vals[61] = (long_v & 0x2000000000000000) == 0x2000000000000000;
                                                        bool_vals[62] = (long_v & 0x4000000000000000) == 0x4000000000000000;
                                                        bool_vals[63] = ((ulong)long_v & 0x8000000000000000) == 0x8000000000000000;
                                                    }
                                                    if (vals.GetType() == typeof(ulong))
                                                    {
                                                        ulong ulong_v = Convert.ToUInt64(vals);
                                                        bool_vals = new bool[64];
                                                        bool_vals[0] = (ulong_v & 0x1) == 0x1;
                                                        bool_vals[1] = (ulong_v & 0x2) == 0x2;
                                                        bool_vals[2] = (ulong_v & 0x4) == 0x4;
                                                        bool_vals[3] = (ulong_v & 0x8) == 0x8;
                                                        bool_vals[4] = (ulong_v & 0x10) == 0x10;
                                                        bool_vals[5] = (ulong_v & 0x20) == 0x20;
                                                        bool_vals[6] = (ulong_v & 0x40) == 0x40;
                                                        bool_vals[7] = (ulong_v & 0x80) == 0x80;
                                                        bool_vals[8] = (ulong_v & 0x100) == 0x100;
                                                        bool_vals[9] = (ulong_v & 0x200) == 0x200;
                                                        bool_vals[10] = (ulong_v & 0x400) == 0x400;
                                                        bool_vals[11] = (ulong_v & 0x800) == 0x800;
                                                        bool_vals[12] = (ulong_v & 0x1000) == 0x1000;
                                                        bool_vals[13] = (ulong_v & 0x2000) == 0x2000;
                                                        bool_vals[14] = (ulong_v & 0x4000) == 0x4000;
                                                        bool_vals[15] = (ulong_v & 0x8000) == 0x8000;
                                                        bool_vals[16] = (ulong_v & 0x10000) == 0x10000;
                                                        bool_vals[17] = (ulong_v & 0x20000) == 0x20000;
                                                        bool_vals[18] = (ulong_v & 0x40000) == 0x40000;
                                                        bool_vals[19] = (ulong_v & 0x80000) == 0x80000;
                                                        bool_vals[20] = (ulong_v & 0x100000) == 0x100000;
                                                        bool_vals[21] = (ulong_v & 0x200000) == 0x200000;
                                                        bool_vals[22] = (ulong_v & 0x400000) == 0x400000;
                                                        bool_vals[23] = (ulong_v & 0x800000) == 0x800000;
                                                        bool_vals[24] = (ulong_v & 0x1000000) == 0x1000000;
                                                        bool_vals[25] = (ulong_v & 0x2000000) == 0x2000000;
                                                        bool_vals[26] = (ulong_v & 0x4000000) == 0x4000000;
                                                        bool_vals[27] = (ulong_v & 0x8000000) == 0x8000000;
                                                        bool_vals[28] = (ulong_v & 0x10000000) == 0x10000000;
                                                        bool_vals[29] = (ulong_v & 0x20000000) == 0x20000000;
                                                        bool_vals[30] = (ulong_v & 0x40000000) == 0x40000000;
                                                        bool_vals[31] = (ulong_v & 0x80000000) == 0x80000000;
                                                        bool_vals[32] = (ulong_v & 0x100000000) == 0x100000000;
                                                        bool_vals[33] = (ulong_v & 0x200000000) == 0x200000000;
                                                        bool_vals[34] = (ulong_v & 0x400000000) == 0x400000000;
                                                        bool_vals[35] = (ulong_v & 0x800000000) == 0x800000000;
                                                        bool_vals[36] = (ulong_v & 0x1000000000) == 0x1000000000;
                                                        bool_vals[37] = (ulong_v & 0x2000000000) == 0x2000000000;
                                                        bool_vals[38] = (ulong_v & 0x4000000000) == 0x4000000000;
                                                        bool_vals[39] = (ulong_v & 0x8000000000) == 0x8000000000;
                                                        bool_vals[40] = (ulong_v & 0x10000000000) == 0x10000000000;
                                                        bool_vals[41] = (ulong_v & 0x20000000000) == 0x20000000000;
                                                        bool_vals[42] = (ulong_v & 0x40000000000) == 0x40000000000;
                                                        bool_vals[43] = (ulong_v & 0x80000000000) == 0x80000000000;
                                                        bool_vals[44] = (ulong_v & 0x100000000000) == 0x100000000000;
                                                        bool_vals[45] = (ulong_v & 0x200000000000) == 0x200000000000;
                                                        bool_vals[46] = (ulong_v & 0x400000000000) == 0x400000000000;
                                                        bool_vals[47] = (ulong_v & 0x800000000000) == 0x800000000000;
                                                        bool_vals[48] = (ulong_v & 0x1000000000000) == 0x1000000000000;
                                                        bool_vals[49] = (ulong_v & 0x2000000000000) == 0x2000000000000;
                                                        bool_vals[50] = (ulong_v & 0x4000000000000) == 0x4000000000000;
                                                        bool_vals[51] = (ulong_v & 0x8000000000000) == 0x8000000000000;
                                                        bool_vals[52] = (ulong_v & 0x10000000000000) == 0x10000000000000;
                                                        bool_vals[53] = (ulong_v & 0x20000000000000) == 0x20000000000000;
                                                        bool_vals[54] = (ulong_v & 0x40000000000000) == 0x40000000000000;
                                                        bool_vals[55] = (ulong_v & 0x80000000000000) == 0x80000000000000;
                                                        bool_vals[56] = (ulong_v & 0x100000000000000) == 0x100000000000000;
                                                        bool_vals[57] = (ulong_v & 0x200000000000000) == 0x200000000000000;
                                                        bool_vals[58] = (ulong_v & 0x400000000000000) == 0x400000000000000;
                                                        bool_vals[59] = (ulong_v & 0x800000000000000) == 0x800000000000000;
                                                        bool_vals[60] = (ulong_v & 0x1000000000000000) == 0x1000000000000000;
                                                        bool_vals[61] = (ulong_v & 0x2000000000000000) == 0x2000000000000000;
                                                        bool_vals[62] = (ulong_v & 0x4000000000000000) == 0x4000000000000000;
                                                        bool_vals[63] = (ulong_v & 0x8000000000000000) == 0x8000000000000000;
                                                    }
                                                    if (vals.GetType() == typeof(ArrayList))
                                                    {
                                                        ArrayList vls = (ArrayList)vals;
                                                        break;
                                                    }
                                                    #endregion
                                                    bool[] wbools = new bool[cMBTag.NumberOfRegisters];
                                                    if (cMBTag.NumberOfRegisters <= bool_vals.Length)
                                                    {
                                                        for (int i = 0; i < cMBTag.NumberOfRegisters; i++)
                                                            wbools[i] = bool_vals[i];
                                                    }
                                                    if (cMBTag.NumberOfRegisters > bool_vals.Length)
                                                    {
                                                        for (int i = 0; i < bool_vals.Length; i++)
                                                            wbools[i] = bool_vals[i];
                                                    }
                                                    #region Проверка связи и соединения
                                                    if (this.Client != null)
                                                    {
                                                        if (!this.Client.Connected)
                                                        {
                                                            //недостоверность
                                                            cMBTag.Veracity = cMBTag.Veracity | 0x1;
                                                            ClientReady = false;
                                                            continue;
                                                        }
                                                    }
                                                    else
                                                    {
                                                        //недостоверность
                                                        cMBTag.Veracity = cMBTag.Veracity | 0x1;
                                                        ClientReady = false;
                                                        continue;
                                                    }
                                                    #endregion
                                                    sw1.Restart();
                                                    if (cMBTag.TimeoutBeforeRequest != 0)
                                                        Thread.Sleep(cMBTag.TimeoutBeforeRequest);
                                                    this.IPMaster.WriteMultipleCoils(cMBTag.Address, cMBTag.BeginAddress, wbools);
                                                    sw1.Stop();
                                                    WorkTime = Math.Round(1000000.0D * (double)(Cycle * sw1.ElapsedTicks), 3); // микросекунды
                                                    if (WorkTime < cMBTag.MinRequestTime)
                                                        cMBTag.MinRequestTime = WorkTime;
                                                    if (WorkTime > cMBTag.MaxRequestTime)
                                                        cMBTag.MaxRequestTime = WorkTime;
                                                    cMBTag.TotalRequestCounter++;
                                                    cMBTag.TotalRequestTime += WorkTime;
                                                    cMBTag.AvrRequestTime = (double)(cMBTag.TotalRequestTime / cMBTag.TotalRequestCounter);

                                                    cMBTag.Veracity = cMBTag.Veracity & 0xFE;
                                                    cMBTag.ChangedFlag = false;
                                                    cMBTag.InAction = false;
                                                }
                                            }
                                            catch (Exception ex)
                                            {
                                                cMBTag.InAction = false;
                                                //недостоверность
                                                cMBTag.Veracity = cMBTag.Veracity | 0x1;

                                                this.ChangeServer();
                                                this.ErrorString = DateTime.Now.ToString() + "." + DateTime.Now.Millisecond.ToString() + ": " + "Ошибка запроса: " + ex.ToString();
                                                cMBTag.Veracity = 1;
                                                this.VeracitiesCounter++;
                                            }
                                        }
                                    }
                                }
                                catch { }
                            }*/
                            #endregion
                            #region Передача WriteMultipleRegisters
                            /*
                            lock (this.WriteMultipleRegistersList)
                            {
                                try
                                {
                                    if (ClientReady)
                                    {
                                        foreach (ModBusWriteMultipleRegistersCS cMBTag in this.WriteMultipleRegistersList)
                                        {
                                            if (cMBTag.Status)
                                                continue;
                                            try
                                            {
                                                if (cMBTag.ChangedFlag)
                                                {
                                                    cMBTag.InAction = true;
                                                    cMBTag.Veracity = cMBTag.Veracity & 0xFE;
                                                    ushort[] ushort_vals = new ushort[1];
                                                    object AttrValue = ((IAttributes)cMBTag).Get_AttributeValue("DO_OUT0");
                                                    UDataConvert ud = new UDataConvert();
                                                    if (AttrValue.GetType() == typeof(float))
                                                    {
                                                        ushort_vals = new ushort[2];
                                                        ud.f1 = Convert.ToSingle(AttrValue);
                                                        if (cMBTag.BytesFlow == 0) // Штатный
                                                        {
                                                            ushort_vals[0] = ud.shrt1;
                                                            ushort_vals[1] = ud.shrt2;
                                                        }
                                                        if (cMBTag.BytesFlow == 1) // перевернутый №1 (HW:LW)
                                                        {
                                                            ushort_vals[0] = ud.shrt2;
                                                            ushort_vals[1] = ud.shrt1;
                                                        }
                                                        if (cMBTag.BytesFlow == 2) // перевернутый №2 (B4:B3:B2:B1)
                                                        {
                                                            UDataConvert fv = new UDataConvert();
                                                            fv.b1 = ud.b4;
                                                            fv.b2 = ud.b3;
                                                            fv.b3 = ud.b2;
                                                            fv.b4 = ud.b1;
                                                            ushort_vals[0] = fv.shrt1;
                                                            ushort_vals[1] = fv.shrt2;
                                                        }
                                                    }
                                                    if (AttrValue.GetType() == typeof(double))
                                                    {
                                                        ushort_vals = new ushort[4];
                                                        ud.d1 = Convert.ToDouble(AttrValue);
                                                        if (cMBTag.BytesFlow == 0) // Штатный
                                                        {
                                                            ushort_vals[0] = ud.shrt1;
                                                            ushort_vals[1] = ud.shrt2;
                                                            ushort_vals[2] = ud.shrt3;
                                                            ushort_vals[3] = ud.shrt4;
                                                        }
                                                        if (cMBTag.BytesFlow == 1) // перевернутый №1 (HW:LW)
                                                        {
                                                            ushort_vals[0] = ud.shrt3;
                                                            ushort_vals[1] = ud.shrt4;
                                                            ushort_vals[2] = ud.shrt1;
                                                            ushort_vals[3] = ud.shrt2;
                                                        }
                                                        if (cMBTag.BytesFlow == 2) // перевернутый №2 (B4:B3:B2:B1)
                                                        {
                                                            ushort_vals[0] = ud.shrt4;
                                                            ushort_vals[1] = ud.shrt3;
                                                            ushort_vals[2] = ud.shrt2;
                                                            ushort_vals[3] = ud.shrt1;
                                                        }
                                                    }
                                                    if (AttrValue.GetType() == typeof(sbyte))
                                                    {
                                                        ushort_vals = new ushort[1];
                                                        ud.b1 = Convert.ToByte(AttrValue);
                                                        ushort_vals[0] = ud.shrt1;
                                                    }
                                                    if (AttrValue.GetType() == typeof(byte))
                                                    {
                                                        ushort_vals = new ushort[1];
                                                        ud.b1 = Convert.ToByte(AttrValue);
                                                        ushort_vals[0] = ud.shrt1;
                                                    }
                                                    if (AttrValue.GetType() == typeof(short))
                                                    {
                                                        ushort_vals = new ushort[1];
                                                        ud.shrt1 = Convert.ToUInt16(AttrValue);
                                                        ushort_vals[0] = ud.shrt1;
                                                    }
                                                    if (AttrValue.GetType() == typeof(ushort))
                                                    {
                                                        ushort_vals = new ushort[1];
                                                        ud.shrt1 = Convert.ToUInt16(AttrValue);
                                                        ushort_vals[0] = ud.shrt1;
                                                    }
                                                    if (AttrValue.GetType() == typeof(int))
                                                    {
                                                        ushort_vals = new ushort[2];
                                                        ud.i1 = Convert.ToUInt32(AttrValue);
                                                        if (cMBTag.BytesFlow == 0) // Штатный
                                                        {
                                                            ushort_vals[0] = ud.shrt1;
                                                            ushort_vals[1] = ud.shrt2;
                                                        }
                                                        if (cMBTag.BytesFlow == 1) // перевернутый №1 (HW:LW)
                                                        {
                                                            ushort_vals[0] = ud.shrt2;
                                                            ushort_vals[1] = ud.shrt3;
                                                        }
                                                        if (cMBTag.BytesFlow == 2) // перевернутый №2 (B4:B3:B2:B1)
                                                        {
                                                            UDataConvert iv = new UDataConvert();
                                                            iv.b1 = ud.b4;
                                                            iv.b2 = ud.b3;
                                                            iv.b3 = ud.b2;
                                                            iv.b4 = ud.b1;
                                                            ushort_vals[0] = iv.shrt1;
                                                            ushort_vals[1] = iv.shrt2;
                                                        }
                                                    }
                                                    if (AttrValue.GetType() == typeof(uint))
                                                    {
                                                        ushort_vals = new ushort[2];
                                                        ud.i1 = Convert.ToUInt32(AttrValue);
                                                        if (cMBTag.BytesFlow == 0) // Штатный
                                                        {
                                                            ushort_vals[0] = ud.shrt1;
                                                            ushort_vals[1] = ud.shrt2;
                                                        }
                                                        if (cMBTag.BytesFlow == 1) // перевернутый №1 (HW:LW)
                                                        {
                                                            ushort_vals[0] = ud.shrt2;
                                                            ushort_vals[1] = ud.shrt1;
                                                        }
                                                        if (cMBTag.BytesFlow == 2) // перевернутый №2 (B4:B3:B2:B1)
                                                        {
                                                            UDataConvert uiv = new UDataConvert();
                                                            uiv.b1 = ud.b4;
                                                            uiv.b2 = ud.b3;
                                                            uiv.b3 = ud.b2;
                                                            uiv.b4 = ud.b1;
                                                            ushort_vals[0] = uiv.shrt1;
                                                            ushort_vals[1] = uiv.shrt2;
                                                        }
                                                    }
                                                    if (AttrValue.GetType() == typeof(long))
                                                    {
                                                        ushort_vals = new ushort[4];
                                                        ud.l1 = Convert.ToUInt64(AttrValue);
                                                        if (cMBTag.BytesFlow == 0) // Штатный
                                                        {
                                                            ushort_vals[0] = ud.shrt1;
                                                            ushort_vals[1] = ud.shrt2;
                                                            ushort_vals[2] = ud.shrt3;
                                                            ushort_vals[3] = ud.shrt4;
                                                        }
                                                        if (cMBTag.BytesFlow == 1) // перевернутый №1 (HW:LW)
                                                        {
                                                            ushort_vals[0] = ud.shrt3;
                                                            ushort_vals[1] = ud.shrt4;
                                                            ushort_vals[2] = ud.shrt1;
                                                            ushort_vals[3] = ud.shrt2;
                                                        }
                                                        if (cMBTag.BytesFlow == 2) // перевернутый №2 (B4:B3:B2:B1)
                                                        {
                                                            ushort_vals[0] = ud.shrt4;
                                                            ushort_vals[1] = ud.shrt3;
                                                            ushort_vals[2] = ud.shrt2;
                                                            ushort_vals[3] = ud.shrt1;
                                                        }
                                                    }
                                                    if (AttrValue.GetType() == typeof(ulong))
                                                    {
                                                        ushort_vals = new ushort[4];
                                                        ud.l1 = Convert.ToUInt64(AttrValue);
                                                        if (cMBTag.BytesFlow == 0) // Штатный
                                                        {
                                                            ushort_vals[0] = ud.shrt1;
                                                            ushort_vals[1] = ud.shrt2;
                                                            ushort_vals[2] = ud.shrt3;
                                                            ushort_vals[3] = ud.shrt4;
                                                        }
                                                        if (cMBTag.BytesFlow == 1) // перевернутый №1 (HW:LW)
                                                        {
                                                            ushort_vals[0] = ud.shrt3;
                                                            ushort_vals[1] = ud.shrt4;
                                                            ushort_vals[2] = ud.shrt1;
                                                            ushort_vals[3] = ud.shrt2;
                                                        }
                                                        if (cMBTag.BytesFlow == 2) // перевернутый №2 (B4:B3:B2:B1)
                                                        {
                                                            ushort_vals[0] = ud.shrt4;
                                                            ushort_vals[1] = ud.shrt3;
                                                            ushort_vals[2] = ud.shrt2;
                                                            ushort_vals[3] = ud.shrt1;
                                                        }
                                                    }
                                                    if (AttrValue.GetType() == typeof(ArrayList))
                                                    {
                                                        ArrayList vls = (ArrayList)AttrValue;
                                                        continue;
                                                    }
                                                    #region Проверка связи и соединения
                                                    if (this.Client != null)
                                                    {
                                                        if (!this.Client.Connected)
                                                        {
                                                            //недостоверность
                                                            cMBTag.Veracity = cMBTag.Veracity | 0x1;
                                                            ClientReady = false;
                                                            continue;
                                                        }
                                                    }
                                                    else
                                                    {
                                                        //недостоверность
                                                        cMBTag.Veracity = cMBTag.Veracity | 0x1;
                                                        ClientReady = false;
                                                        continue;
                                                    }
                                                    #endregion
                                                    sw1.Restart();
                                                    if (cMBTag.TimeoutBeforeRequest != 0)
                                                        Thread.Sleep(cMBTag.TimeoutBeforeRequest);
                                                    this.IPMaster.WriteMultipleRegisters(cMBTag.Address, cMBTag.BeginAddress, ushort_vals);
                                                    cMBTag.ChangedFlag = false;
                                                    sw1.Stop();
                                                    WorkTime = Math.Round(1000000.0D * (double)(Cycle * sw1.ElapsedTicks), 3); // микросекунды
                                                    if (WorkTime < cMBTag.MinRequestTime)
                                                        cMBTag.MinRequestTime = WorkTime;
                                                    if (WorkTime > cMBTag.MaxRequestTime)
                                                        cMBTag.MaxRequestTime = WorkTime;
                                                    cMBTag.TotalRequestCounter++;
                                                    cMBTag.TotalRequestTime += WorkTime;
                                                    cMBTag.AvrRequestTime = (double)(cMBTag.TotalRequestTime / cMBTag.TotalRequestCounter);
                                                    cMBTag.InAction = false;
                                                }
                                            }
                                            catch (Exception ex)
                                            {
                                                cMBTag.InAction = false;
                                                //недостоверность
                                                cMBTag.Veracity = cMBTag.Veracity | 0x1;

                                                this.ChangeServer();
                                                this.ErrorString = DateTime.Now.ToString() + "." + DateTime.Now.Millisecond.ToString() + ": " + "Ошибка запроса: " + ex.ToString();
                                                cMBTag.Veracity = 1;
                                                this.VeracitiesCounter++;
                                            }
                                        }
                                    }
                                }
                                catch { }
                            } */
                            #endregion

                            #region Опрос Сервисных регистров
                            //if (this.FunctionID == 2)
                            {
                                try
                                {
                                    if (ClientReady)
                                    {
                                        //foreach (ModBusReadHoldingRegistersCS cMBTag in this.ReadHoldingRegistersList)
                                        {
                                            try
                                            {
                                                #region Проверка связи и соединения
                                                if (this.Client != null)
                                                {
                                                    if (!this.Client.Connected)
                                                    {
                                                        ClientReady = false;
                                                        continue;
                                                    }
                                                }
                                                else
                                                {
                                                    ClientReady = false;
                                                    continue;
                                                }
                                                #endregion
                                                sw1.Restart();
                                                UshortVals = this.IPMaster.ReadHoldingRegisters(this.Address, 5, 2);
                                                sw1.Stop();
                                                WorkTime = Math.Round(1000000.0D * (double)(Cycle * sw1.ElapsedTicks), 3); // микросекунды
                                                /*if (WorkTime < cMBTag.MinRequestTime)
                                                    cMBTag.MinRequestTime = WorkTime;
                                                if (WorkTime > cMBTag.MaxRequestTime)
                                                    cMBTag.MaxRequestTime = WorkTime;
                                                cMBTag.TotalRequestCounter++;
                                                cMBTag.TotalRequestTime += WorkTime;
                                                cMBTag.AvrRequestTime = (double)(cMBTag.TotalRequestTime / cMBTag.TotalRequestCounter);*/

                                                this.ServiceReg0.Value = UshortVals[0];
                                                this.ServiceReg1.Value = UshortVals[1];


                                            }
                                            catch (Exception ex)
                                            {
                                                this.ErrorString = DateTime.Now.ToString() + "." + DateTime.Now.Millisecond.ToString() + ": " + "Ошибка запроса: " + ex.ToString();
                                                this.VeracitiesCounter++;
                                            }
                                        }
                                    }
                                }
                                catch { }
                            }
                            #endregion

                        }
                    }
                }
                catch (Exception ex) { this.ErrorString = ex.ToString(); }
                sw.Stop();
                this.WorkTime = Math.Round(1000.0D * (double)(Cycle * sw.ElapsedTicks), 3); // миллисекунды
                if (!this.DebugFlag)
                    CyclesCounter++;
                Thread.Sleep(this.ThreadSleepValue);
            }
        }
    }

    public class ModBusRTUServerCS : INotifyPropertyChanged
    {
        /// <summary>
        /// Номер функции для чтения (0 - Coils, 1 - Discreet Inputs, 2 - Holding, 3 - Inputs)
        /// </summary>
        public int FunctionID = 0;
        public ushort StartAddress = 0;
        public ushort NumberOfRegisters = 1;
        public int ThreadSleepValue = 100;
        public bool SendCommandFlag = false;
        public ushort SendValue = 0;
        public ushort SendAddress = 0;

        /// <summary>
        /// RS-порт
        /// </summary>
        public SerialPort COMport;
        /// <summary>
        /// Подключение к устройству
        /// </summary>
        public ModbusSerialMaster RTUMaster;
        /// <summary>
        /// Количество пропусков попыток связи
        /// </summary>
        public int RetriesCounter = 0;
        public Thread reInitThread;
        public int VeracitiesCounter = 0;
        public bool WorkerFlag = false;
        public ulong CyclesCounter = 0;
        public ulong PrevCycleCounter = 0;

        public int ClientNullCounter = 0;
        public int ClientNotConnectedCounter = 0;
        public int NoAnswerCounter = 0;

        public byte Address = 1;
        string _ErrorString = "";
        public string ErrorString
        {
            set
            {
                if (this._ErrorString != value)
                {
                    this._ErrorString = value;
                    this.OnPropertyChanged();
                }
            }
            get
            {
                return this._ErrorString;
            }
        }
        public MDBValue ServiceReg0 = new MDBValue();
        public MDBValue ServiceReg1 = new MDBValue();

        public bool ServerChangigFlag = false;
        public double WorkTime = 0;

        /// <summary>
        /// Счетчик тактов когда сервис вообще не ведет обмен. По превышению его надо рестартовать.
        /// </summary>
        public int HaltCounter = 0;
        /// <summary>
        /// Запретить изменение счетчика циклов
        /// </summary>
        public bool DebugFlag = false;

        public ObservableCollection<MDBValue> Values = new ObservableCollection<MDBValue>();

        public event PropertyChangedEventHandler PropertyChanged = delegate { };
        public void OnPropertyChanged([CallerMemberName] string propertyName = null)
        {
            this.PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
        }
        void InitClient()
        {
            try
            {
                this.COMport.RtsEnable = false;
                this.COMport.DtrEnable = false;
                this.COMport.Encoding = Encoding.ASCII;
                this.COMport.Handshake = Handshake.None;
                //this.RS.COMport.NewLine = "" + (char)0x0D + (char)0x0A;
                this.COMport.Encoding = Encoding.GetEncoding(1251);
                this.COMport.ReadTimeout = 1000;
                this.COMport.WriteTimeout = 1000;
                this.COMport.Open();
            }
            catch (Exception ex)
            {
                this.ErrorString = ("[" + DateTime.Now.ToString() + "." + DateTime.Now.Millisecond + "] ошибка инициализации: " + ex.ToString());
            }
        }
        public void RTUworker(object tsk)
        {
            bool[] BoolVals;
            ushort[] UshortVals;

            long Freq = Stopwatch.Frequency;
            double Cycle = 1.0D / (double)Freq;
            Stopwatch sw = new Stopwatch();
            Stopwatch sw1 = new Stopwatch();
            double WorkTime = 0.0D;

            while (this.WorkerFlag)
            {
                sw.Reset();
                sw.Start();
                try
                {
                    if (!this.COMport.IsOpen)
                    {
                        if (!this.ServerChangigFlag)
                        {
                            this.ClientNullCounter++;
                            this.InitClient();
                        }
                    }
                    else
                    {
                        if (this.RTUMaster == null)
                        {
                            this.RTUMaster = ModbusSerialMaster.CreateRtu(this.COMport);
                            this.RTUMaster.Transport.ReadTimeout = 1000;
                            this.RTUMaster.Transport.WriteTimeout = 1000;
                            this.ClientNullCounter++;
                        }
                        else
                        {

                            bool ClientReady = true;
                            #region Опрос ReadCoils
                            if (this.FunctionID == 0)
                            {
                                try
                                {
                                    if (ClientReady)
                                    {
                                        //foreach (ModBusReadCoilsCS cMBTag in this.ReadCoilsList)
                                        {
                                            try
                                            {
                                                #region Проверка связи и соединения
                                                /*if (this.Client != null)
                                                {
                                                    if (!this.Client.Connected)
                                                    {
                                                        ClientReady = false;
                                                        continue;
                                                    }
                                                }
                                                else
                                                {
                                                    ClientReady = false;
                                                    continue;
                                                }*/
                                                #endregion
                                                sw1.Restart();
                                                BoolVals = this.RTUMaster.ReadCoils(this.Address, this.StartAddress, this.NumberOfRegisters);
                                                sw1.Stop();
                                                WorkTime = Math.Round(1000000.0D * (double)(Cycle * sw1.ElapsedTicks), 3); // микросекунды
                                                /*if (WorkTime < cMBTag.MinRequestTime)
                                                    cMBTag.MinRequestTime = WorkTime;
                                                if (WorkTime > cMBTag.MaxRequestTime)
                                                    cMBTag.MaxRequestTime = WorkTime;
                                                cMBTag.TotalRequestCounter++;
                                                cMBTag.TotalRequestTime += WorkTime;
                                                cMBTag.AvrRequestTime = (double)(cMBTag.TotalRequestTime / cMBTag.TotalRequestCounter);*/
                                                for (int i = 0; i < BoolVals.Length; i++)
                                                {
                                                    this.Values[i].Value = BoolVals[i] ? (ushort)1 : (ushort)0;
                                                }
                                            }
                                            catch (Exception ex)
                                            {
                                                this.ErrorString = DateTime.Now.ToString() + "." + DateTime.Now.Millisecond.ToString() + ": " + "Ошибка запроса: " + ex.ToString();
                                                this.VeracitiesCounter++;
                                            }
                                        }
                                    }
                                }
                                catch { }
                            }
                            #endregion
                            #region Опрос ReadHoldingRegisters
                            if (this.FunctionID == 2)
                            {
                                try
                                {
                                    if (ClientReady)
                                    {
                                        //foreach (ModBusReadHoldingRegistersCS cMBTag in this.ReadHoldingRegistersList)
                                        {
                                            try
                                            {
                                                #region Проверка связи и соединения
                                                /*if (this.Client != null)
                                                {
                                                    if (!this.Client.Connected)
                                                    {
                                                        ClientReady = false;
                                                        continue;
                                                    }
                                                }
                                                else
                                                {
                                                    ClientReady = false;
                                                    continue;
                                                }*/
                                                #endregion
                                                sw1.Restart();
                                                UshortVals = this.RTUMaster.ReadHoldingRegisters(this.Address, this.StartAddress, this.NumberOfRegisters);
                                                sw1.Stop();
                                                WorkTime = Math.Round(1000000.0D * (double)(Cycle * sw1.ElapsedTicks), 3); // микросекунды
                                                /*if (WorkTime < cMBTag.MinRequestTime)
                                                    cMBTag.MinRequestTime = WorkTime;
                                                if (WorkTime > cMBTag.MaxRequestTime)
                                                    cMBTag.MaxRequestTime = WorkTime;
                                                cMBTag.TotalRequestCounter++;
                                                cMBTag.TotalRequestTime += WorkTime;
                                                cMBTag.AvrRequestTime = (double)(cMBTag.TotalRequestTime / cMBTag.TotalRequestCounter);*/
                                                for (int i = 0; i < UshortVals.Length; i++)
                                                {
                                                    this.Values[i].Value = UshortVals[i];
                                                }
                                            }
                                            catch (Exception ex)
                                            {
                                                this.ErrorString = DateTime.Now.ToString() + "." + DateTime.Now.Millisecond.ToString() + ": " + "Ошибка запроса: " + ex.ToString();
                                                this.VeracitiesCounter++;
                                            }
                                        }
                                    }
                                }
                                catch { }
                            }
                            #endregion
                            #region Опрос ReadInputRegisters
                            if (this.FunctionID == 3)
                            {
                                try
                                {
                                    if (ClientReady)
                                    {
                                        //foreach (ModBusReadInputRegistersCS cMBTag in this.ReadInputRegistersList)
                                        {
                                            try
                                            {
                                                #region Проверка связи и соединения
                                                /*if (this.Client != null)
                                                {
                                                    if (!this.Client.Connected)
                                                    {
                                                        ClientReady = false;
                                                        continue;
                                                    }
                                                }
                                                else
                                                {
                                                    ClientReady = false;
                                                    continue;
                                                }*/
                                                #endregion
                                                sw1.Restart();
                                                UshortVals = this.RTUMaster.ReadInputRegisters(this.Address, this.StartAddress, this.NumberOfRegisters);
                                                sw1.Stop();
                                                WorkTime = Math.Round(1000000.0D * (double)(Cycle * sw1.ElapsedTicks), 3); // микросекунды
                                                /*if (WorkTime < cMBTag.MinRequestTime)
                                                    cMBTag.MinRequestTime = WorkTime;
                                                if (WorkTime > cMBTag.MaxRequestTime)
                                                    cMBTag.MaxRequestTime = WorkTime;
                                                cMBTag.TotalRequestCounter++;
                                                cMBTag.TotalRequestTime += WorkTime;
                                                cMBTag.AvrRequestTime = (double)(cMBTag.TotalRequestTime / cMBTag.TotalRequestCounter);*/
                                                for (int i = 0; i < UshortVals.Length; i++)
                                                {
                                                    this.Values[i].Value = UshortVals[i];
                                                }
                                            }
                                            catch (Exception ex)
                                            {
                                                this.ErrorString = DateTime.Now.ToString() + "." + DateTime.Now.Millisecond.ToString() + ": " + "Ошибка запроса: " + ex.ToString();
                                                this.VeracitiesCounter++;
                                            }
                                        }
                                    }
                                }
                                catch { }
                            }
                            #endregion;
                            #region Опрос ReadInputs
                            if (this.FunctionID == 1)
                            {
                                try
                                {
                                    if (ClientReady)
                                    {
                                        //foreach (ModBusReadInputsCS cMBTag in this.ReadInputsList)
                                        {
                                            try
                                            {
                                                #region Проверка связи и соединения
                                                /*if (this.Client != null)
                                                {
                                                    if (!this.Client.Connected)
                                                    {
                                                        ClientReady = false;
                                                        continue;
                                                    }
                                                }
                                                else
                                                {
                                                    ClientReady = false;
                                                    continue;
                                                }*/
                                                #endregion
                                                sw1.Restart();
                                                BoolVals = this.RTUMaster.ReadInputs(this.Address, this.StartAddress, this.NumberOfRegisters);
                                                sw1.Stop();
                                                WorkTime = Math.Round(1000000.0D * (double)(Cycle * sw1.ElapsedTicks), 3); // микросекунды
                                                /*if (WorkTime < cMBTag.MinRequestTime)
                                                    cMBTag.MinRequestTime = WorkTime;
                                                if (WorkTime > cMBTag.MaxRequestTime)
                                                    cMBTag.MaxRequestTime = WorkTime;
                                                cMBTag.TotalRequestCounter++;
                                                cMBTag.TotalRequestTime += WorkTime;
                                                cMBTag.AvrRequestTime = (double)(cMBTag.TotalRequestTime / cMBTag.TotalRequestCounter);*/
                                                for (int i = 0; i < BoolVals.Length; i++)
                                                {
                                                    this.Values[i].Value = BoolVals[i] ? (ushort)1 : (ushort)0;
                                                }
                                            }
                                            catch (Exception ex)
                                            {
                                                this.ErrorString = DateTime.Now.ToString() + "." + DateTime.Now.Millisecond.ToString() + ": " + "Ошибка запроса: " + ex.ToString();
                                                this.VeracitiesCounter++;
                                            }
                                        }
                                    }
                                }
                                catch { }
                            }
                            #endregion
                            #region Передача WriteCoilList
                            /*
                            lock (this.WriteCoilList)
                            {
                                try
                                {
                                    if (ClientReady)
                                    {
                                        foreach (ModBusWriteCoilCS cMBTag in this.WriteCoilList)
                                        {
                                            if (cMBTag.Status)
                                                continue;
                                            try
                                            {
                                                if (cMBTag.ChangedFlag)
                                                {
                                                    cMBTag.InAction = true;
                                                    #region Проверка связи и соединения
                                                    if (this.Client != null)
                                                    {
                                                        if (!this.Client.Connected)
                                                        {
                                                            //недостоверность
                                                            cMBTag.Veracity = cMBTag.Veracity | 0x1;
                                                            ClientReady = false;
                                                            continue;
                                                        }
                                                    }
                                                    else
                                                    {
                                                        //недостоверность
                                                        cMBTag.Veracity = cMBTag.Veracity | 0x1;
                                                        ClientReady = false;
                                                        continue;
                                                    }
                                                    #endregion
                                                    sw1.Restart();
                                                    if (cMBTag.TimeoutBeforeRequest != 0)
                                                        Thread.Sleep(cMBTag.TimeoutBeforeRequest);
                                                    this.IPMaster.WriteSingleCoil(cMBTag.Address, cMBTag.BeginAddress, Convert.ToBoolean(((IAttributes)cMBTag).Get_AttributeValue("DO_OUT0")));
                                                    sw1.Stop();
                                                    WorkTime = Math.Round(1000000.0D * (double)(Cycle * sw1.ElapsedTicks), 3); // микросекунды
                                                    if (WorkTime < cMBTag.MinRequestTime)
                                                        cMBTag.MinRequestTime = WorkTime;
                                                    if (WorkTime > cMBTag.MaxRequestTime)
                                                        cMBTag.MaxRequestTime = WorkTime;
                                                    cMBTag.TotalRequestCounter++;
                                                    cMBTag.TotalRequestTime += WorkTime;
                                                    cMBTag.AvrRequestTime = (double)(cMBTag.TotalRequestTime / cMBTag.TotalRequestCounter);

                                                    cMBTag.Veracity = cMBTag.Veracity & 0xFE;
                                                    cMBTag.ChangedFlag = false;
                                                    cMBTag.InAction = false;
                                                }
                                            }
                                            catch (Exception ex)
                                            {
                                                cMBTag.InAction = false;
                                                //недостоверность
                                                cMBTag.Veracity = cMBTag.Veracity | 0x1;

                                                this.ChangeServer();
                                                this.ErrorString = DateTime.Now.ToString() + "." + DateTime.Now.Millisecond.ToString() + ": " + "Ошибка запроса: " + ex.ToString();
                                                cMBTag.Veracity = 1;
                                                this.VeracitiesCounter++;
                                            }
                                        }
                                    }
                                }
                                catch { }
                            } */
                            #endregion
                            #region Передача WriteRegister

                            if (this.SendCommandFlag)
                            {
                                try
                                {
                                    if (ClientReady)
                                    {
                                        try
                                        {
                                            //if (cMBTag.ChangedFlag)
                                            {
                                                #region Проверка связи и соединения
                                                /*if (this.Client != null)
                                                {
                                                    if (!this.Client.Connected)
                                                    {
                                                        ClientReady = false;
                                                        continue;
                                                    }
                                                }
                                                else
                                                {
                                                    ClientReady = false;
                                                    continue;
                                                }*/
                                                #endregion
                                                sw1.Restart();
                                                this.RTUMaster.WriteSingleRegister(this.Address, this.SendAddress, this.SendValue);
                                                sw1.Stop();
                                                WorkTime = Math.Round(1000000.0D * (double)(Cycle * sw1.ElapsedTicks), 3); // микросекунды
                                                /*if (WorkTime < cMBTag.MinRequestTime)
                                                    cMBTag.MinRequestTime = WorkTime;
                                                if (WorkTime > cMBTag.MaxRequestTime)
                                                    cMBTag.MaxRequestTime = WorkTime;
                                                cMBTag.TotalRequestCounter++;
                                                cMBTag.TotalRequestTime += WorkTime;
                                                cMBTag.AvrRequestTime = (double)(cMBTag.TotalRequestTime / cMBTag.TotalRequestCounter);*/

                                            }
                                        }
                                        catch (Exception ex)
                                        {
                                            this.ErrorString = DateTime.Now.ToString() + "." + DateTime.Now.Millisecond.ToString() + ": " + "Ошибка запроса: " + ex.ToString();
                                            this.VeracitiesCounter++;
                                        }

                                    }
                                }
                                catch { }
                            }
                            #endregion
                            #region Передача WriteMultipleCoils
                            /*
                            lock (this.WriteMultipleCoilsList)
                            {
                                try
                                {
                                    if (ClientReady)
                                    {
                                        foreach (ModBusWriteMultipleCoilsCS cMBTag in this.WriteMultipleCoilsList)
                                        {
                                            if (cMBTag.Status)
                                                continue;
                                            try
                                            {
                                                if (cMBTag.ChangedFlag)
                                                {
                                                    cMBTag.InAction = true;
                                                    object vals = ((IAttributes)cMBTag).Get_AttributeValue("DO_OUT0");
                                                    bool[] bool_vals = new bool[8];
                                                    #region Значение в массив бит
                                                    if (vals.GetType() == typeof(sbyte))
                                                    {
                                                        sbyte sbyte_v = Convert.ToSByte(vals);
                                                        bool_vals = new bool[8];
                                                        bool_vals[0] = (sbyte_v & 0x1) == 0x1;
                                                        bool_vals[1] = (sbyte_v & 0x2) == 0x2;
                                                        bool_vals[2] = (sbyte_v & 0x4) == 0x4;
                                                        bool_vals[3] = (sbyte_v & 0x8) == 0x8;
                                                        bool_vals[4] = (sbyte_v & 0x10) == 0x10;
                                                        bool_vals[5] = (sbyte_v & 0x20) == 0x20;
                                                        bool_vals[6] = (sbyte_v & 0x40) == 0x40;
                                                        bool_vals[7] = (sbyte_v & 0x80) == 0x80;
                                                    }
                                                    if (vals.GetType() == typeof(byte))
                                                    {
                                                        byte byte_v = Convert.ToByte(vals);
                                                        bool_vals = new bool[8];
                                                        bool_vals[0] = (byte_v & 0x1) == 0x1;
                                                        bool_vals[1] = (byte_v & 0x2) == 0x2;
                                                        bool_vals[2] = (byte_v & 0x4) == 0x4;
                                                        bool_vals[3] = (byte_v & 0x8) == 0x8;
                                                        bool_vals[4] = (byte_v & 0x10) == 0x10;
                                                        bool_vals[5] = (byte_v & 0x20) == 0x20;
                                                        bool_vals[6] = (byte_v & 0x40) == 0x40;
                                                        bool_vals[7] = (byte_v & 0x80) == 0x80;
                                                    }
                                                    if (vals.GetType() == typeof(short))
                                                    {
                                                        short short_v = Convert.ToInt16(vals);
                                                        bool_vals = new bool[16];
                                                        bool_vals[0] = (short_v & 0x1) == 0x1;
                                                        bool_vals[1] = (short_v & 0x2) == 0x2;
                                                        bool_vals[2] = (short_v & 0x4) == 0x4;
                                                        bool_vals[3] = (short_v & 0x8) == 0x8;
                                                        bool_vals[4] = (short_v & 0x10) == 0x10;
                                                        bool_vals[5] = (short_v & 0x20) == 0x20;
                                                        bool_vals[6] = (short_v & 0x40) == 0x40;
                                                        bool_vals[7] = (short_v & 0x80) == 0x80;
                                                        bool_vals[8] = (short_v & 0x100) == 0x100;
                                                        bool_vals[9] = (short_v & 0x200) == 0x200;
                                                        bool_vals[10] = (short_v & 0x400) == 0x400;
                                                        bool_vals[11] = (short_v & 0x800) == 0x800;
                                                        bool_vals[12] = (short_v & 0x1000) == 0x1000;
                                                        bool_vals[13] = (short_v & 0x2000) == 0x2000;
                                                        bool_vals[14] = (short_v & 0x4000) == 0x4000;
                                                        bool_vals[15] = (short_v & 0x8000) == 0x8000;
                                                    }
                                                    if (vals.GetType() == typeof(ushort))
                                                    {
                                                        ushort ushort_v = Convert.ToUInt16(vals);
                                                        bool_vals = new bool[16];
                                                        bool_vals[0] = (ushort_v & 0x1) == 0x1;
                                                        bool_vals[1] = (ushort_v & 0x2) == 0x2;
                                                        bool_vals[2] = (ushort_v & 0x4) == 0x4;
                                                        bool_vals[3] = (ushort_v & 0x8) == 0x8;
                                                        bool_vals[4] = (ushort_v & 0x10) == 0x10;
                                                        bool_vals[5] = (ushort_v & 0x20) == 0x20;
                                                        bool_vals[6] = (ushort_v & 0x40) == 0x40;
                                                        bool_vals[7] = (ushort_v & 0x80) == 0x80;
                                                        bool_vals[8] = (ushort_v & 0x100) == 0x100;
                                                        bool_vals[9] = (ushort_v & 0x200) == 0x200;
                                                        bool_vals[10] = (ushort_v & 0x400) == 0x400;
                                                        bool_vals[11] = (ushort_v & 0x800) == 0x800;
                                                        bool_vals[12] = (ushort_v & 0x1000) == 0x1000;
                                                        bool_vals[13] = (ushort_v & 0x2000) == 0x2000;
                                                        bool_vals[14] = (ushort_v & 0x4000) == 0x4000;
                                                        bool_vals[15] = (ushort_v & 0x8000) == 0x8000;
                                                    }
                                                    if (vals.GetType() == typeof(int))
                                                    {
                                                        int int_v = Convert.ToInt32(vals);
                                                        bool_vals = new bool[32];
                                                        bool_vals[0] = (int_v & 0x1) == 0x1;
                                                        bool_vals[1] = (int_v & 0x2) == 0x2;
                                                        bool_vals[2] = (int_v & 0x4) == 0x4;
                                                        bool_vals[3] = (int_v & 0x8) == 0x8;
                                                        bool_vals[4] = (int_v & 0x10) == 0x10;
                                                        bool_vals[5] = (int_v & 0x20) == 0x20;
                                                        bool_vals[6] = (int_v & 0x40) == 0x40;
                                                        bool_vals[7] = (int_v & 0x80) == 0x80;
                                                        bool_vals[8] = (int_v & 0x100) == 0x100;
                                                        bool_vals[9] = (int_v & 0x200) == 0x200;
                                                        bool_vals[10] = (int_v & 0x400) == 0x400;
                                                        bool_vals[11] = (int_v & 0x800) == 0x800;
                                                        bool_vals[12] = (int_v & 0x1000) == 0x1000;
                                                        bool_vals[13] = (int_v & 0x2000) == 0x2000;
                                                        bool_vals[14] = (int_v & 0x4000) == 0x4000;
                                                        bool_vals[15] = (int_v & 0x8000) == 0x8000;
                                                        bool_vals[16] = (int_v & 0x10000) == 0x10000;
                                                        bool_vals[17] = (int_v & 0x20000) == 0x20000;
                                                        bool_vals[18] = (int_v & 0x40000) == 0x40000;
                                                        bool_vals[19] = (int_v & 0x80000) == 0x80000;
                                                        bool_vals[20] = (int_v & 0x100000) == 0x100000;
                                                        bool_vals[21] = (int_v & 0x200000) == 0x200000;
                                                        bool_vals[22] = (int_v & 0x400000) == 0x400000;
                                                        bool_vals[23] = (int_v & 0x800000) == 0x800000;
                                                        bool_vals[24] = (int_v & 0x1000000) == 0x1000000;
                                                        bool_vals[25] = (int_v & 0x2000000) == 0x2000000;
                                                        bool_vals[26] = (int_v & 0x4000000) == 0x4000000;
                                                        bool_vals[27] = (int_v & 0x8000000) == 0x8000000;
                                                        bool_vals[28] = (int_v & 0x10000000) == 0x10000000;
                                                        bool_vals[29] = (int_v & 0x20000000) == 0x20000000;
                                                        bool_vals[30] = (int_v & 0x40000000) == 0x40000000;
                                                        bool_vals[31] = (int_v & 0x80000000) == 0x80000000;
                                                    }
                                                    if (vals.GetType() == typeof(uint))
                                                    {
                                                        uint uint_v = Convert.ToUInt32(vals);
                                                        bool_vals = new bool[32];
                                                        bool_vals[0] = (uint_v & 0x1) == 0x1;
                                                        bool_vals[1] = (uint_v & 0x2) == 0x2;
                                                        bool_vals[2] = (uint_v & 0x4) == 0x4;
                                                        bool_vals[3] = (uint_v & 0x8) == 0x8;
                                                        bool_vals[4] = (uint_v & 0x10) == 0x10;
                                                        bool_vals[5] = (uint_v & 0x20) == 0x20;
                                                        bool_vals[6] = (uint_v & 0x40) == 0x40;
                                                        bool_vals[7] = (uint_v & 0x80) == 0x80;
                                                        bool_vals[8] = (uint_v & 0x100) == 0x100;
                                                        bool_vals[9] = (uint_v & 0x200) == 0x200;
                                                        bool_vals[10] = (uint_v & 0x400) == 0x400;
                                                        bool_vals[11] = (uint_v & 0x800) == 0x800;
                                                        bool_vals[12] = (uint_v & 0x1000) == 0x1000;
                                                        bool_vals[13] = (uint_v & 0x2000) == 0x2000;
                                                        bool_vals[14] = (uint_v & 0x4000) == 0x4000;
                                                        bool_vals[15] = (uint_v & 0x8000) == 0x8000;
                                                        bool_vals[16] = (uint_v & 0x10000) == 0x10000;
                                                        bool_vals[17] = (uint_v & 0x20000) == 0x20000;
                                                        bool_vals[18] = (uint_v & 0x40000) == 0x40000;
                                                        bool_vals[19] = (uint_v & 0x80000) == 0x80000;
                                                        bool_vals[20] = (uint_v & 0x100000) == 0x100000;
                                                        bool_vals[21] = (uint_v & 0x200000) == 0x200000;
                                                        bool_vals[22] = (uint_v & 0x400000) == 0x400000;
                                                        bool_vals[23] = (uint_v & 0x800000) == 0x800000;
                                                        bool_vals[24] = (uint_v & 0x1000000) == 0x1000000;
                                                        bool_vals[25] = (uint_v & 0x2000000) == 0x2000000;
                                                        bool_vals[26] = (uint_v & 0x4000000) == 0x4000000;
                                                        bool_vals[27] = (uint_v & 0x8000000) == 0x8000000;
                                                        bool_vals[28] = (uint_v & 0x10000000) == 0x10000000;
                                                        bool_vals[29] = (uint_v & 0x20000000) == 0x20000000;
                                                        bool_vals[30] = (uint_v & 0x40000000) == 0x40000000;
                                                        bool_vals[31] = (uint_v & 0x80000000) == 0x80000000;
                                                    }
                                                    if (vals.GetType() == typeof(long))
                                                    {
                                                        long long_v = Convert.ToInt64(vals);
                                                        bool_vals = new bool[64];
                                                        bool_vals[0] = (long_v & 0x1) == 0x1;
                                                        bool_vals[1] = (long_v & 0x2) == 0x2;
                                                        bool_vals[2] = (long_v & 0x4) == 0x4;
                                                        bool_vals[3] = (long_v & 0x8) == 0x8;
                                                        bool_vals[4] = (long_v & 0x10) == 0x10;
                                                        bool_vals[5] = (long_v & 0x20) == 0x20;
                                                        bool_vals[6] = (long_v & 0x40) == 0x40;
                                                        bool_vals[7] = (long_v & 0x80) == 0x80;
                                                        bool_vals[8] = (long_v & 0x100) == 0x100;
                                                        bool_vals[9] = (long_v & 0x200) == 0x200;
                                                        bool_vals[10] = (long_v & 0x400) == 0x400;
                                                        bool_vals[11] = (long_v & 0x800) == 0x800;
                                                        bool_vals[12] = (long_v & 0x1000) == 0x1000;
                                                        bool_vals[13] = (long_v & 0x2000) == 0x2000;
                                                        bool_vals[14] = (long_v & 0x4000) == 0x4000;
                                                        bool_vals[15] = (long_v & 0x8000) == 0x8000;
                                                        bool_vals[16] = (long_v & 0x10000) == 0x10000;
                                                        bool_vals[17] = (long_v & 0x20000) == 0x20000;
                                                        bool_vals[18] = (long_v & 0x40000) == 0x40000;
                                                        bool_vals[19] = (long_v & 0x80000) == 0x80000;
                                                        bool_vals[20] = (long_v & 0x100000) == 0x100000;
                                                        bool_vals[21] = (long_v & 0x200000) == 0x200000;
                                                        bool_vals[22] = (long_v & 0x400000) == 0x400000;
                                                        bool_vals[23] = (long_v & 0x800000) == 0x800000;
                                                        bool_vals[24] = (long_v & 0x1000000) == 0x1000000;
                                                        bool_vals[25] = (long_v & 0x2000000) == 0x2000000;
                                                        bool_vals[26] = (long_v & 0x4000000) == 0x4000000;
                                                        bool_vals[27] = (long_v & 0x8000000) == 0x8000000;
                                                        bool_vals[28] = (long_v & 0x10000000) == 0x10000000;
                                                        bool_vals[29] = (long_v & 0x20000000) == 0x20000000;
                                                        bool_vals[30] = (long_v & 0x40000000) == 0x40000000;
                                                        bool_vals[31] = (long_v & 0x80000000) == 0x80000000;
                                                        bool_vals[32] = (long_v & 0x100000000) == 0x100000000;
                                                        bool_vals[33] = (long_v & 0x200000000) == 0x200000000;
                                                        bool_vals[34] = (long_v & 0x400000000) == 0x400000000;
                                                        bool_vals[35] = (long_v & 0x800000000) == 0x800000000;
                                                        bool_vals[36] = (long_v & 0x1000000000) == 0x1000000000;
                                                        bool_vals[37] = (long_v & 0x2000000000) == 0x2000000000;
                                                        bool_vals[38] = (long_v & 0x4000000000) == 0x4000000000;
                                                        bool_vals[39] = (long_v & 0x8000000000) == 0x8000000000;
                                                        bool_vals[40] = (long_v & 0x10000000000) == 0x10000000000;
                                                        bool_vals[41] = (long_v & 0x20000000000) == 0x20000000000;
                                                        bool_vals[42] = (long_v & 0x40000000000) == 0x40000000000;
                                                        bool_vals[43] = (long_v & 0x80000000000) == 0x80000000000;
                                                        bool_vals[44] = (long_v & 0x100000000000) == 0x100000000000;
                                                        bool_vals[45] = (long_v & 0x200000000000) == 0x200000000000;
                                                        bool_vals[46] = (long_v & 0x400000000000) == 0x400000000000;
                                                        bool_vals[47] = (long_v & 0x800000000000) == 0x800000000000;
                                                        bool_vals[48] = (long_v & 0x1000000000000) == 0x1000000000000;
                                                        bool_vals[49] = (long_v & 0x2000000000000) == 0x2000000000000;
                                                        bool_vals[50] = (long_v & 0x4000000000000) == 0x4000000000000;
                                                        bool_vals[51] = (long_v & 0x8000000000000) == 0x8000000000000;
                                                        bool_vals[52] = (long_v & 0x10000000000000) == 0x10000000000000;
                                                        bool_vals[53] = (long_v & 0x20000000000000) == 0x20000000000000;
                                                        bool_vals[54] = (long_v & 0x40000000000000) == 0x40000000000000;
                                                        bool_vals[55] = (long_v & 0x80000000000000) == 0x80000000000000;
                                                        bool_vals[56] = (long_v & 0x100000000000000) == 0x100000000000000;
                                                        bool_vals[57] = (long_v & 0x200000000000000) == 0x200000000000000;
                                                        bool_vals[58] = (long_v & 0x400000000000000) == 0x400000000000000;
                                                        bool_vals[59] = (long_v & 0x800000000000000) == 0x800000000000000;
                                                        bool_vals[60] = (long_v & 0x1000000000000000) == 0x1000000000000000;
                                                        bool_vals[61] = (long_v & 0x2000000000000000) == 0x2000000000000000;
                                                        bool_vals[62] = (long_v & 0x4000000000000000) == 0x4000000000000000;
                                                        bool_vals[63] = ((ulong)long_v & 0x8000000000000000) == 0x8000000000000000;
                                                    }
                                                    if (vals.GetType() == typeof(ulong))
                                                    {
                                                        ulong ulong_v = Convert.ToUInt64(vals);
                                                        bool_vals = new bool[64];
                                                        bool_vals[0] = (ulong_v & 0x1) == 0x1;
                                                        bool_vals[1] = (ulong_v & 0x2) == 0x2;
                                                        bool_vals[2] = (ulong_v & 0x4) == 0x4;
                                                        bool_vals[3] = (ulong_v & 0x8) == 0x8;
                                                        bool_vals[4] = (ulong_v & 0x10) == 0x10;
                                                        bool_vals[5] = (ulong_v & 0x20) == 0x20;
                                                        bool_vals[6] = (ulong_v & 0x40) == 0x40;
                                                        bool_vals[7] = (ulong_v & 0x80) == 0x80;
                                                        bool_vals[8] = (ulong_v & 0x100) == 0x100;
                                                        bool_vals[9] = (ulong_v & 0x200) == 0x200;
                                                        bool_vals[10] = (ulong_v & 0x400) == 0x400;
                                                        bool_vals[11] = (ulong_v & 0x800) == 0x800;
                                                        bool_vals[12] = (ulong_v & 0x1000) == 0x1000;
                                                        bool_vals[13] = (ulong_v & 0x2000) == 0x2000;
                                                        bool_vals[14] = (ulong_v & 0x4000) == 0x4000;
                                                        bool_vals[15] = (ulong_v & 0x8000) == 0x8000;
                                                        bool_vals[16] = (ulong_v & 0x10000) == 0x10000;
                                                        bool_vals[17] = (ulong_v & 0x20000) == 0x20000;
                                                        bool_vals[18] = (ulong_v & 0x40000) == 0x40000;
                                                        bool_vals[19] = (ulong_v & 0x80000) == 0x80000;
                                                        bool_vals[20] = (ulong_v & 0x100000) == 0x100000;
                                                        bool_vals[21] = (ulong_v & 0x200000) == 0x200000;
                                                        bool_vals[22] = (ulong_v & 0x400000) == 0x400000;
                                                        bool_vals[23] = (ulong_v & 0x800000) == 0x800000;
                                                        bool_vals[24] = (ulong_v & 0x1000000) == 0x1000000;
                                                        bool_vals[25] = (ulong_v & 0x2000000) == 0x2000000;
                                                        bool_vals[26] = (ulong_v & 0x4000000) == 0x4000000;
                                                        bool_vals[27] = (ulong_v & 0x8000000) == 0x8000000;
                                                        bool_vals[28] = (ulong_v & 0x10000000) == 0x10000000;
                                                        bool_vals[29] = (ulong_v & 0x20000000) == 0x20000000;
                                                        bool_vals[30] = (ulong_v & 0x40000000) == 0x40000000;
                                                        bool_vals[31] = (ulong_v & 0x80000000) == 0x80000000;
                                                        bool_vals[32] = (ulong_v & 0x100000000) == 0x100000000;
                                                        bool_vals[33] = (ulong_v & 0x200000000) == 0x200000000;
                                                        bool_vals[34] = (ulong_v & 0x400000000) == 0x400000000;
                                                        bool_vals[35] = (ulong_v & 0x800000000) == 0x800000000;
                                                        bool_vals[36] = (ulong_v & 0x1000000000) == 0x1000000000;
                                                        bool_vals[37] = (ulong_v & 0x2000000000) == 0x2000000000;
                                                        bool_vals[38] = (ulong_v & 0x4000000000) == 0x4000000000;
                                                        bool_vals[39] = (ulong_v & 0x8000000000) == 0x8000000000;
                                                        bool_vals[40] = (ulong_v & 0x10000000000) == 0x10000000000;
                                                        bool_vals[41] = (ulong_v & 0x20000000000) == 0x20000000000;
                                                        bool_vals[42] = (ulong_v & 0x40000000000) == 0x40000000000;
                                                        bool_vals[43] = (ulong_v & 0x80000000000) == 0x80000000000;
                                                        bool_vals[44] = (ulong_v & 0x100000000000) == 0x100000000000;
                                                        bool_vals[45] = (ulong_v & 0x200000000000) == 0x200000000000;
                                                        bool_vals[46] = (ulong_v & 0x400000000000) == 0x400000000000;
                                                        bool_vals[47] = (ulong_v & 0x800000000000) == 0x800000000000;
                                                        bool_vals[48] = (ulong_v & 0x1000000000000) == 0x1000000000000;
                                                        bool_vals[49] = (ulong_v & 0x2000000000000) == 0x2000000000000;
                                                        bool_vals[50] = (ulong_v & 0x4000000000000) == 0x4000000000000;
                                                        bool_vals[51] = (ulong_v & 0x8000000000000) == 0x8000000000000;
                                                        bool_vals[52] = (ulong_v & 0x10000000000000) == 0x10000000000000;
                                                        bool_vals[53] = (ulong_v & 0x20000000000000) == 0x20000000000000;
                                                        bool_vals[54] = (ulong_v & 0x40000000000000) == 0x40000000000000;
                                                        bool_vals[55] = (ulong_v & 0x80000000000000) == 0x80000000000000;
                                                        bool_vals[56] = (ulong_v & 0x100000000000000) == 0x100000000000000;
                                                        bool_vals[57] = (ulong_v & 0x200000000000000) == 0x200000000000000;
                                                        bool_vals[58] = (ulong_v & 0x400000000000000) == 0x400000000000000;
                                                        bool_vals[59] = (ulong_v & 0x800000000000000) == 0x800000000000000;
                                                        bool_vals[60] = (ulong_v & 0x1000000000000000) == 0x1000000000000000;
                                                        bool_vals[61] = (ulong_v & 0x2000000000000000) == 0x2000000000000000;
                                                        bool_vals[62] = (ulong_v & 0x4000000000000000) == 0x4000000000000000;
                                                        bool_vals[63] = (ulong_v & 0x8000000000000000) == 0x8000000000000000;
                                                    }
                                                    if (vals.GetType() == typeof(ArrayList))
                                                    {
                                                        ArrayList vls = (ArrayList)vals;
                                                        break;
                                                    }
                                                    #endregion
                                                    bool[] wbools = new bool[cMBTag.NumberOfRegisters];
                                                    if (cMBTag.NumberOfRegisters <= bool_vals.Length)
                                                    {
                                                        for (int i = 0; i < cMBTag.NumberOfRegisters; i++)
                                                            wbools[i] = bool_vals[i];
                                                    }
                                                    if (cMBTag.NumberOfRegisters > bool_vals.Length)
                                                    {
                                                        for (int i = 0; i < bool_vals.Length; i++)
                                                            wbools[i] = bool_vals[i];
                                                    }
                                                    #region Проверка связи и соединения
                                                    if (this.Client != null)
                                                    {
                                                        if (!this.Client.Connected)
                                                        {
                                                            //недостоверность
                                                            cMBTag.Veracity = cMBTag.Veracity | 0x1;
                                                            ClientReady = false;
                                                            continue;
                                                        }
                                                    }
                                                    else
                                                    {
                                                        //недостоверность
                                                        cMBTag.Veracity = cMBTag.Veracity | 0x1;
                                                        ClientReady = false;
                                                        continue;
                                                    }
                                                    #endregion
                                                    sw1.Restart();
                                                    if (cMBTag.TimeoutBeforeRequest != 0)
                                                        Thread.Sleep(cMBTag.TimeoutBeforeRequest);
                                                    this.IPMaster.WriteMultipleCoils(cMBTag.Address, cMBTag.BeginAddress, wbools);
                                                    sw1.Stop();
                                                    WorkTime = Math.Round(1000000.0D * (double)(Cycle * sw1.ElapsedTicks), 3); // микросекунды
                                                    if (WorkTime < cMBTag.MinRequestTime)
                                                        cMBTag.MinRequestTime = WorkTime;
                                                    if (WorkTime > cMBTag.MaxRequestTime)
                                                        cMBTag.MaxRequestTime = WorkTime;
                                                    cMBTag.TotalRequestCounter++;
                                                    cMBTag.TotalRequestTime += WorkTime;
                                                    cMBTag.AvrRequestTime = (double)(cMBTag.TotalRequestTime / cMBTag.TotalRequestCounter);

                                                    cMBTag.Veracity = cMBTag.Veracity & 0xFE;
                                                    cMBTag.ChangedFlag = false;
                                                    cMBTag.InAction = false;
                                                }
                                            }
                                            catch (Exception ex)
                                            {
                                                cMBTag.InAction = false;
                                                //недостоверность
                                                cMBTag.Veracity = cMBTag.Veracity | 0x1;

                                                this.ChangeServer();
                                                this.ErrorString = DateTime.Now.ToString() + "." + DateTime.Now.Millisecond.ToString() + ": " + "Ошибка запроса: " + ex.ToString();
                                                cMBTag.Veracity = 1;
                                                this.VeracitiesCounter++;
                                            }
                                        }
                                    }
                                }
                                catch { }
                            }*/
                            #endregion
                            #region Передача WriteMultipleRegisters
                            /*
                            lock (this.WriteMultipleRegistersList)
                            {
                                try
                                {
                                    if (ClientReady)
                                    {
                                        foreach (ModBusWriteMultipleRegistersCS cMBTag in this.WriteMultipleRegistersList)
                                        {
                                            if (cMBTag.Status)
                                                continue;
                                            try
                                            {
                                                if (cMBTag.ChangedFlag)
                                                {
                                                    cMBTag.InAction = true;
                                                    cMBTag.Veracity = cMBTag.Veracity & 0xFE;
                                                    ushort[] ushort_vals = new ushort[1];
                                                    object AttrValue = ((IAttributes)cMBTag).Get_AttributeValue("DO_OUT0");
                                                    UDataConvert ud = new UDataConvert();
                                                    if (AttrValue.GetType() == typeof(float))
                                                    {
                                                        ushort_vals = new ushort[2];
                                                        ud.f1 = Convert.ToSingle(AttrValue);
                                                        if (cMBTag.BytesFlow == 0) // Штатный
                                                        {
                                                            ushort_vals[0] = ud.shrt1;
                                                            ushort_vals[1] = ud.shrt2;
                                                        }
                                                        if (cMBTag.BytesFlow == 1) // перевернутый №1 (HW:LW)
                                                        {
                                                            ushort_vals[0] = ud.shrt2;
                                                            ushort_vals[1] = ud.shrt1;
                                                        }
                                                        if (cMBTag.BytesFlow == 2) // перевернутый №2 (B4:B3:B2:B1)
                                                        {
                                                            UDataConvert fv = new UDataConvert();
                                                            fv.b1 = ud.b4;
                                                            fv.b2 = ud.b3;
                                                            fv.b3 = ud.b2;
                                                            fv.b4 = ud.b1;
                                                            ushort_vals[0] = fv.shrt1;
                                                            ushort_vals[1] = fv.shrt2;
                                                        }
                                                    }
                                                    if (AttrValue.GetType() == typeof(double))
                                                    {
                                                        ushort_vals = new ushort[4];
                                                        ud.d1 = Convert.ToDouble(AttrValue);
                                                        if (cMBTag.BytesFlow == 0) // Штатный
                                                        {
                                                            ushort_vals[0] = ud.shrt1;
                                                            ushort_vals[1] = ud.shrt2;
                                                            ushort_vals[2] = ud.shrt3;
                                                            ushort_vals[3] = ud.shrt4;
                                                        }
                                                        if (cMBTag.BytesFlow == 1) // перевернутый №1 (HW:LW)
                                                        {
                                                            ushort_vals[0] = ud.shrt3;
                                                            ushort_vals[1] = ud.shrt4;
                                                            ushort_vals[2] = ud.shrt1;
                                                            ushort_vals[3] = ud.shrt2;
                                                        }
                                                        if (cMBTag.BytesFlow == 2) // перевернутый №2 (B4:B3:B2:B1)
                                                        {
                                                            ushort_vals[0] = ud.shrt4;
                                                            ushort_vals[1] = ud.shrt3;
                                                            ushort_vals[2] = ud.shrt2;
                                                            ushort_vals[3] = ud.shrt1;
                                                        }
                                                    }
                                                    if (AttrValue.GetType() == typeof(sbyte))
                                                    {
                                                        ushort_vals = new ushort[1];
                                                        ud.b1 = Convert.ToByte(AttrValue);
                                                        ushort_vals[0] = ud.shrt1;
                                                    }
                                                    if (AttrValue.GetType() == typeof(byte))
                                                    {
                                                        ushort_vals = new ushort[1];
                                                        ud.b1 = Convert.ToByte(AttrValue);
                                                        ushort_vals[0] = ud.shrt1;
                                                    }
                                                    if (AttrValue.GetType() == typeof(short))
                                                    {
                                                        ushort_vals = new ushort[1];
                                                        ud.shrt1 = Convert.ToUInt16(AttrValue);
                                                        ushort_vals[0] = ud.shrt1;
                                                    }
                                                    if (AttrValue.GetType() == typeof(ushort))
                                                    {
                                                        ushort_vals = new ushort[1];
                                                        ud.shrt1 = Convert.ToUInt16(AttrValue);
                                                        ushort_vals[0] = ud.shrt1;
                                                    }
                                                    if (AttrValue.GetType() == typeof(int))
                                                    {
                                                        ushort_vals = new ushort[2];
                                                        ud.i1 = Convert.ToUInt32(AttrValue);
                                                        if (cMBTag.BytesFlow == 0) // Штатный
                                                        {
                                                            ushort_vals[0] = ud.shrt1;
                                                            ushort_vals[1] = ud.shrt2;
                                                        }
                                                        if (cMBTag.BytesFlow == 1) // перевернутый №1 (HW:LW)
                                                        {
                                                            ushort_vals[0] = ud.shrt2;
                                                            ushort_vals[1] = ud.shrt3;
                                                        }
                                                        if (cMBTag.BytesFlow == 2) // перевернутый №2 (B4:B3:B2:B1)
                                                        {
                                                            UDataConvert iv = new UDataConvert();
                                                            iv.b1 = ud.b4;
                                                            iv.b2 = ud.b3;
                                                            iv.b3 = ud.b2;
                                                            iv.b4 = ud.b1;
                                                            ushort_vals[0] = iv.shrt1;
                                                            ushort_vals[1] = iv.shrt2;
                                                        }
                                                    }
                                                    if (AttrValue.GetType() == typeof(uint))
                                                    {
                                                        ushort_vals = new ushort[2];
                                                        ud.i1 = Convert.ToUInt32(AttrValue);
                                                        if (cMBTag.BytesFlow == 0) // Штатный
                                                        {
                                                            ushort_vals[0] = ud.shrt1;
                                                            ushort_vals[1] = ud.shrt2;
                                                        }
                                                        if (cMBTag.BytesFlow == 1) // перевернутый №1 (HW:LW)
                                                        {
                                                            ushort_vals[0] = ud.shrt2;
                                                            ushort_vals[1] = ud.shrt1;
                                                        }
                                                        if (cMBTag.BytesFlow == 2) // перевернутый №2 (B4:B3:B2:B1)
                                                        {
                                                            UDataConvert uiv = new UDataConvert();
                                                            uiv.b1 = ud.b4;
                                                            uiv.b2 = ud.b3;
                                                            uiv.b3 = ud.b2;
                                                            uiv.b4 = ud.b1;
                                                            ushort_vals[0] = uiv.shrt1;
                                                            ushort_vals[1] = uiv.shrt2;
                                                        }
                                                    }
                                                    if (AttrValue.GetType() == typeof(long))
                                                    {
                                                        ushort_vals = new ushort[4];
                                                        ud.l1 = Convert.ToUInt64(AttrValue);
                                                        if (cMBTag.BytesFlow == 0) // Штатный
                                                        {
                                                            ushort_vals[0] = ud.shrt1;
                                                            ushort_vals[1] = ud.shrt2;
                                                            ushort_vals[2] = ud.shrt3;
                                                            ushort_vals[3] = ud.shrt4;
                                                        }
                                                        if (cMBTag.BytesFlow == 1) // перевернутый №1 (HW:LW)
                                                        {
                                                            ushort_vals[0] = ud.shrt3;
                                                            ushort_vals[1] = ud.shrt4;
                                                            ushort_vals[2] = ud.shrt1;
                                                            ushort_vals[3] = ud.shrt2;
                                                        }
                                                        if (cMBTag.BytesFlow == 2) // перевернутый №2 (B4:B3:B2:B1)
                                                        {
                                                            ushort_vals[0] = ud.shrt4;
                                                            ushort_vals[1] = ud.shrt3;
                                                            ushort_vals[2] = ud.shrt2;
                                                            ushort_vals[3] = ud.shrt1;
                                                        }
                                                    }
                                                    if (AttrValue.GetType() == typeof(ulong))
                                                    {
                                                        ushort_vals = new ushort[4];
                                                        ud.l1 = Convert.ToUInt64(AttrValue);
                                                        if (cMBTag.BytesFlow == 0) // Штатный
                                                        {
                                                            ushort_vals[0] = ud.shrt1;
                                                            ushort_vals[1] = ud.shrt2;
                                                            ushort_vals[2] = ud.shrt3;
                                                            ushort_vals[3] = ud.shrt4;
                                                        }
                                                        if (cMBTag.BytesFlow == 1) // перевернутый №1 (HW:LW)
                                                        {
                                                            ushort_vals[0] = ud.shrt3;
                                                            ushort_vals[1] = ud.shrt4;
                                                            ushort_vals[2] = ud.shrt1;
                                                            ushort_vals[3] = ud.shrt2;
                                                        }
                                                        if (cMBTag.BytesFlow == 2) // перевернутый №2 (B4:B3:B2:B1)
                                                        {
                                                            ushort_vals[0] = ud.shrt4;
                                                            ushort_vals[1] = ud.shrt3;
                                                            ushort_vals[2] = ud.shrt2;
                                                            ushort_vals[3] = ud.shrt1;
                                                        }
                                                    }
                                                    if (AttrValue.GetType() == typeof(ArrayList))
                                                    {
                                                        ArrayList vls = (ArrayList)AttrValue;
                                                        continue;
                                                    }
                                                    #region Проверка связи и соединения
                                                    if (this.Client != null)
                                                    {
                                                        if (!this.Client.Connected)
                                                        {
                                                            //недостоверность
                                                            cMBTag.Veracity = cMBTag.Veracity | 0x1;
                                                            ClientReady = false;
                                                            continue;
                                                        }
                                                    }
                                                    else
                                                    {
                                                        //недостоверность
                                                        cMBTag.Veracity = cMBTag.Veracity | 0x1;
                                                        ClientReady = false;
                                                        continue;
                                                    }
                                                    #endregion
                                                    sw1.Restart();
                                                    if (cMBTag.TimeoutBeforeRequest != 0)
                                                        Thread.Sleep(cMBTag.TimeoutBeforeRequest);
                                                    this.IPMaster.WriteMultipleRegisters(cMBTag.Address, cMBTag.BeginAddress, ushort_vals);
                                                    cMBTag.ChangedFlag = false;
                                                    sw1.Stop();
                                                    WorkTime = Math.Round(1000000.0D * (double)(Cycle * sw1.ElapsedTicks), 3); // микросекунды
                                                    if (WorkTime < cMBTag.MinRequestTime)
                                                        cMBTag.MinRequestTime = WorkTime;
                                                    if (WorkTime > cMBTag.MaxRequestTime)
                                                        cMBTag.MaxRequestTime = WorkTime;
                                                    cMBTag.TotalRequestCounter++;
                                                    cMBTag.TotalRequestTime += WorkTime;
                                                    cMBTag.AvrRequestTime = (double)(cMBTag.TotalRequestTime / cMBTag.TotalRequestCounter);
                                                    cMBTag.InAction = false;
                                                }
                                            }
                                            catch (Exception ex)
                                            {
                                                cMBTag.InAction = false;
                                                //недостоверность
                                                cMBTag.Veracity = cMBTag.Veracity | 0x1;

                                                this.ChangeServer();
                                                this.ErrorString = DateTime.Now.ToString() + "." + DateTime.Now.Millisecond.ToString() + ": " + "Ошибка запроса: " + ex.ToString();
                                                cMBTag.Veracity = 1;
                                                this.VeracitiesCounter++;
                                            }
                                        }
                                    }
                                }
                                catch { }
                            } */
                            #endregion

                            #region Опрос Сервисных регистров
                            //if (this.FunctionID == 2)
                            {
                                try
                                {
                                    if (ClientReady)
                                    {
                                        //foreach (ModBusReadHoldingRegistersCS cMBTag in this.ReadHoldingRegistersList)
                                        {
                                            try
                                            {
                                                #region Проверка связи и соединения
                                                /*if (this.Client != null)
                                                {
                                                    if (!this.Client.Connected)
                                                    {
                                                        ClientReady = false;
                                                        continue;
                                                    }
                                                }
                                                else
                                                {
                                                    ClientReady = false;
                                                    continue;
                                                }*/
                                                #endregion
                                                sw1.Restart();
                                                UshortVals = this.RTUMaster.ReadHoldingRegisters(this.Address, 5, 2);
                                                sw1.Stop();
                                                WorkTime = Math.Round(1000000.0D * (double)(Cycle * sw1.ElapsedTicks), 3); // микросекунды
                                                /*if (WorkTime < cMBTag.MinRequestTime)
                                                    cMBTag.MinRequestTime = WorkTime;
                                                if (WorkTime > cMBTag.MaxRequestTime)
                                                    cMBTag.MaxRequestTime = WorkTime;
                                                cMBTag.TotalRequestCounter++;
                                                cMBTag.TotalRequestTime += WorkTime;
                                                cMBTag.AvrRequestTime = (double)(cMBTag.TotalRequestTime / cMBTag.TotalRequestCounter);*/

                                                this.ServiceReg0.Value = UshortVals[0];
                                                this.ServiceReg1.Value = UshortVals[1];


                                            }
                                            catch (Exception ex)
                                            {
                                                this.ErrorString = DateTime.Now.ToString() + "." + DateTime.Now.Millisecond.ToString() + ": " + "Ошибка запроса: " + ex.ToString();
                                                this.VeracitiesCounter++;
                                            }
                                        }
                                    }
                                }
                                catch { }
                            }
                            #endregion

                        }
                    }
                }
                catch (Exception ex) { this.ErrorString = ex.ToString(); }
                sw.Stop();
                this.WorkTime = Math.Round(1000.0D * (double)(Cycle * sw.ElapsedTicks), 3); // миллисекунды
                if (!this.DebugFlag)
                    CyclesCounter++;
                Thread.Sleep(this.ThreadSleepValue);
            }
        }
    }

    public class MDBValue : INotifyPropertyChanged
    {
        public int Id { set; get; }
        public ushort Address { set; get; }
        public ushort PrevValue = 0;
        ushort _Value = 0;
        public ushort Value
        {
            set
            {
                if (this._Value != value)
                {
                    this._Value = value;
                    this.OnPropertyChanged();
                }
            }
            get
            {
                return this._Value;
            }
        }

        public event PropertyChangedEventHandler PropertyChanged = delegate { };
        public void OnPropertyChanged([CallerMemberName] string propertyName = null)
        {
            this.PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
        }
    }
}
