﻿using System;

namespace GlobalCF
{
	public static class RSR2_IPS_Helper
	{
		public static GKDriver Create()
		{
			var driver = new GKDriver()
			{
				DriverTypeNo = 0xC5,
				DriverType = GKDriverType.RSR2_IPS,
				UID = new Guid("127CF67C-1275-4DC3-8AED-216FA29BCFA9"),
				Name = "Извещатель-преобразователь состояния",
				ShortName = "ИПС",
				HasZone = true,
				IsPlaceable = true,
			};

			GKDriversHelper.AddAvailableStateClasses(driver, XStateClass.Test);
			GKDriversHelper.AddAvailableStateClasses(driver, XStateClass.Fire1);
			GKDriversHelper.AddAvailableStateClasses(driver, XStateClass.Fire2);
			GKDriversHelper.AddAvailableStateClasses(driver, XStateClass.Attention);
			GKDriversHelper.AddAvailableStateClasses(driver, XStateClass.ConnectionLost);

			GKDriversHelper.AddAvailableStateBits(driver, GKStateBit.Test);
			GKDriversHelper.AddAvailableStateBits(driver, GKStateBit.Fire1);
			GKDriversHelper.AddAvailableStateBits(driver, GKStateBit.Fire2);

			GKDriversHelper.AddIntProprety(driver, 0, "Адрес RS-485 (привязка 1)", 1, 1, 247,true);
			GKDriversHelper.AddIntProprety(driver, 0, "", 0xC5, 0xC5, 0xC5, false, true, true);
			GKDriversHelper.AddIntProprety(driver, 1, "Адрес регистра данных (привязка 2)", 1, 1, 65535);
			var property2 = new GKDriverProperty()
			{
				No = 2,
				Name = "Команда чтения регистра",
				Caption = "Команда чтения регистра",
				Default = 1
			};
			GKDriversHelper.AddPropertyParameter(property2, "0x01", 1);
			GKDriversHelper.AddPropertyParameter(property2, "0x03", 3);
			GKDriversHelper.AddPropertyParameter(property2, "0x04", 4);
			driver.Properties.Add(property2);

			GKDriversHelper.AddIntProprety(driver, 3, "Адрес регистра неисправности (привязка 3)", 1, 1, 65535);
			GKDriversHelper.AddIntProprety(driver, 4, "Маска регистра неисправности", 0, 0, 65535);

			byte numberProperty = 5;

			for(var i = 0; i <= 15; i++)
			{
				if (i != 0 && i % 2 == 0)
					numberProperty++;

				var property3 = new GKDriverProperty()
				{
					No = numberProperty,
					IsHieghByte = i % 2 != 0,
					IsLowByte = i % 2 == 0,
					Default = 0,
					Mask = 15,
					Min = 0,
					Max = 15,
					DriverPropertyType = GKDriverPropertyTypeEnum.IntType,
					CanNotEdit = true
				};
				if (i == 1)
				{
					property3.Name = " Бит 1 «Внимание»";
					property3.Caption = " Бит 1 «Внимание»";
					property3.CanNotEdit = false;
				}
				if (i == 2)
				{
					property3.Name = "Бит 2 «Сработка 1»";
					property3.Caption = "Бит 2 «Сработка 1»";
					property3.CanNotEdit = false;
				}
				if (i == 3)
				{
					property3.Name = "Бит 3 «Сработка 2»";
					property3.Caption = "Бит 3 «Сработка 2»";
					property3.CanNotEdit = false;
				}
				driver.Properties.Add(property3);

				var property4 = new GKDriverProperty()
				{
					No = numberProperty,
					IsHieghByte = i % 2 != 0,
					IsLowByte = i % 2 == 0,
					Name = "Валидность Бит " + i,
					Caption = "Валидность Бит " + i,
					Default = 0,
					Mask = 16,
					CanNotEdit = true
				};
				GKDriversHelper.AddPropertyParameter(property4, "Нет", 0);
				GKDriversHelper.AddPropertyParameter(property4, "Да", 16);
				if (i <= 3 && i >= 1 || i == 5)
					property4.CanNotEdit = false;
				driver.Properties.Add(property4);

				var property5 = new GKDriverProperty()
				{
					No = numberProperty,
					IsHieghByte = i % 2 != 0,
					IsLowByte = i % 2 == 0,
					Name = "Инверсия Бит " + i,
					Caption = "Инверсия Бит " + i,
					Default = 0,
					Mask = 32,
					CanNotEdit = true
				};
				GKDriversHelper.AddPropertyParameter(property5, "Нет", 0);
				GKDriversHelper.AddPropertyParameter(property5, "Да", 32);
				if (i <= 3 && i >= 1 || i == 5)
					property5.CanNotEdit = false;
				driver.Properties.Add(property5);
			}
			return driver;
		}
	}
}