﻿using System;

namespace GlobalCF
{
	public static class RSR2_KPP_Helper
	{
		public static GKDriver Create()
		{
			var driver = new GKDriver()
			{
				DriverTypeNo = 0xD6,
				DriverType = GKDriverType.RSR2_KPP,
				UID = new Guid("13286BE1-D14D-479E-8F44-E98225340F7C"),
				Name = "Контроллер пиропатрона",
				ShortName = "КПП",
				IsControlDevice = true,
				HasLogic = true,
				IsPlaceable = true,
				DriverClassification = GKDriver.DriverClassifications.ActuatingDevice
			};

			GKDriversHelper.AddControlAvailableStates(driver);
			driver.AvailableStateBits.Add(GKStateBit.Off);
			driver.AvailableStateBits.Add(GKStateBit.TurningOn);
			GKDriversHelper.AddAvailableStateClasses(driver, XStateClass.AutoOff);
			GKDriversHelper.AddAvailableStateClasses(driver, XStateClass.On);
			GKDriversHelper.AddAvailableStateClasses(driver, XStateClass.TurningOn);
			GKDriversHelper.AddAvailableStateClasses(driver, XStateClass.Off);
			GKDriversHelper.AddAvailableStateClasses(driver, XStateClass.Test);

			GKDriversHelper.AddCommandBit(driver, GKStateBit.TurnOn_InManual);
			GKDriversHelper.AddCommandBit(driver, GKStateBit.TurnOnNow_InManual);
			GKDriversHelper.AddCommandBit(driver, GKStateBit.TurnOff_InManual);

			GKDriversHelper.AddIntProprety(driver, 0, "Задержка на включение, с", 0, 0, 65535);
			GKDriversHelper.AddIntProprety(driver, 1, "Порог КЗ", 5, 0, 65535);
			GKDriversHelper.AddIntProprety(driver, 2, "Порог ОБР", 45, 0, 65535);

			driver.MeasureParameters.Add(new GKMeasureParameter { No = 1, Name = "Отсчет задержки на включение, с", IsDelay = true, IsNotVisible = true });
			driver.MeasureParameters.Add(new GKMeasureParameter { No = 2, Name = "Отсчет удержания, с", IsDelay = true, IsNotVisible = true });
			driver.MeasureParameters.Add(new GKMeasureParameter() { No = 3, Name = "Сопротивление, Ом", InternalName = "Resistance" });
			driver.MeasureParameters.Add(new GKMeasureParameter { No = 4, Name = "Напряжение на конденсаторах, В", InternalName = "Voltage_1", Multiplier = 10 });
			driver.MeasureParameters.Add(new GKMeasureParameter { No = 5, Name = "Напряжение заряда, В", InternalName = "Voltage_2", Multiplier = 10 });
			return driver;
		}
	}
}