﻿using System;

namespace GlobalCF
{
	public class RSR2_MDU_Helper
	{
		public static GKDriver Create()
		{
			var driver = new GKDriver()
			{
				DriverTypeNo = 0xDC,
				DriverType = GKDriverType.RSR2_MDU,
				UID = new Guid("1BD3CDB3-7427-4FE8-9B56-22C14C9F5435"),
				Name = "Модуль автоматики дымоудаления",
				ShortName = "МДУ220",
				IsControlDevice = true,
				HasLogic = true,
				IsPlaceable = true,
				DriverClassification = GKDriver.DriverClassifications.ActuatingDevice
			};

			driver.AvailableStateBits.Add(GKStateBit.Norm);
			driver.AvailableStateBits.Add(GKStateBit.Off);
			driver.AvailableStateBits.Add(GKStateBit.TurningOn);
			driver.AvailableStateBits.Add(GKStateBit.TurningOff);
			driver.AvailableStateBits.Add(GKStateBit.Failure);
			GKDriversHelper.AddControlAvailableStates(driver);
			GKDriversHelper.AddAvailableStateClasses(driver, XStateClass.AutoOff);
			GKDriversHelper.AddAvailableStateClasses(driver, XStateClass.On);
			GKDriversHelper.AddAvailableStateClasses(driver, XStateClass.Off);
			GKDriversHelper.AddAvailableStateClasses(driver, XStateClass.TurningOn);
			GKDriversHelper.AddAvailableStateClasses(driver, XStateClass.TurningOff);
			GKDriversHelper.AddAvailableStateBits(driver, GKStateBit.Test);
			GKDriversHelper.AddAvailableStateClasses(driver, XStateClass.Test);

			GKDriversHelper.AddCommandBit(driver, GKStateBit.TurnOn_InManual);
			GKDriversHelper.AddCommandBit(driver, GKStateBit.TurnOnNow_InManual);
			GKDriversHelper.AddCommandBit(driver, GKStateBit.TurnOff_InManual);
			GKDriversHelper.AddCommandBit(driver, GKStateBit.Stop_InManual);

			var property1 = new GKDriverProperty()
			{
				No = 0,
				Name = "Задержка на включение, с",
				Caption = "Задержка на включение, с",
				Default = 0,
				DriverPropertyType = GKDriverPropertyTypeEnum.IntType,
				Min = 0,
				Max = 65535
			};
			driver.Properties.Add(property1);

			var property2 = new GKDriverProperty()
			{
				No = 1,
				Name = "Время включения, с",
				Caption = "Время включения, с",
				Default = 128,
				DriverPropertyType = GKDriverPropertyTypeEnum.IntType,
				Min = 1,
				Max = 65535
			};
			driver.Properties.Add(property2);

			var property3 = new GKDriverProperty()
			{
				No = 2,
				Name = "Время выключения, с",
				Caption = "Время выключения, с",
				Default = 128,
				DriverPropertyType = GKDriverPropertyTypeEnum.IntType,
				Min = 1,
				Max = 65535
			};
			driver.Properties.Add(property3);

			var property4 = new GKDriverProperty()
			{
				No = 3,
				Name = "Тип привода",
				Caption = "Тип привода",
				Default = 0,
				IsLowByte = true
			};
			var property4Parameter1 = new GKDriverPropertyParameter()
			{
				Name = "Реверсивный",
				Value = 0
			};
			var property4Parameter2 = new GKDriverPropertyParameter()
			{
				Name = "Пружинный",
				Value = 1
			};
			property4.Parameters.Add(property4Parameter1);
			property4.Parameters.Add(property4Parameter2);
			driver.Properties.Add(property4);

			var property5 = new GKDriverProperty()
			{
				No = 3,
				Name = "Контроль КВ НОРМА",
				Caption = "Контроль КВ НОРМА",
				Default = 0,
				IsHieghByte = true,
				Mask = 1
			};
			var property5Parameter1 = new GKDriverPropertyParameter()
			{
				Name = "С контролем",
				Value = 0
			};
			var property5Parameter2 = new GKDriverPropertyParameter()
			{
				Name = "Без контроля",
				Value = 1
			};
			property5.Parameters.Add(property5Parameter1);
			property5.Parameters.Add(property5Parameter2);
			driver.Properties.Add(property5);

			var property6 = new GKDriverProperty()
			{
				No = 3,
				Name = "Контроль КВ ЗАЩИТА",
				Caption = "Контроль КВ ЗАЩИТА",
				Default = 0,
				IsHieghByte = true,
				Mask = 2
			};
			var property6Parameter1 = new GKDriverPropertyParameter()
			{
				Name = "С контролем",
				Value = 0
			};
			var property6Parameter2 = new GKDriverPropertyParameter()
			{
				Name = "Без контроля",
				Value = 2
			};
			property6.Parameters.Add(property6Parameter1);
			property6.Parameters.Add(property6Parameter2);
			driver.Properties.Add(property6);

			var property7 = new GKDriverProperty()
			{
				No = 3,
				Name = "Контроль ДУ НОРМА",
				Caption = "Контроль ДУ НОРМА",
				Default = 0,
				IsHieghByte = true,
				Mask = 4
			};
			var property7Parameter1 = new GKDriverPropertyParameter()
			{
				Name = "С контролем",
				Value = 0
			};
			var property7Parameter2 = new GKDriverPropertyParameter()
			{
				Name = "Без контроля",
				Value = 4
			};
			property7.Parameters.Add(property7Parameter1);
			property7.Parameters.Add(property7Parameter2);
			driver.Properties.Add(property7);

			var property8 = new GKDriverProperty()
			{
				No = 3,
				Name = "Контроль ДУ ЗАЩИТА",
				Caption = "Контроль ДУ ЗАЩИТА",
				Default = 0,
				IsHieghByte = true,
				Mask = 8
			};
			var property8Parameter1 = new GKDriverPropertyParameter()
			{
				Name = "С контролем",
				Value = 0
			};
			var property8Parameter2 = new GKDriverPropertyParameter()
			{
				Name = "Без контроля",
				Value = 8
			};
			property8.Parameters.Add(property8Parameter1);
			property8.Parameters.Add(property8Parameter2);
			driver.Properties.Add(property8);

			return driver;
		}
	}
}