﻿using System;

namespace GlobalCF
{
	public static class RSR2_MTS_Helper
	{
		public static GKDriver Create()
		{
			var driver = new GKDriver()
			{
				DriverTypeNo = 0xD2,
				DriverType = GKDriverType.RSR2_MTS,
				UID = new Guid("7C329145-1619-4C28-8665-21054B2D95B4"),
				Name = "Модуль термосопротивления",
				ShortName = "МТС",
				IsPlaceable = true,
				DriverClassification = GKDriver.DriverClassifications.Other
			};

			GKDriversHelper.AddAvailableStateBits(driver, GKStateBit.Test);
			GKDriversHelper.AddAvailableStateBits(driver, GKStateBit.Fire1);
			GKDriversHelper.AddControlAvailableStates(driver);

			GKDriversHelper.AddAvailableStateClasses(driver, XStateClass.Test);
			GKDriversHelper.AddAvailableStateClasses(driver, XStateClass.Fire1);

			GKDriversHelper.AddIntProprety(driver, 0, "U АЛС в 0.1В", 120, 0, 65535);
			var property1 = new GKDriverProperty()
			{
				No = 1,
				Name = "Конфигурация",
				Caption = "Конфигурация",
				Default = 1,
				IsLowByte = true,
				Mask = 0x03
			};
			GKDriversHelper.AddPropertyParameter(property1, "Сработка на увеличение температуры", 0);
			GKDriversHelper.AddPropertyParameter(property1, "Сработка на уменьшение температуры", 1);
			driver.Properties.Add(property1);

			GKDriversHelper.AddIntProprety(driver, 2, "Обрыв, °C", -40, 0, 65535).DriverPropertyType = GKDriverPropertyTypeEnum.NegativeShortType;
			GKDriversHelper.AddIntProprety(driver, 3, "Сработка вх/вых, °C", 65, 0, 65535).DriverPropertyType = GKDriverPropertyTypeEnum.NegativeShortType;
			GKDriversHelper.AddIntProprety(driver, 4, "Сработка вых/вх, °C", 75, 0, 65535).DriverPropertyType = GKDriverPropertyTypeEnum.NegativeShortType;
			GKDriversHelper.AddIntProprety(driver, 5, "КЗ, °C", 150, 0, 65535).DriverPropertyType = GKDriverPropertyTypeEnum.NegativeShortType;

			driver.MeasureParameters.Add(new GKMeasureParameter { No = 1, Name = "t, °C", HasNegativeValue = true });
			driver.MeasureParameters.Add(new GKMeasureParameter { No = 2, Name = "U АЛС, B", Multiplier = 10});

			return driver;
		}
	}
}