﻿using System;
using System.Collections.Generic;

namespace GlobalCF
{
	public static class RSR2_MVP_Helper
	{
		public static GKDriver Create()
		{
			var driver = new GKDriver()
			{
				DriverTypeNo = 0xE5,
				DriverType = GKDriverType.RSR2_MVP,
				UID = new Guid("0B1BD00D-680B-4A80-AC9A-659FD7F85BB4"),
				Name = "Модуль ветвления и подпитки",
				ShortName = "МВП",
				IsPlaceable = true,
				AutoCreateChildren = new List<GKDriverType>() { GKDriverType.RSR2_MVP_Part },
				DriverClassification = GKDriver.DriverClassifications.Other
			};

			GKDriversHelper.AddAvailableStateBits(driver, GKStateBit.Test);
			GKDriversHelper.AddAvailableStateClasses(driver, XStateClass.Test);
			GKDriversHelper.AddCommandBit(driver, GKStateBit.TurnOn_InAutomatic, 0, "Включить АЛС1");
			GKDriversHelper.AddCommandBit(driver, GKStateBit.TurnOn_InAutomatic, 1, "Включить АЛС2");
			GKDriversHelper.AddCommandBit(driver, GKStateBit.TurnOn_InAutomatic, 2, "Включить АЛС3");
			GKDriversHelper.AddCommandBit(driver, GKStateBit.TurnOn_InAutomatic, 3, "Включить АЛС4");
			GKDriversHelper.AddCommandBit(driver, GKStateBit.TurnOff_InAutomatic, 0, "Выключить АЛС1");
			GKDriversHelper.AddCommandBit(driver, GKStateBit.TurnOff_InAutomatic, 1, "Выключить АЛС2");
			GKDriversHelper.AddCommandBit(driver, GKStateBit.TurnOff_InAutomatic, 2, "Выключить АЛС3");
			GKDriversHelper.AddCommandBit(driver, GKStateBit.TurnOff_InAutomatic, 3, "Выключить АЛС4");

			GKDriversHelper.AddIntProprety(driver, 0, "Число АУ на АЛС3 МВП", 0, 0, 250).CanNotEdit = true;
			GKDriversHelper.AddIntProprety(driver, 1, "Число АУ на АЛС4 МВП", 0, 0, 250).CanNotEdit = true;
			var property = new GKDriverProperty()
			{
				No = 2,
				Name = "Порог, В",
				Caption = "Порог, В",
				DriverPropertyType = GKDriverPropertyTypeEnum.IntType,
				Default = (ushort)180,
				Min = (ushort)80,
				Max = (ushort)700,
				Multiplier = 10
			};
			driver.Properties.Add(property);

			var property1 = new GKDriverProperty()
			{
				No = 3,
				Name = "Контроль питания",
				Caption = "Контроль питания",
				Default = 0,
				IsLowByte = true,
				Mask = 0x01
			};

			GKDriversHelper.AddPropertyParameter(property1, "С контролем питания", 0);
			GKDriversHelper.AddPropertyParameter(property1, "Без контроля питания", 1);
			driver.Properties.Add(property1);

			driver.MeasureParameters.Add(new GKMeasureParameter() { No = 1, Name = "U АЛC1, В", InternalName = "Voltage_1", Multiplier = 5 });
			driver.MeasureParameters.Add(new GKMeasureParameter() { No = 2, Name = "U АЛС2, В", InternalName = "Voltage_2", Multiplier = 5 });
			driver.MeasureParameters.Add(new GKMeasureParameter() { No = 3, Name = "U АЛС3, В", InternalName = "Voltage_3", Multiplier = 5 });
			driver.MeasureParameters.Add(new GKMeasureParameter() { No = 4, Name = "U АЛС4, В", InternalName = "Voltage_4", Multiplier = 5 });
			driver.MeasureParameters.Add(new GKMeasureParameter() { Name = "I АЛС1, мА" });
			driver.MeasureParameters.Add(new GKMeasureParameter() { Name = "I АЛС2, мА" });
			driver.MeasureParameters.Add(new GKMeasureParameter() { Name = "I АЛС3, мА" });
			driver.MeasureParameters.Add(new GKMeasureParameter() { Name = "I АЛС4, мА" });
			driver.MeasureParameters.Add(new GKMeasureParameter() { Name = "U , B", InternalName = "Voltage", Multiplier = 5 });

			return driver;
		}
	}
}