﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.Threading;
using System.Net;
using System.Net.Sockets;
using Newtonsoft.Json;
using System.Runtime.InteropServices;
using System.Xml;
using System.IO;
using System.IO.Ports;
using Microsoft.Win32;
using Modbus;
using System.Collections.ObjectModel;

namespace GlobalCF
{
    /// <summary>
    /// Логика взаимодействия для MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        public MsgConsoleCS MessengesConsole = new MsgConsoleCS();
        string CurrentTheme = "GrayTheme";
        int ALS1LeftLineCount = 0;
        int ALS1RightLineCount = 0;
        int ALS1TotalLineCount = 0;
        int ALS2LeftLineCount = 0;
        int ALS2RightLineCount = 0;
        int ALS2TotalLineCount = 0;
        GKDriverProperty SelectedProperty;
        GKDevice SelectedDevice;
        List<GKDevice> TotalDevices = new List<GKDevice>();
        List<GKDevice> ALS1RightSideDevices;
        List<GKDevice> ALS1LeftSideDevices;
        List<GKDevice> ALS2RightSideDevices;
        List<GKDevice> ALS2LeftSideDevices;

        bool DataRequestFlag = false;
        bool KAUConnectionCkeckFlag = false;

        ModBusTCPServerCS mdbTCPSrv = new ModBusTCPServerCS();
        ModBusRTUServerCS mdbRTUSrv = new ModBusRTUServerCS();
        MDBValue mdbValue;

        //public ObservableCollection<MessageCS> MessagesList = new ObservableCollection<MessageCS>();
        bool ConsoleUpdaterWork = true;
        object lockitems = new object();

        bool ROMsendingStarted = false;

        public MainWindow()
        {
            InitializeComponent();
            var themePath = "./Styles/" + CurrentTheme + ".xaml";
            Application.Current.Resources.MergedDictionaries.Add(new ResourceDictionary() { Source = new Uri(themePath, UriKind.Relative) });

            this.PropertiesList.SelectionChanged += PropertiesList_SelectionChanged;

            BindingOperations.EnableCollectionSynchronization(this.MessengesConsole.MessagesList, lockitems); //Включаем синхронизацию для межпоточной работы
            this.MessagesListView.ItemsSource = this.MessengesConsole.MessagesList;

            MessageCS startMsg = new MessageCS(DateTime.Now, "M", "Приложение", "Приложение запущено________________________________________________________________________________________________");
            this.AddNewMessage2Console(startMsg);
            this.ALS1treeviewListLeft.PreviewMouseDown += TreeviewListLeft_PreviewMouseDown;
            this.ALS1treeviewListLeft.SelectedItemChanged += TreeviewListLeft_SelectedItemChanged;
            this.ALS1treeviewListRight.PreviewMouseDown += TreeviewListRight_PreviewMouseDown;
            this.ALS1treeviewListRight.SelectedItemChanged += TreeviewListLeft_SelectedItemChanged;
            this.ALS2treeviewListLeft.PreviewMouseDown += TreeviewListLeft_PreviewMouseDown;
            this.ALS2treeviewListLeft.SelectedItemChanged += TreeviewListLeft_SelectedItemChanged;
            this.ALS2treeviewListRight.PreviewMouseDown += TreeviewListRight_PreviewMouseDown;
            this.ALS2treeviewListRight.SelectedItemChanged += TreeviewListLeft_SelectedItemChanged;
            this.Closing += MainWindow_Closing;
            this.CheckConfig();
            this.BaudRateCombo.SelectedIndex = 11;
            this.ParityCombo.SelectedIndex = 2;
            this.DataBitsCombo.SelectedIndex = 1;
            this.StopBitCombo.SelectedIndex = 1;
            this.RegistersList.SelectionChanged += RegistersList_SelectionChanged;
            this.MDBfunctionCmbBox.SelectedIndex = 2;
            string[] pnames = SerialPort.GetPortNames();
            foreach(string p in pnames)
                this.SerialPortNameCmbBox.Items.Add(p);
            this.MDBtcpRadioBtn.IsChecked = true;
            this.RSBaudRateCombo.SelectedIndex = 11;
            this.ExchngSpeedCmbBox.SelectedIndex = 2;
            //ThreadPool.QueueUserWorkItem(new WaitCallback(this.ConsoleUpdater), null);
        }

        private void ConsoleUpdater(object status)
        {
            MessageCS[] msgsList;
            while (this.ConsoleUpdaterWork)
            {
                if (this.MessengesConsole.MessagesList.Count != 0)
                {
                    lock (this.MessengesConsole.MessagesList)
                    {
                        msgsList = new MessageCS[this.MessengesConsole.MessagesList.Count];
                        this.MessengesConsole.MessagesList.CopyTo(msgsList, 0);
                        this.MessengesConsole.MessagesList.Clear();
                    }
                    //lock (this.MessagesList)
                    {
                        //Application.Current.Dispatcher.BeginInvoke(System.Windows.Threading.DispatcherPriority.Normal, new Action(() =>
                        //{
                            foreach (MessageCS m in msgsList)
                            {
                                //this.MessagesList.Insert(0, m);
                            }
                        //}));
                    }
                    /*Application.Current.Dispatcher.BeginInvoke(System.Windows.Threading.DispatcherPriority.Normal, new Action(() =>
                    {
                        this.MessagesListView.ItemsSource = this.MessagesList;
                        this.MessagesListView.Items.Refresh();
                    }));*/
                }
                Thread.Sleep(1000);
            }
        }

        private void MainWindow_Closing(object sender, System.ComponentModel.CancelEventArgs e)
        {
            this.DataRequestFlag = false;
            this.KAUConnectionCkeckFlag = false;
            this.ConsoleUpdaterWork = false;

            XmlDocument xDoc = new XmlDocument();
            XmlNode xRoot = xDoc.CreateNode(XmlNodeType.Element, "DEVICES", "");
            XmlNode xLeftNodes = xDoc.CreateNode(XmlNodeType.Element, "ALS1LEFTSIDE", "");
            XmlNode xRightNodes = xDoc.CreateNode(XmlNodeType.Element, "ALS1RIGHTSIDE", "");
            if ((this.ALS1LeftSideDevices != null) && (this.ALS1LeftSideDevices.Count != 0))
            {
                foreach (GKDevice d in this.ALS1LeftSideDevices)
                {
                    xLeftNodes.AppendChild(d.ToXML(xDoc));
                }
            }
            if ((this.ALS1RightSideDevices != null) && (this.ALS1RightSideDevices.Count != 0))
            {
                foreach (GKDevice d in this.ALS1RightSideDevices)
                {
                    xRightNodes.AppendChild(d.ToXML(xDoc));
                }
            }
            xRoot.AppendChild(xLeftNodes);
            xRoot.AppendChild(xRightNodes);
            xDoc.AppendChild(xRoot);
            xLeftNodes = xDoc.CreateNode(XmlNodeType.Element, "ALS2LEFTSIDE", "");
            xRightNodes = xDoc.CreateNode(XmlNodeType.Element, "ALS2RIGHTSIDE", "");
            if ((this.ALS2LeftSideDevices != null) && (this.ALS2LeftSideDevices.Count != 0))
            {
                foreach (GKDevice d in this.ALS2LeftSideDevices)
                {
                    xLeftNodes.AppendChild(d.ToXML(xDoc));
                }
            }
            if ((this.ALS2RightSideDevices != null) && (this.ALS2RightSideDevices.Count != 0))
            {
                foreach (GKDevice d in this.ALS2RightSideDevices)
                {
                    xRightNodes.AppendChild(d.ToXML(xDoc));
                }
            }
            xRoot.AppendChild(xLeftNodes);
            xRoot.AppendChild(xRightNodes);
            xDoc.AppendChild(xRoot);
            xDoc.Save("GlobalCF.cnfg");

            if (this.mdbTCPSrv != null)
                this.mdbTCPSrv.WorkerFlag = false;
            if (this.mdbRTUSrv != null)
                this.mdbRTUSrv.WorkerFlag = false;

            MessageCS startMsg = new MessageCS(DateTime.Now, "M", "Приложение", "Приложение остановлено");
            this.AddNewMessage2Console(startMsg);

            #region Messages saving
            if (!Directory.Exists(AppDomain.CurrentDomain.BaseDirectory + "\\MessengesCosole"))
            {
                Directory.CreateDirectory(AppDomain.CurrentDomain.BaseDirectory + "\\MessengesCosole");
            }
            this.MessengesConsole.Save(AppDomain.CurrentDomain.BaseDirectory + "\\MessengesCosole");
            //this.SaveMsgs(AppDomain.CurrentDomain.BaseDirectory + "\\MessengesCosole");
            #endregion
        }
        public void SaveMsgs(string LogPath)
        {
            /*string FIleName = System.IO.Path.Combine(LogPath, DateTime.Now.ToString("yyyyMMdd") + "_msgs.gcflog");
            StreamWriter sw = new StreamWriter(FIleName, true);
            for (int i = 0; i < this.MessagesList.Count; i++)
            {
                MessageCS msg = this.MessagesList[i];
                sw.WriteLine(msg.ToString());
            }
            sw.Flush();
            sw.Close();*/
        }
        private void CheckConfig()
        {
            XmlDocument xDoc = new XmlDocument();
            try
            {
                xDoc.Load("GlobalCF.cnfg");
                MessageBoxResult result = MessageBox.Show("Обнаружен файл конфигурации. Загрузить предыдущую конфигурацию?", "", MessageBoxButton.YesNo);
                if (result == MessageBoxResult.Yes)
                {
                    foreach (XmlNode xNode in xDoc.DocumentElement.ChildNodes)
                    {
                        if (xNode.Name == "ALS1LEFTSIDE")
                        {
                            this.ALS1LeftSideDevices = new List<GKDevice>();
                            foreach (XmlNode xNd in xNode.ChildNodes)
                            {
                                GKDevice d = new GKDevice();
                                d.FromXML(xNd);
                                this.ALS1LeftSideDevices.Add(d);
                                this.ALS1treeviewListLeft.Items.Add(d.GetTree());
                            }
                        }
                        if (xNode.Name == "ALS1RIGHTSIDE")
                        {
                            this.ALS1RightSideDevices = new List<GKDevice>();
                            foreach (XmlNode xNd in xNode.ChildNodes)
                            {
                                GKDevice d = new GKDevice();
                                d.FromXML(xNd);
                                this.ALS1RightSideDevices.Add(d);
                                this.ALS1treeviewListRight.Items.Add(d.GetTree());
                            }
                        }
                        if (xNode.Name == "ALS2LEFTSIDE")
                        {
                            this.ALS2LeftSideDevices = new List<GKDevice>();
                            foreach (XmlNode xNd in xNode.ChildNodes)
                            {
                                GKDevice d = new GKDevice();
                                d.FromXML(xNd);
                                this.ALS2LeftSideDevices.Add(d);
                                this.ALS2treeviewListLeft.Items.Add(d.GetTree());
                            }
                        }
                        if (xNode.Name == "ALS2RIGHTSIDE")
                        {
                            this.ALS2RightSideDevices = new List<GKDevice>();
                            foreach (XmlNode xNd in xNode.ChildNodes)
                            {
                                GKDevice d = new GKDevice();
                                d.FromXML(xNd);
                                this.ALS2RightSideDevices.Add(d);
                                this.ALS2treeviewListRight.Items.Add(d.GetTree());
                            }
                        }
                    }
                    this.CompareDevicesLists();
                }
            }
            catch { }
        }

        private void TreeviewListRight_PreviewMouseDown(object sender, MouseButtonEventArgs e)
        {
        }
        private void TreeviewListLeft_PreviewMouseDown(object sender, MouseButtonEventArgs e)
        {
        }
        private void TreeviewListLeft_SelectedItemChanged(object sender, RoutedPropertyChangedEventArgs<object> e)
        {
            if (e.NewValue != null)
            {
                this.SelectedDevice = (GKDevice)((TreeViewItem)e.NewValue).Tag;
                //this.ControlBoxContent.Children.Clear();

                this.Interface();

                this.PropertiesList.ItemsSource = this.SelectedDevice.Driver.Properties;

                Binding bind1 = new Binding();
                bind1.Source = this.SelectedDevice;
                bind1.Path = new PropertyPath("TestState");
                bind1.Mode = BindingMode.TwoWay;
                this.TestFlagChkBox.SetBinding(CheckBox.IsCheckedProperty, bind1);
                //this.TestFlagChkBox.IsChecked = ((GKDevice)e.AddedItems[0]).TestState;

                Binding bind2 = new Binding();
                bind2.Source = this.SelectedDevice;
                bind2.Path = new PropertyPath("FailureState");
                bind2.Mode = BindingMode.TwoWay;
                this.FailureFlagChkBox.SetBinding(CheckBox.IsCheckedProperty, bind2);
                //this.FailureFlagChkBox.IsChecked = ((GKDevice)e.AddedItems[0]).FailureState;

                Binding bind4 = new Binding();
                bind4.Source = this.SelectedDevice;
                bind4.Path = new PropertyPath("Mode1");
                bind4.Mode = BindingMode.TwoWay;
                this.Mode1ChkBox.SetBinding(CheckBox.IsCheckedProperty, bind4);
                //this.Mode1ChkBox.IsChecked= ((GKDevice)e.AddedItems[0]).Mode1;

                Binding bind5 = new Binding();
                bind5.Source = this.SelectedDevice;
                bind5.Path = new PropertyPath("Mode2");
                bind5.Mode = BindingMode.TwoWay;
                this.Mode2ChkBox.SetBinding(CheckBox.IsCheckedProperty, bind5);
                //this.Mode2ChkBox.IsChecked = ((GKDevice)e.AddedItems[0]).Mode2;

                //Mode
                Binding bindMode = new Binding();
                bindMode.Source = this.SelectedDevice;
                bindMode.Path = new PropertyPath("Mode");
                bindMode.Mode = BindingMode.TwoWay;
                this.ModeComboBox.SetBinding(ComboBox.SelectedIndexProperty, bindMode);

                Binding bind6 = new Binding();
                bind6.Source = this.SelectedDevice;
                bind6.Path = new PropertyPath("Fire1");
                bind6.Mode = BindingMode.TwoWay;
                this.Fire1ChkBox.SetBinding(CheckBox.IsCheckedProperty, bind6);
                //this.Fire1ChkBox.IsChecked = ((GKDevice)e.AddedItems[0]).Fire1;

                Binding bind7 = new Binding();
                bind7.Source = this.SelectedDevice;
                bind7.Path = new PropertyPath("Fire2");
                bind7.Mode = BindingMode.TwoWay;
                this.Fire2ChkBox.SetBinding(CheckBox.IsCheckedProperty, bind7);
                //this.Fire2ChkBox.IsChecked = ((GKDevice)e.AddedItems[0]).Fire2;

                Binding bind8 = new Binding();
                bind8.Source = this.SelectedDevice;
                bind8.Path = new PropertyPath("OnState");
                bind8.Mode = BindingMode.TwoWay;
                this.OnFlagChkBox.SetBinding(CheckBox.IsCheckedProperty, bind8);

                Binding bind9 = new Binding();
                bind9.Source = this.SelectedDevice;
                bind9.Path = new PropertyPath("OffState");
                bind9.Mode = BindingMode.TwoWay;
                this.OffFlagChkBox.SetBinding(CheckBox.IsCheckedProperty, bind9);

                Binding bind10 = new Binding();
                bind10.Source = this.SelectedDevice;
                bind10.Path = new PropertyPath("AddCCstate");
                bind10.Mode = BindingMode.TwoWay;
                this.AddCCstate.SetBinding(Label.ContentProperty, bind10);

                Binding bind11 = new Binding();
                bind11.Source = this.SelectedDevice;
                bind11.Path = new PropertyPath("OnAction");
                bind11.Mode = BindingMode.TwoWay;
                this.OnActionChkBox.SetBinding(CheckBox.IsCheckedProperty, bind11);

                Binding bind12 = new Binding();
                bind12.Source = this.SelectedDevice;
                bind12.Path = new PropertyPath("OffAction");
                bind12.Mode = BindingMode.TwoWay;
                this.OffActionChkBox.SetBinding(CheckBox.IsCheckedProperty, bind12);

                this.SetVisibilityForCommzndButtons();
                /*if (this.SelectedDevice.Driver.AvailableCommandBits.Count != 0)
                {
                    this.ControlGroupBox.Visibility = Visibility.Visible;
                    foreach (CommandBit cmd in this.SelectedDevice.Driver.AvailableCommandBits)
                    {
                        Button btn = new Button();
                        btn.Content = cmd.Name;
                        btn.Tag = cmd.GKStateBit;

                        btn.Click += Btn_Click;
                        this.ControlBoxContent.Children.Add(btn);
                    }
                }
                else
                {
                    //this.SelectedDevice = null;
                    this.ControlGroupBox.Visibility = Visibility.Collapsed;
                }*/
                if (this.SelectedDevice.DataFromDevice != null)
                {
                    this.DataFromDevice.Text = String.Join(", ", this.SelectedDevice.DataFromDevice.data);
                }
            }
        }

        private void Interface()
        {
            this.Mode1ChkBox.Visibility = Visibility.Collapsed;
            this.Bit1CCChkBox.Visibility = Visibility.Collapsed;
            this.Fire1ChkBox.Visibility = Visibility.Collapsed;
            this.Fire2ChkBox.Visibility = Visibility.Collapsed;
            this.TestFlagChkBox.Visibility = Visibility.Collapsed;
            this.FailureFlagChkBox.Visibility = Visibility.Collapsed;
            this.OnFlagChkBox.Visibility = Visibility.Collapsed;
            this.OffFlagChkBox.Visibility = Visibility.Collapsed;
            this.OnActionChkBox.Visibility = Visibility.Collapsed;
            this.OffActionChkBox.Visibility = Visibility.Collapsed;
            foreach (byte k in this.SelectedDevice.StateWord1.Keys)
            {
                string StText = this.SelectedDevice.StateWord1[k];
                switch (k)
                {
                    case 0: //Mode1
                        //this.Mode1ChkBox.Visibility = Visibility.Visible;
                        //if (StText != "") this.Mode1ChkBox.Content = StText;
                        break;
                    case 1: //Bit1
                        //this.Bit1CCChkBox.Visibility = Visibility.Visible;
                        //if (StText != "") this.Bit1CCChkBox.Content = StText;
                        break;
                    case 2: //Fire1
                        this.Fire1ChkBox.Visibility = Visibility.Visible;
                        if (StText != "") this.Fire1ChkBox.Content = StText;
                        break;
                    case 3: //Fire2
                        this.Fire2ChkBox.Visibility = Visibility.Visible;
                        if (StText != "") this.Fire2ChkBox.Content = StText;
                        break;
                    case 4: //TestFlag
                        this.TestFlagChkBox.Visibility = Visibility.Visible;
                        if (StText != "") this.TestFlagChkBox.Content = StText;
                        break;
                    case 5: //FailureFlag
                        this.FailureFlagChkBox.Visibility = Visibility.Visible;
                        if (StText != "") this.FailureFlagChkBox.Content = StText;
                        break;
                    case 7: //OnFlag
                        this.OnFlagChkBox.Visibility = Visibility.Visible;
                        if (StText != "") this.OnFlagChkBox.Content = StText;
                        break;
                    case 8: //OffFlag
                        this.OffFlagChkBox.Visibility = Visibility.Visible;
                        if (StText != "") this.OffFlagChkBox.Content = StText;
                        break;
                    case 9: //OnAction
                        this.OnActionChkBox.Visibility = Visibility.Visible;
                        if (StText != "") this.OnActionChkBox.Content = StText;
                        break;
                    case 10: //OffAction
                        this.OffActionChkBox.Visibility = Visibility.Visible;
                        if (StText != "") this.OffActionChkBox.Content = StText;
                        break;
                }
            }
            foreach (byte k in this.SelectedDevice.StateWord2.Keys)
            {
                string StText = this.SelectedDevice.StateWord2[k];
                switch (k)
                {
                    /*case 0:
                        //this.Bit0ChkBox.Visibility = Visibility.Visible;
                        //this.Bit0ChkBox.Content = StText;
                        break;
                    case 1:
                        //this.Bit1ChkBox.Visibility = Visibility.Visible;
                        //this.Bit1ChkBox.Content = StText;
                        break;
                    case 2:
                        this.Fire1ChkBox.Visibility = Visibility.Visible;
                        this.Fire1ChkBox.Content = StText;
                        break;
                    case 3:
                        this.Fire2ChkBox.Visibility = Visibility.Visible;
                        this.Fire2ChkBox.Content = StText;
                        break;
                    case 4:
                        this.TestFlagChkBox.Visibility = Visibility.Visible;
                        this.TestFlagChkBox.Content = StText;
                        break;
                    case 5:
                        this.FailureFlagChkBox.Visibility = Visibility.Visible;
                        this.FailureFlagChkBox.Content = StText;
                        break;*/
                    case 6:
                        //this.Bit6ChkBox.Visibility = Visibility.Visible;
                        //this.Bit6ChkBox.Content = StText;
                        break;
                    case 7:
                        this.OnFlagChkBox.Visibility = Visibility.Visible;
                        this.OnFlagChkBox.Content = StText;
                        break;
                    case 8:
                        this.OffFlagChkBox.Visibility = Visibility.Visible;
                        this.OffFlagChkBox.Content = StText;
                        break;
                    case 9:
                        this.OnActionChkBox.Visibility = Visibility.Visible;
                        this.OnActionChkBox.Content = StText;
                        break;
                    case 10:
                        this.OffActionChkBox.Visibility = Visibility.Visible;
                        this.OffActionChkBox.Content = StText;
                        break;
                    case 11:
                        //this.Bit11ChkBox.Visibility = Visibility.Visible;
                        //this.Bit11ChkBox.Content = StText;
                        break;
                    case 12:
                        //this.Bit12ChkBox.Visibility = Visibility.Visible;
                        //this.Bit12ChkBox.Content = StText;
                        break;
                    case 13:
                        //this.Bit13ChkBox.Visibility = Visibility.Visible;
                        //this.Bit13ChkBox.Content = StText;
                        break;
                    case 14:
                        //this.Bit14ChkBox.Visibility = Visibility.Visible;
                        //this.Bit14ChkBox.Content = StText;
                        break;
                    case 15:
                        //this.Bit15ChkBox.Visibility = Visibility.Visible;
                        //this.Bit15ChkBox.Content = StText;
                        break;
                }
            }
            foreach (byte k in this.SelectedDevice.CMDControls.Keys)
            {
                string StText = this.SelectedDevice.CMDControls[k];
                switch (k)
                {
                    case 0:
                        this.Cntr0_Btn.Visibility = Visibility.Visible;
                        if (StText != "") this.Cntr0_Btn.Content = StText;
                        break;
                    case 1:
                        this.Cntr1_Btn.Visibility = Visibility.Visible;
                        if (StText != "") this.Cntr1_Btn.Content = StText;
                        break;
                    case 2:
                        this.Cntr2_Btn.Visibility = Visibility.Visible;
                        if (StText != "") this.Cntr2_Btn.Content = StText;
                        break;
                    case 3:
                        this.Cntr3_Btn.Visibility = Visibility.Visible;
                        if (StText != "") this.Cntr3_Btn.Content = StText;
                        break;
                    case 4:
                        this.Cntr4_Btn.Visibility = Visibility.Visible;
                        if (StText != "") this.Cntr4_Btn.Content = StText;
                        break;
                    case 5:
                        //this.Cntr5_Btn.Visibility = Visibility.Visible;
                        //if (StText != "") this.Cntr5_Btn.Content = StText;
                        break;
                    case 6:
                        //this.Cntr6_Btn.Visibility = Visibility.Visible;
                        //if (StText != "") this.Cntr6_Btn.Content = StText;
                        break;
                    case 7:
                        this.Cntr7_Btn.Visibility = Visibility.Visible;
                        if (StText != "") this.Cntr7_Btn.Content = StText;
                        break;
                    case 8:
                        this.Cntr8_Btn.Visibility = Visibility.Visible;
                        if (StText != "") this.Cntr8_Btn.Content = StText;
                        break;
                    case 9:
                        this.Cntr9_Btn.Visibility = Visibility.Visible;
                        if (StText != "") this.Cntr9_Btn.Content = StText;
                        break;
                    case 10:
                        this.Cntr10_Btn.Visibility = Visibility.Visible;
                        if (StText != "") this.Cntr10_Btn.Content = StText;
                        break;
                    case 11:
                        this.Cntr11_Btn.Visibility = Visibility.Visible;
                        if (StText != "") this.Cntr11_Btn.Content = StText;
                        break;
                    case 12:
                        this.Cntr12_Btn.Visibility = Visibility.Visible;
                        if (StText != "") this.Cntr12_Btn.Content = StText;
                        break;
                    case 13:
                        this.Cntr13_Btn.Visibility = Visibility.Visible;
                        if (StText != "") this.Cntr13_Btn.Content = StText;
                        break;
                    case 14:
                        this.Cntr14_Btn.Visibility = Visibility.Visible;
                        if (StText != "") this.Cntr14_Btn.Content = StText;
                        break;
                    case 15:
                        this.Cntr15_Btn.Visibility = Visibility.Visible;
                        if (StText != "") this.Cntr15_Btn.Content = StText;
                        break;
                }
            }
        }
        private void PropertiesList_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            if (e.AddedItems.Count != 0)
            {
                SelectedProperty = ((GKDriverProperty)e.AddedItems[0]);
                if ((SelectedProperty.DriverPropertyType == GKDriverPropertyTypeEnum.IntType) || (SelectedProperty.DriverPropertyType == GKDriverPropertyTypeEnum.NegativeShortType)
                    || (SelectedProperty.DriverPropertyType == GKDriverPropertyTypeEnum.StringType))
                {
                    this.ValueTxtBox.Visibility = Visibility.Visible;
                    this.ValueCmbBox.Visibility = Visibility.Collapsed;
                    //this.ValueTxtBox.Text = SelectedProperty.Default.ToString();
                    //DisplayValue
                    Binding bindDV = new Binding();
                    bindDV.Source = SelectedProperty;
                    bindDV.Path = new PropertyPath("DisplayValue");
                    bindDV.Mode = BindingMode.TwoWay;
                    this.ValueTxtBox.SetBinding(TextBox.TextProperty, bindDV);
                }
                if ((SelectedProperty.DriverPropertyType == GKDriverPropertyTypeEnum.BoolType) || (SelectedProperty.DriverPropertyType == GKDriverPropertyTypeEnum.EnumType))
                {
                    this.ValueCmbBox.Items.Clear();
                    foreach (GKDriverPropertyParameter pp in SelectedProperty.Parameters)
                    {
                        this.ValueCmbBox.Items.Add(pp.Name);
                    }
                    foreach (GKDriverPropertyParameter p in this.SelectedProperty.Parameters)
                    {
                        if (p.Value == this.SelectedProperty.Default)
                            this.ValueCmbBox.SelectedItem = this.SelectedProperty.Parameters[this.SelectedProperty.Parameters.IndexOf(p)].Name;
                    }
                    this.ValueTxtBox.Visibility = Visibility.Collapsed;
                    this.ValueCmbBox.Visibility = Visibility.Visible;
                }
            }
        }

        private void Btn_Click(object sender, RoutedEventArgs e)
        {
            byte alsn = 0;
            if (this.ALSTabControl.SelectedIndex == 0)
                alsn = 1;
            else
                alsn = 2;

            if (this.SelectedDevice != null)
            {
                SendManager.WriteDeviceCommand(4, this.SelectedDevice, alsn, ((GKStateBit)((Button)sender).Tag));
            }
        }
        //Сканирование всех линий АЛС
        private void ScanAllLines_Click(object sender, RoutedEventArgs e)
        {
            //if (this.KAUConnectionCkeckFlag)
            {
                //this.KAUConnectionCkeckFlag = false;
                //this.ScanAllLines.Visibility = Visibility.Visible;
            }
            //else
            {
                //this.ScanAllLines.Visibility = Visibility.Collapsed;
                //this.KAUConnectionCkeckFlag = true;
                //while ((this.ALS1treeviewListRight.Items.Count != 0) && (this.ALS1treeviewListLeft.Items.Count != 0) && (this.ALS2treeviewListRight.Items.Count != 0) && (this.ALS2treeviewListLeft.Items.Count != 0))
                //{
                    //Application.Current.Dispatcher.BeginInvoke(System.Windows.Threading.DispatcherPriority.Normal, new Action(() =>
                    //{
                    this.ALS1treeviewListRight.Items.Clear();
                    //this.ALS1treeviewListRight.Items.Refresh();
                    this.ALS1treeviewListLeft.Items.Clear();
                    //this.ALS1treeviewListLeft.InvalidateVisual();
                    this.ALS2treeviewListRight.Items.Clear();
                    this.ALS2treeviewListLeft.Items.Clear();
                    //}));
                //}
                //Application.Current.Dispatcher.BeginInvoke(System.Windows.Threading.DispatcherPriority.Normal, new Action(() =>
                //{
                this.ScanAll();
                //}));
            }
        }

        private void ScanAll() 
        {
            {
                this.KAUinfoLabel.Content = SendManager.GetDeviceInfo(4);
                ThreadPool.QueueUserWorkItem(new WaitCallback(this.BackgroundKAUConnectChecker), null);

                this.AddNewMessage2Console(new MessageCS(DateTime.Now, "C", "Пользователь", "Запуск сканирования линий"));

                var result = SendManager.Ping(4);
                if (result.HasError)
                {
                    this.AddNewMessage2Console(new MessageCS(DateTime.Now, "W", "Система", "Не удается подключиться к МОПС с адресом " + this.IPaddressTxtBox.Text));
                    MessageBox.Show("Не удается подключиться к МОПС с адресом " + this.IPaddressTxtBox.Text, "ОШИБКА ПОДКЛЮЧЕНИЯ!");
                    return;
                }
                List<GKDevice> devs = new List<GKDevice>();
                try
                {
                    //Отправляем Команду №0 для инициализации сканирования всех устройств на всех линиях
                    SendManager.ScanDevices(0, devs);
                    //Ждем 5 секунд перед запросом пакета данных по результатам сканирования линий
                    Thread.Sleep(10000);
                    //Сканируем АЛС1-основную (1)
                    try
                    {
                        this.ALS1LeftSideDevices = new List<GKDevice>();
                        SendManager.ScanDevices(1, this.ALS1LeftSideDevices);
                        this.ALS1treeviewListLeft.Items.Clear();
                        foreach (GKDevice d in this.ALS1LeftSideDevices)
                        {
                            this.ALS1treeviewListLeft.Items.Add(d.GetTree());
                        }
                    }
                    catch (Exception ex)
                    {
                        this.CompareDevicesLists();
                        this.AddNewMessage2Console(new MessageCS(DateTime.Now, "A", "Система", ex.ToString()));
                        MessageBox.Show(ex.ToString(), "ОШИБКА!");
                    }
                    //Сканируем АЛС1-резервную (2)
                    try
                    {
                        this.ALS1RightSideDevices = new List<GKDevice>();
                        SendManager.ScanDevices(2, this.ALS1RightSideDevices);
                        this.ALS1treeviewListRight.Items.Clear();
                        foreach (GKDevice d in this.ALS1RightSideDevices)
                        {
                            this.ALS1treeviewListRight.Items.Add(d.GetTree());
                        }
                    }
                    catch (Exception ex)
                    {
                        this.CompareDevicesLists();
                        this.AddNewMessage2Console(new MessageCS(DateTime.Now, "A", "Система", ex.ToString()));
                        MessageBox.Show(ex.ToString(), "ОШИБКА!");
                    }

                    //Сканируем АЛС2-основную (3)
                    try
                    {
                        this.ALS2LeftSideDevices = new List<GKDevice>();
                        SendManager.ScanDevices(3, this.ALS2LeftSideDevices);
                        this.ALS2treeviewListLeft.Items.Clear();
                        foreach (GKDevice d in this.ALS2LeftSideDevices)
                        {
                            this.ALS2treeviewListLeft.Items.Add(d.GetTree());
                        }
                    }
                    catch (Exception ex)
                    {
                        this.CompareDevicesLists();
                        this.AddNewMessage2Console(new MessageCS(DateTime.Now, "A", "Система", ex.ToString()));
                        MessageBox.Show(ex.ToString(), "ОШИБКА!");
                    }
                    //Сканируем АЛС2-резервную (4)
                    try
                    {
                        this.ALS2RightSideDevices = new List<GKDevice>();
                        SendManager.ScanDevices(4, this.ALS2RightSideDevices);
                        this.ALS2treeviewListRight.Items.Clear();
                        foreach (GKDevice d in this.ALS2RightSideDevices)
                        {
                            this.ALS2treeviewListRight.Items.Add(d.GetTree());
                        }
                    }
                    catch (Exception ex)
                    {
                        this.CompareDevicesLists();
                        this.AddNewMessage2Console(new MessageCS(DateTime.Now, "A", "Система", ex.ToString()));
                        MessageBox.Show(ex.ToString(), "ОШИБКА!");
                    }
                    this.CompareDevicesLists();
                }
                catch (Exception ex)
                {
                    this.CompareDevicesLists();
                    this.AddNewMessage2Console(new MessageCS(DateTime.Now, "A", "Система", ex.ToString()));
                    MessageBox.Show(ex.ToString(), "ОШИБКА!");
                }
                this.SettingsAttantionLabel.Visibility = Visibility.Visible;
                MessageBox.Show("Сканирование завершено.", "Сканирование линий АЛС");
            }
        }
        //Сравнить ветки АЛС на предмет несоответствий по линиям АЛС и модулям АУ
        private void CompareDevicesLists()
        {
            #region ALS1
            if ((this.ALS1LeftSideDevices.Count == 0) && (this.ALS1RightSideDevices.Count == 0))
            {
                this.ALS1TotalDeviceNumbers.Content = "АЛС1:";
            }
            else
            {
                Dictionary<uint, GKDevice> ALS1DevDictionary = new Dictionary<uint, GKDevice>();
                int ALS1ErrorCounter = 0;
                foreach (GKDevice d in this.ALS1LeftSideDevices)
                {
                    if (!ALS1DevDictionary.ContainsKey(d.SerialNo))
                        ALS1DevDictionary.Add(d.SerialNo, d);
                }
                this.ALS1LeftLineCount = ALS1DevDictionary.Count;
                this.ALS1LeftGrpBox.Header = "Основная линия (" + this.ALS1LeftLineCount + ")";

                Dictionary<uint, GKDevice> RDevDictionary = new Dictionary<uint, GKDevice>();
                foreach (GKDevice d in this.ALS1RightSideDevices)
                {
                    if (!RDevDictionary.ContainsKey(d.SerialNo))
                        RDevDictionary.Add(d.SerialNo, d);
                }
                this.ALS1RightLineCount = RDevDictionary.Count;
                this.ALS1RightGrpBox.Header = "Резервная линия (" + this.ALS1RightLineCount + ")";

                foreach (GKDevice d in this.ALS1RightSideDevices)
                {
                    if (!ALS1DevDictionary.ContainsKey(d.SerialNo))
                    {
                        ALS1ErrorCounter++;
                    }
                }
                if (!((ALS1ErrorCounter == 0) && (this.ALS1LeftLineCount != 0) && (this.ALS1RightLineCount != 0)))
                {
                    ALS1ErrorCounter++;
                }
                if (ALS1ErrorCounter == 0)
                {
                    this.ALS1TotalDeviceNumbers.Foreground = Brushes.LightGreen;
                    this.ALS1TotalDeviceNumbers.Content = "АЛС1: НОРМА! Списки устройств соответствуют.";
                }
                else
                {
                    this.ALS1TotalDeviceNumbers.Foreground = Brushes.Red;
                    this.ALS1TotalDeviceNumbers.Content = "АЛС1: ОШИБКА! Расхождение по устройствам между линиями!";
                }
            }
            #endregion
            #region ALS2
            if ((this.ALS2LeftSideDevices.Count == 0) && (this.ALS2RightSideDevices.Count == 0))
            {
                this.ALS2TotalDeviceNumbers.Content = "АЛС2:";
            }
            else
            {
                Dictionary<uint, GKDevice> ALS2DevDictionary = new Dictionary<uint, GKDevice>();
                int ALS2ErrorCounter = 0;
                foreach (GKDevice d in this.ALS2LeftSideDevices)
                {
                    if (!ALS2DevDictionary.ContainsKey(d.SerialNo))
                        ALS2DevDictionary.Add(d.SerialNo, d);
                }
                this.ALS2LeftLineCount = ALS2DevDictionary.Count;
                this.ALS2LeftGrpBox.Header = "Основная линия (" + this.ALS2LeftLineCount + ")";

                Dictionary<uint, GKDevice> ALS2RDevDictionary = new Dictionary<uint, GKDevice>();
                foreach (GKDevice d in this.ALS2RightSideDevices)
                {
                    if (!ALS2RDevDictionary.ContainsKey(d.SerialNo))
                        ALS2RDevDictionary.Add(d.SerialNo, d);
                }
                this.ALS2RightLineCount = ALS2RDevDictionary.Count;
                this.ALS2RightGrpBox.Header = "Резервная линия (" + this.ALS2RightLineCount + ")";

                foreach (GKDevice d in this.ALS2RightSideDevices)
                {
                    if (!ALS2DevDictionary.ContainsKey(d.SerialNo))
                    {
                        ALS2ErrorCounter++;
                    }
                }
                if (!((ALS2ErrorCounter == 0) && (this.ALS2LeftLineCount != 0) && (this.ALS2RightLineCount != 0)))
                {
                    ALS2ErrorCounter++;
                }
                if (ALS2ErrorCounter == 0)
                {
                    this.ALS2TotalDeviceNumbers.Foreground = Brushes.LightGreen;
                    this.ALS2TotalDeviceNumbers.Content = "АЛС2: НОРМА! Списки устройств соответствуют.";
                }
                else
                {
                    this.ALS2TotalDeviceNumbers.Foreground = Brushes.Red;
                    this.ALS2TotalDeviceNumbers.Content = "АЛС2: ОШИБКА! Расхождение по устройствам между линиями!";
                }
            }
            #endregion
        }

        private void ValueTxtBox_TextChanged(object sender, TextChangedEventArgs e)
        {
            if (this.SelectedProperty != null)
            {
                try
                {
                    this.SelectedProperty.Default = ushort.Parse(this.ValueTxtBox.Text);
                }
                catch { }

                this.PropertiesList.Items.Refresh();
            }
        }
        private void ValueCmbBox_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            if ((this.SelectedProperty != null) && (e.AddedItems.Count > 0))
            {
                try
                {
                    this.SelectedProperty.Default = (ushort)this.SelectedProperty.Parameters[this.ValueCmbBox.Items.IndexOf(e.AddedItems[0])].Value;
                }
                catch { }

                this.PropertiesList.Items.Refresh();
            }
        }
        private void IPaddressTxtBox_TextChanged(object sender, TextChangedEventArgs e)
        {
            SendManager.ipAddress = this.IPaddressTxtBox.Text;
        }

        //Запустить алгоритм передачи конфигурации по всем АУ всех АЛС в МОПС по UDP
        private void SaveSettingsIntoDevice_Click(object sender, RoutedEventArgs e)
        {
            this.AddNewMessage2Console(new MessageCS(DateTime.Now, "C", "Пользователь", "Запись параметров в устройства"));
            SendManager.GoToTechnologicalRegime(4);
            //Thread.Sleep(5000);
            if (SendManager.IsInTechnologicalRegime(4))
            {
                this.WriteConfigToGK();
                this.IntoWorkModeBtn_Click(sender, e);
            }
            else
            {
                SendManager.GoToTechnologicalRegime(4);
                //Thread.Sleep(5000);
                if (SendManager.IsInTechnologicalRegime(4))
                {
                    this.WriteConfigToGK();
                    this.IntoWorkModeBtn_Click(sender, e);
                }
                else
                {
                    this.AddNewMessage2Console(new MessageCS(DateTime.Now, "W", "Система", "Не удалось перевести устройство в технологический режим. Попробуйте еще раз."));
                    MessageBox.Show("Не удалось перевести устройство в технологический режим. Попробуйте еще раз.", "ПЕРЕХОД В ТЕХНОЛОГИЧЕСКИЙ РЕЖИМ");
                    return;
                }
            }
            while (!SendManager.GetPollingStatus())
            {
                Thread.Sleep(500);
            }
            this.SettingsAttantionLabel.Visibility = Visibility.Collapsed; 
            MessageBox.Show("Успешно завершено.", "Запись параметров в АУ");
        }
        //Записать конфигурацию во все модули АУ по всем АЛС
        private void WriteConfigToGK()
        {
            byte pNum = 1;
            byte pTotal = (byte)this.ALS1LeftSideDevices.Count;
            foreach (GKDevice dv in this.ALS1LeftSideDevices)
            {
                if ((dv.Devices != null) && (dv.Devices.Count > 0))
                {
                    pTotal += (byte)dv.Devices.Count;
                }
            }
            foreach (GKDevice dv in this.ALS1LeftSideDevices)
            {
                SendManager.WriteDeviceParameters(4, dv, 1, pNum++, pTotal);
                if ((dv.Devices != null) && (dv.Devices.Count > 0))
                {
                    foreach (GKDevice sdv in dv.Devices)
                    {
                        SendManager.WriteDeviceParameters(4, sdv, 1, pNum++, pTotal);
                    }
                }
            }
            SendManager.FinalizeWritingDeviceParameters(4, 1);
            pNum = 1;
            pTotal = (byte)this.ALS2LeftSideDevices.Count;
            foreach (GKDevice dv in this.ALS2LeftSideDevices)
            {
                if ((dv.Devices != null) && (dv.Devices.Count > 0))
                {
                    pTotal += (byte)dv.Devices.Count;
                }
            }
            foreach (GKDevice dv in this.ALS2LeftSideDevices)
            {
                SendManager.WriteDeviceParameters(4, dv, 2, pNum++, pTotal);
                if ((dv.Devices != null) && (dv.Devices.Count > 0))
                {
                    foreach (GKDevice sdv in dv.Devices)
                    {
                        SendManager.WriteDeviceParameters(4, sdv, 2, pNum++, pTotal);
                    }
                }
            }
            SendManager.FinalizeWritingDeviceParameters(4, 2);
        }

        private void IntoWorkModeBtn_Click(object sender, RoutedEventArgs e)
        {
            SendManager.GoToWorkingRegime(4);
            Thread.Sleep(5000);
            if (SendManager.IsInWorkingRegime(4))
            {

            }
            else
            {
                SendManager.GoToWorkingRegime(4);
                Thread.Sleep(5000);
                if (SendManager.IsInWorkingRegime(4))
                {

                }
                else
                {
                    MessageBox.Show("Не удалось перевести устройство в рабочий режим. Попробуйте еще раз.", "ПЕРЕХОД В РАБОЧИЙ РЕЖИМ");
                }
            }
        }

        private void Border_MouseLeftButtonDown(object sender, MouseButtonEventArgs e)
        {
            if (e.ClickCount == 2)
                this.MaximizedWindowBtn_Click(this, e);
            else
                this.DragMove();// Для перемещение ока
        }
        private void CloseWindowBtn_Click(object sender, RoutedEventArgs e)
        {
            this.Close();
        }
        private void MaximizedWindowBtn_Click(object sender, RoutedEventArgs e)
        {
            if (this.WindowState == WindowState.Maximized)
            {
                this.WindowState = WindowState.Normal;
            }
            else
            {
                this.WindowState = WindowState.Maximized;
            }
        }
        private void MinimizedWindowBtn_Click(object sender, RoutedEventArgs e)
        {
            if (this.WindowState == WindowState.Minimized)
            {
                this.WindowState = WindowState.Normal;
            }
            else
            {
                this.WindowState = WindowState.Minimized;
            }
        }
        //Включить-Отключить режим циклического опроса модулей АУ по UDP
        private void RequestDataBtn_Click(object sender, RoutedEventArgs e)
        {
            if (this.DataRequestFlag)
            {
                this.AddNewMessage2Console(new MessageCS(DateTime.Now, "C", "Пользователь", "Отключен режим циклического опроса устройств"));
                this.DataRequestFlag = false;
                this.RequestDataBtn.Foreground = Brushes.WhiteSmoke;
            }
            else
            {
                this.AddNewMessage2Console(new MessageCS(DateTime.Now, "C", "Пользователь", "Включен режим циклического опроса устройств"));
                this.DataRequestFlag = true;
                this.RequestDataBtn.Foreground = Brushes.Green;
                ThreadPool.QueueUserWorkItem(new WaitCallback(this.BackgroundWorker), null);
            }
        }
        private void AddNewMessage2Console(MessageCS msg)
        {
            //Application.Current.Dispatcher.BeginInvoke(System.Windows.Threading.DispatcherPriority.Normal, new Action(() =>
            //{
                this.MessengesConsole.MessagesList.Insert(0, msg);
                //this.MessagesListView.ItemsSource = this.MessengesConsole.MessagesList;
                //this.MessagesListView.Items.Refresh();
            //}));
        }
        //Поток проверки связи с МОПС
        private void BackgroundKAUConnectChecker(object status)
        {
            while (this.KAUConnectionCkeckFlag)
            {
                SendResult res = SendManager.Ping(4);
                if (res.HasError)
                {
                    Application.Current.Dispatcher.BeginInvoke(
    System.Windows.Threading.DispatcherPriority.Normal,
    new Action(() =>
    {
        try
        {
            this.ConnectKAUBtn.Foreground = Brushes.Red;
            this.AddNewMessage2Console(new MessageCS(DateTime.Now, "W", "МОПС-R2", "Связь с МОПС потеряна!"));
        }
        catch { }
    }));
                }
                else
                {
                    Application.Current.Dispatcher.BeginInvoke(
    System.Windows.Threading.DispatcherPriority.Normal,
    new Action(() =>
    {
        try
        {
            this.ConnectKAUBtn.Foreground = Brushes.Green;
            //this.MessengesConsole.MessagesList.Insert(0, new MessageCS(DateTime.Now, "M", "МОПС-R2", "Связь с МОПС восстановлена."));
            byte UpdateSTS = res.Bytes[0];
            string UstateStr = "";
            switch (UpdateSTS)
            {
                case 0: UstateStr = "..."; this.KAUStsLabel.Foreground = Brushes.White; break;
                case 1: UstateStr = "Прием файла"; this.KAUStsLabel.Foreground = Brushes.LightGray; break;
                case 2: UstateStr = "Проверка"; this.KAUStsLabel.Foreground = Brushes.LightPink; break;
                case 3: UstateStr = "Загрузка в МК"; this.KAUStsLabel.Foreground = Brushes.WhiteSmoke; break;
                case 4: UstateStr = "УСПЕШНО"; this.KAUStsLabel.Foreground = Brushes.Green; break;
                case 5: UstateStr = "ОШИБКА"; this.KAUStsLabel.Foreground = Brushes.Red; break;
            }
            byte MKupdatePacks2SOM = res.Bytes[1];
            byte MKcurrentPack2SOM = res.Bytes[2];
            byte[] t1 = new byte[2];
            t1[1] = res.Bytes[3];
            t1[0] = res.Bytes[4];
            ushort MKupadtePacks2MK = BitConverter.ToUInt16(t1, 0);
            t1[1] = res.Bytes[5];
            t1[0] = res.Bytes[6];
            ushort MKcurretPack2MK = BitConverter.ToUInt16(t1, 0);
            byte ErrorCode = res.Bytes[7];
            this.KAUStsLabel.Content = "Текущий пакет из ПК в SOM: " + MKupdatePacks2SOM + " (" + MKcurrentPack2SOM + ") Текущий пакет из SOM в МК: " + MKcurretPack2MK + " (" + MKupadtePacks2MK + ")" + " / Статус обновления: "+ UstateStr +" / Ошибка: "+ErrorCode;
            if (UpdateSTS == 4)
            {
                MessageBox.Show("Обноеление прошивки успешно выполнено!");
            }
        }
        catch(Exception ex) {
            string answ = String.Join(" ", res.Bytes);
            this.KAUStsLabel.Content = "Read answer error! " + answ + " -> " + ex.ToString(); }
    }));
                }
                Thread.Sleep(1000);
            }
            Application.Current.Dispatcher.BeginInvoke(
System.Windows.Threading.DispatcherPriority.Normal,
new Action(() =>
{
try
{
    this.ConnectKAUBtn.Foreground = Brushes.White;
}
catch { }
}));
        }
        //Поток обмена с устройствами АУ по АЛС________________________________________________________________________________________
        private void BackgroundWorker(object status)
        {
            while (this.DataRequestFlag)
            {
                List<MyData> devList = new List<MyData>();
                #region ALS1
                bool res = SendManager.RequestDataFromKAU(4, out devList, 1);
                if (!res)
                {
                    //MessageBox.Show("Не удалось получить параметры от МОПС!", "ОШИБКА!");
                }
                else
                {
                    foreach (GKDevice d in this.ALS1LeftSideDevices)
                    {
                        try
                        {
                            //uint sn = uint.Parse();
                            MyData dp = devList.First(x => ((x.ser_no == d.SerialNo) && (x.addr == d.IntAddress))); //devList[d.SerialNo.ToString() + "_" + d.IntAddress];
                            if (dp != null)
                            {
                                byte CC1 = BytesHelper.ShortToBytes((ushort)dp.CC)[1];
                                byte CC2 = BytesHelper.ShortToBytes((ushort)dp.CC)[0];
                                d.DataFromDevice = dp;

                                d.ConnectionState = ((byte)(CC2 >> 3) & (byte)0x1) == 0x1;
                                if (!d.PrevConnectionState)
                                {
                                    if (d.ConnectionState)
                                    {
                                        this.AddNewMessage2Console(new MessageCS(DateTime.Now, "L", "АЛС1." + d.IntAddress + "." + d.Driver.ShortName, "Потеря связи с АУ - сработка"));
                                    }
                                }
                                else
                                {
                                    if (!d.ConnectionState)
                                    {
                                        this.AddNewMessage2Console(new MessageCS(DateTime.Now, "L", "АЛС1." + d.IntAddress + "." + d.Driver.ShortName, "Потеря связи с АУ - сброшен"));
                                    }
                                }
                                d.PrevConnectionState = d.ConnectionState;

                                d.TestState = ((byte)(CC1 >> 4) & (byte)0x1) == 0x1;
                                if (!d.PrevTestState)
                                {
                                    if (d.TestState)
                                    {
                                        this.AddNewMessage2Console(new MessageCS(DateTime.Now, "M", "АЛС1." + d.IntAddress + "." + d.Driver.ShortName, "ТЕСТ - сработка"));
                                    }
                                }
                                else
                                {
                                    if (!d.TestState)
                                    {
                                        this.AddNewMessage2Console(new MessageCS(DateTime.Now, "M", "АЛС1." + d.IntAddress + "." + d.Driver.ShortName, "ТЕСТ - сброшен"));
                                    }
                                }
                                d.PrevTestState = d.TestState;

                                d.FailureState = ((byte)(CC1 >> 5) & (byte)0x1) == 0x1;
                                if (!d.PrevFailureState)
                                {
                                    if (d.FailureState)
                                    {
                                        this.AddNewMessage2Console(new MessageCS(DateTime.Now, "A", "АЛС1." + d.IntAddress + "." + d.Driver.ShortName, "НЕИСПРАВНОСТЬ - сработка"));
                                    }
                                }
                                else
                                {
                                    if (!d.FailureState)
                                    {
                                        this.AddNewMessage2Console(new MessageCS(DateTime.Now, "M", "АЛС1." + d.IntAddress + "." + d.Driver.ShortName, "НЕИСПРАВНОСТЬ - сброшен"));
                                    }
                                }
                                d.PrevFailureState = d.FailureState;

                                d.Fire1 = ((byte)(CC1 >> 2) & (byte)0x1) == 0x1;
                                if (!d.PrevFire1)
                                {
                                    if (d.Fire1)
                                    {
                                        this.AddNewMessage2Console(new MessageCS(DateTime.Now, "A", "АЛС1." + d.IntAddress + "." + d.Driver.ShortName, "ПОЖАР1 - сработка"));
                                    }
                                }
                                else
                                {
                                    if (!d.Fire1)
                                    {
                                        this.AddNewMessage2Console(new MessageCS(DateTime.Now, "M", "АЛС1." + d.IntAddress + "." + d.Driver.ShortName, "ПОЖАР1 - сброшен"));
                                    }
                                }
                                d.PrevFire1 = d.Fire1;

                                d.Fire2 = ((byte)(CC1 >> 3) & (byte)0x1) == 0x1;
                                if (!d.PrevFire2)
                                {
                                    if (d.Fire2)
                                    {
                                        this.AddNewMessage2Console(new MessageCS(DateTime.Now, "A", "АЛС1." + d.IntAddress + "." + d.Driver.ShortName, "ПОЖАР2 - сработка"));
                                    }
                                }
                                else
                                {
                                    if (!d.Fire2)
                                    {
                                        this.AddNewMessage2Console(new MessageCS(DateTime.Now, "M", "АЛС1." + d.IntAddress + "." + d.Driver.ShortName, "ПОЖАР2 - сброшен"));
                                    }
                                }
                                d.PrevFire2 = d.Fire2;

                                d.Mode1 = ((byte)(CC1 >> 0) & (byte)0x1) == 0x1;
                                d.Mode2 = ((byte)(CC1 >> 6) & (byte)0x1) == 0x1;

                                d.OnState = ((byte)(CC1 >> 7) & (byte)0x1) == 0x1;
                                if (!d.PrevOnState)
                                {
                                    if (d.OnState)
                                    {
                                        this.AddNewMessage2Console(new MessageCS(DateTime.Now, "M", "АЛС1." + d.IntAddress + "." + d.Driver.ShortName, "ВКЛЮЧЕНО - сработка"));
                                    }
                                }
                                else
                                {
                                    if (!d.OnState)
                                    {
                                        this.AddNewMessage2Console(new MessageCS(DateTime.Now, "M", "АЛС1." + d.IntAddress + "." + d.Driver.ShortName, "ВКЛЮЧЕНО - сброшен"));
                                    }
                                }
                                d.PrevOnState = d.OnState;

                                d.OffState = ((byte)(CC2 >> 0) & (byte)0x1) == 0x1;
                                if (!d.PrevOffState)
                                {
                                    if (d.OffState)
                                    {
                                        this.AddNewMessage2Console(new MessageCS(DateTime.Now, "M", "АЛС1." + d.IntAddress + "." + d.Driver.ShortName, "ОТКЛЮЧЕНО - сработка"));
                                    }
                                }
                                else
                                {
                                    if (!d.OffState)
                                    {
                                        this.AddNewMessage2Console(new MessageCS(DateTime.Now, "M", "АЛС1." + d.IntAddress + "." + d.Driver.ShortName, "ОТКЛЮЧЕНО - сброшен"));
                                    }
                                }
                                d.PrevOffState = d.OffState;

                                d.AddCCstate = BytesHelper.SubstructShort(dp.data, 2);

                                d.OnAction = ((byte)(CC2 >> 1) & (byte)0x1) == 0x1;
                                if (!d.PrevOnAction)
                                {
                                    if (d.OnAction)
                                    {
                                        this.AddNewMessage2Console(new MessageCS(DateTime.Now, "M", "АЛС1." + d.IntAddress + "." + d.Driver.ShortName, "ВКЛЮЧЕНИЕ - сработка"));
                                    }
                                }
                                else
                                {
                                    if (!d.OnAction)
                                    {
                                        this.AddNewMessage2Console(new MessageCS(DateTime.Now, "M", "АЛС1." + d.IntAddress + "." + d.Driver.ShortName, "ВКЛЮЧЕНИЕ - сброшен"));
                                    }
                                }
                                d.PrevOnAction = d.OnAction;

                                d.OffAction = ((byte)(CC2 >> 2) & (byte)0x1) == 0x1;
                                if (!d.PrevOffAction)
                                {
                                    if (d.OffAction)
                                    {
                                        this.AddNewMessage2Console(new MessageCS(DateTime.Now, "M", "АЛС1." + d.IntAddress + "." + d.Driver.ShortName, "ОТКЛЮЧЕНИЕ - сработка"));
                                    }
                                }
                                else
                                {
                                    if (!d.OffAction)
                                    {
                                        this.AddNewMessage2Console(new MessageCS(DateTime.Now, "M", "АЛС1." + d.IntAddress + "." + d.Driver.ShortName, "ОТКЛЮЧЕНИЕ - сброшен"));
                                    }
                                }
                                d.PrevOffAction = d.OffAction;
                            }
                        }
                        catch { }
                        if ((d.Devices != null) && (d.Devices.Count > 0))
                        {
                            foreach (GKDevice sd in d.Devices)
                            {
                                try
                                {
                                    //uint sn = uint.Parse();
                                    MyData dp = devList.First(x => ((x.ser_no == sd.SerialNo)&&(x.addr == sd.IntAddress)));//devList[sd.SerialNo.ToString() + "_" + sd.IntAddress];
                                    if (dp != null)
                                    {
                                        byte CC1 = BytesHelper.ShortToBytes((ushort)dp.CC)[1];
                                        byte CC2 = BytesHelper.ShortToBytes((ushort)dp.CC)[0];
                                        sd.DataFromDevice = dp;

                                        sd.ConnectionState = ((byte)(CC2 >> 3) & (byte)0x1) == 0x1;
                                        if (!sd.PrevConnectionState)
                                        {
                                            if (sd.ConnectionState)
                                            {
                                                this.AddNewMessage2Console(new MessageCS(DateTime.Now, "L", "АЛС1." + sd.IntAddress + "." + sd.Driver.ShortName, "Потеря связи с АУ - сработка"));
                                            }
                                        }
                                        else
                                        {
                                            if (!sd.ConnectionState)
                                            {
                                                this.AddNewMessage2Console(new MessageCS(DateTime.Now, "L", "АЛС1." + sd.IntAddress + "." + sd.Driver.ShortName, "Потеря связи с АУ - сброшен"));
                                            }
                                        }
                                        sd.PrevConnectionState = sd.ConnectionState;

                                        sd.TestState = ((byte)(CC1 >> 4) & (byte)0x1) == 0x1;
                                        if (!sd.PrevTestState)
                                        {
                                            if (sd.TestState)
                                            {
                                                this.AddNewMessage2Console(new MessageCS(DateTime.Now, "M", "АЛС1." + sd.IntAddress + "." + sd.Driver.ShortName, "ТЕСТ - сработка"));
                                            }
                                        }
                                        else
                                        {
                                            if (!sd.TestState)
                                            {
                                                this.AddNewMessage2Console(new MessageCS(DateTime.Now, "M", "АЛС1." + sd.IntAddress + "." + sd.Driver.ShortName, "ТЕСТ - сброшен"));
                                            }
                                        }
                                        sd.PrevTestState = sd.TestState;

                                        sd.FailureState = ((byte)(CC1 >> 5) & (byte)0x1) == 0x1;
                                        if (!sd.PrevFailureState)
                                        {
                                            if (sd.FailureState)
                                            {
                                                this.AddNewMessage2Console(new MessageCS(DateTime.Now, "A", "АЛС1." + sd.IntAddress + "." + sd.Driver.ShortName, "НЕИСПРАВНОСТЬ - сработка"));
                                            }
                                        }
                                        else
                                        {
                                            if (!sd.FailureState)
                                            {
                                                this.AddNewMessage2Console(new MessageCS(DateTime.Now, "M", "АЛС1." + sd.IntAddress + "." + sd.Driver.ShortName, "НЕИСПРАВНОСТЬ - сброшен"));
                                            }
                                        }
                                        sd.PrevFailureState = sd.FailureState;

                                        sd.Fire1 = ((byte)(CC1 >> 2) & (byte)0x1) == 0x1;
                                        if (!sd.PrevFire1)
                                        {
                                            if (sd.Fire1)
                                            {
                                                this.AddNewMessage2Console(new MessageCS(DateTime.Now, "A", "АЛС1." + sd.IntAddress + "." + sd.Driver.ShortName, "ПОЖАР1 - сработка"));
                                            }
                                        }
                                        else
                                        {
                                            if (!sd.Fire1)
                                            {
                                                this.AddNewMessage2Console(new MessageCS(DateTime.Now, "M", "АЛС1." + sd.IntAddress + "." + sd.Driver.ShortName, "ПОЖАР1 - сброшен"));
                                            }
                                        }
                                        sd.PrevFire1 = sd.Fire1;

                                        sd.Fire2 = ((byte)(CC1 >> 3) & (byte)0x1) == 0x1;
                                        if (!sd.PrevFire2)
                                        {
                                            if (sd.Fire2)
                                            {
                                                this.AddNewMessage2Console(new MessageCS(DateTime.Now, "A", "АЛС1." + sd.IntAddress + "." + sd.Driver.ShortName, "ПОЖАР2 - сработка"));
                                            }
                                        }
                                        else
                                        {
                                            if (!sd.Fire2)
                                            {
                                                this.AddNewMessage2Console(new MessageCS(DateTime.Now, "M", "АЛС1." + sd.IntAddress + "." + sd.Driver.ShortName, "ПОЖАР2 - сброшен"));
                                            }
                                        }
                                        sd.PrevFire2 = sd.Fire2;

                                        sd.Mode1 = ((byte)(CC1 >> 0) & (byte)0x1) == 0x1;
                                        sd.Mode2 = ((byte)(CC1 >> 6) & (byte)0x1) == 0x1;

                                        sd.OnState = ((byte)(CC1 >> 7) & (byte)0x1) == 0x1;
                                        if (!sd.PrevOnState)
                                        {
                                            if (sd.OnState)
                                            {
                                                this.AddNewMessage2Console(new MessageCS(DateTime.Now, "M", "АЛС1." + sd.IntAddress + "." + sd.Driver.ShortName, "ВКЛЮЧЕНО - сработка"));
                                            }
                                        }
                                        else
                                        {
                                            if (!sd.OnState)
                                            {
                                                this.AddNewMessage2Console(new MessageCS(DateTime.Now, "M", "АЛС1." + sd.IntAddress + "." + sd.Driver.ShortName, "ВКЛЮЧЕНО - сброшен"));
                                            }
                                        }
                                        sd.PrevOnState = sd.OnState;

                                        sd.OffState = ((byte)(CC2 >> 0) & (byte)0x1) == 0x1;
                                        if (!sd.PrevOffState)
                                        {
                                            if (sd.OffState)
                                            {
                                                this.AddNewMessage2Console(new MessageCS(DateTime.Now, "M", "АЛС1." + sd.IntAddress + "." + sd.Driver.ShortName, "ОТКЛЮЧЕНО - сработка"));
                                            }
                                        }
                                        else
                                        {
                                            if (!sd.OffState)
                                            {
                                                this.AddNewMessage2Console(new MessageCS(DateTime.Now, "M", "АЛС1." + sd.IntAddress + "." + sd.Driver.ShortName, "ОТКЛЮЧЕНО - сброшен"));
                                            }
                                        }
                                        sd.PrevOffState = sd.OffState;

                                        sd.AddCCstate = BytesHelper.SubstructShort(dp.data, 2);

                                        sd.OnAction = ((byte)(CC2 >> 1) & (byte)0x1) == 0x1;
                                        if (!sd.PrevOnAction)
                                        {
                                            if (sd.OnAction)
                                            {
                                                this.AddNewMessage2Console(new MessageCS(DateTime.Now, "M", "АЛС1." + sd.IntAddress + "." + sd.Driver.ShortName, "ВКЛЮЧЕНИЕ - сработка"));
                                            }
                                        }
                                        else
                                        {
                                            if (!sd.OnAction)
                                            {
                                                this.AddNewMessage2Console(new MessageCS(DateTime.Now, "M", "АЛС1." + sd.IntAddress + "." + sd.Driver.ShortName, "ВКЛЮЧЕНИЕ - сброшен"));
                                            }
                                        }
                                        sd.PrevOnAction = sd.OnAction;

                                        sd.OffAction = ((byte)(CC2 >> 2) & (byte)0x1) == 0x1;
                                        if (!sd.PrevOffAction)
                                        {
                                            if (sd.OffAction)
                                            {
                                                this.AddNewMessage2Console(new MessageCS(DateTime.Now, "M", "АЛС1." + sd.IntAddress + "." + sd.Driver.ShortName, "ОТКЛЮЧЕНИЕ - сработка"));
                                            }
                                        }
                                        else
                                        {
                                            if (!sd.OffAction)
                                            {
                                                this.AddNewMessage2Console(new MessageCS(DateTime.Now, "M", "АЛС1." + sd.IntAddress + "." + sd.Driver.ShortName, "ОТКЛЮЧЕНИЕ - сброшен"));
                                            }
                                        }
                                        sd.PrevOffAction = sd.OffAction;
                                    }
                                }
                                catch { }
                                sd.deviceMessagesList.Reverse();
                                foreach (MessageCS ms in sd.deviceMessagesList)
                                {
                                    this.AddNewMessage2Console(ms);
                                }
                                sd.deviceMessagesList.Clear();
                            }
                        }
                        d.deviceMessagesList.Reverse();
                        foreach (MessageCS ms in d.deviceMessagesList)
                        {
                            this.AddNewMessage2Console(ms);
                        }
                        d.deviceMessagesList.Clear();
                    }
                    foreach (GKDevice d in this.ALS1RightSideDevices)
                    {
                        try
                        {
                            MyData dp = devList.First(x => ((x.ser_no == d.SerialNo) && (x.addr == d.IntAddress))); //devList[d.SerialNo.ToString() + "_" + d.IntAddress];
                            if (dp != null)
                            {
                                byte CC1 = BytesHelper.ShortToBytes((ushort)dp.CC)[1];
                                byte CC2 = BytesHelper.ShortToBytes((ushort)dp.CC)[0];
                                d.DataFromDevice = dp;

                                d.ConnectionState = ((byte)(CC2 >> 3) & (byte)0x1) == 0x1;
                                if (!d.PrevConnectionState)
                                {
                                    if (d.ConnectionState)
                                    {
                                        this.AddNewMessage2Console(new MessageCS(DateTime.Now, "L", "АЛС1." + d.IntAddress + "." + d.Driver.ShortName, "Потеря связи с АУ - сработка"));
                                    }
                                }
                                else
                                {
                                    if (!d.ConnectionState)
                                    {
                                        this.AddNewMessage2Console(new MessageCS(DateTime.Now, "L", "АЛС1." + d.IntAddress + "." + d.Driver.ShortName, "Потеря связи с АУ - сброшен"));
                                    }
                                }
                                d.PrevConnectionState = d.ConnectionState;

                                d.TestState = ((byte)(CC1 >> 4) & (byte)0x1) == 0x1;
                                if (!d.PrevTestState)
                                {
                                    if (d.TestState)
                                    {
                                        this.AddNewMessage2Console(new MessageCS(DateTime.Now, "M", "АЛС1." + d.IntAddress + "." + d.Driver.ShortName, "ТЕСТ - сработка"));
                                    }
                                }
                                else
                                {
                                    if (!d.TestState)
                                    {
                                        this.AddNewMessage2Console(new MessageCS(DateTime.Now, "M", "АЛС1." + d.IntAddress + "." + d.Driver.ShortName, "ТЕСТ - сброшен"));
                                    }
                                }
                                d.PrevTestState = d.TestState;

                                d.FailureState = ((byte)(CC1 >> 5) & (byte)0x1) == 0x1;
                                if (!d.PrevFailureState)
                                {
                                    if (d.FailureState)
                                    {
                                        this.AddNewMessage2Console(new MessageCS(DateTime.Now, "A", "АЛС1." + d.IntAddress + "." + d.Driver.ShortName, "НЕИСПРАВНОСТЬ - сработка"));
                                    }
                                }
                                else
                                {
                                    if (!d.FailureState)
                                    {
                                        this.AddNewMessage2Console(new MessageCS(DateTime.Now, "M", "АЛС1." + d.IntAddress + "." + d.Driver.ShortName, "НЕИСПРАВНОСТЬ - сброшен"));
                                    }
                                }
                                d.PrevFailureState = d.FailureState;

                                d.Fire1 = ((byte)(CC1 >> 2) & (byte)0x1) == 0x1;
                                if (!d.PrevFire1)
                                {
                                    if (d.Fire1)
                                    {
                                        this.AddNewMessage2Console(new MessageCS(DateTime.Now, "A", "АЛС1." + d.IntAddress + "." + d.Driver.ShortName, "ПОЖАР1 - сработка"));
                                    }
                                }
                                else
                                {
                                    if (!d.Fire1)
                                    {
                                        this.AddNewMessage2Console(new MessageCS(DateTime.Now, "M", "АЛС1." + d.IntAddress + "." + d.Driver.ShortName, "ПОЖАР1 - сброшен"));
                                    }
                                }
                                d.PrevFire1 = d.Fire1;

                                d.Fire2 = ((byte)(CC1 >> 3) & (byte)0x1) == 0x1;
                                if (!d.PrevFire2)
                                {
                                    if (d.Fire2)
                                    {
                                        this.AddNewMessage2Console(new MessageCS(DateTime.Now, "A", "АЛС1." + d.IntAddress + "." + d.Driver.ShortName, "ПОЖАР2 - сработка"));
                                    }
                                }
                                else
                                {
                                    if (!d.Fire2)
                                    {
                                        this.AddNewMessage2Console(new MessageCS(DateTime.Now, "M", "АЛС1." + d.IntAddress + "." + d.Driver.ShortName, "ПОЖАР2 - сброшен"));
                                    }
                                }
                                d.PrevFire2 = d.Fire2;

                                d.Mode1 = ((byte)(CC1 >> 0) & (byte)0x1) == 0x1;
                                d.Mode2 = ((byte)(CC1 >> 6) & (byte)0x1) == 0x1;

                                d.OnState = ((byte)(CC1 >> 7) & (byte)0x1) == 0x1;
                                if (!d.PrevOnState)
                                {
                                    if (d.OnState)
                                    {
                                        this.AddNewMessage2Console(new MessageCS(DateTime.Now, "M", "АЛС1." + d.IntAddress + "." + d.Driver.ShortName, "ВКЛЮЧЕНО - сработка"));
                                    }
                                }
                                else
                                {
                                    if (!d.OnState)
                                    {
                                        this.AddNewMessage2Console(new MessageCS(DateTime.Now, "M", "АЛС1." + d.IntAddress + "." + d.Driver.ShortName, "ВКЛЮЧЕНО - сброшен"));
                                    }
                                }
                                d.PrevOnState = d.OnState;

                                d.OffState = ((byte)(CC2 >> 0) & (byte)0x1) == 0x1;
                                if (!d.PrevOffState)
                                {
                                    if (d.OffState)
                                    {
                                        this.AddNewMessage2Console(new MessageCS(DateTime.Now, "M", "АЛС1." + d.IntAddress + "." + d.Driver.ShortName, "ОТКЛЮЧЕНО - сработка"));
                                    }
                                }
                                else
                                {
                                    if (!d.OffState)
                                    {
                                        this.AddNewMessage2Console(new MessageCS(DateTime.Now, "M", "АЛС1." + d.IntAddress + "." + d.Driver.ShortName, "ОТКЛЮЧЕНО - сброшен"));
                                    }
                                }
                                d.PrevOffState = d.OffState;

                                d.AddCCstate = BytesHelper.SubstructShort(dp.data, 2);

                                d.OnAction = ((byte)(CC2 >> 1) & (byte)0x1) == 0x1;
                                if (!d.PrevOnAction)
                                {
                                    if (d.OnAction)
                                    {
                                        this.AddNewMessage2Console(new MessageCS(DateTime.Now, "M", "АЛС1." + d.IntAddress + "." + d.Driver.ShortName, "ВКЛЮЧЕНИЕ - сработка"));
                                    }
                                }
                                else
                                {
                                    if (!d.OnAction)
                                    {
                                        this.AddNewMessage2Console(new MessageCS(DateTime.Now, "M", "АЛС1." + d.IntAddress + "." + d.Driver.ShortName, "ВКЛЮЧЕНИЕ - сброшен"));
                                    }
                                }
                                d.PrevOnAction = d.OnAction;

                                d.OffAction = ((byte)(CC2 >> 2) & (byte)0x1) == 0x1;
                                if (!d.PrevOffAction)
                                {
                                    if (d.OffAction)
                                    {
                                        this.AddNewMessage2Console(new MessageCS(DateTime.Now, "M", "АЛС1." + d.IntAddress + "." + d.Driver.ShortName, "ОТКЛЮЧЕНИЕ - сработка"));
                                    }
                                }
                                else
                                {
                                    if (!d.OffAction)
                                    {
                                        this.AddNewMessage2Console(new MessageCS(DateTime.Now, "M", "АЛС1." + d.IntAddress + "." + d.Driver.ShortName, "ОТКЛЮЧЕНИЕ - сброшен"));
                                    }
                                }
                                d.PrevOffAction = d.OffAction;
                            }
                        }
                        catch { }
                        if ((d.Devices != null) && (d.Devices.Count > 0))
                        {
                            foreach (GKDevice sd in d.Devices)
                            {
                                try
                                {
                                    //uint sn = uint.Parse();
                                    MyData dp = devList.First(x => ((x.ser_no == sd.SerialNo) && (x.addr == sd.IntAddress))); //devList[sd.SerialNo.ToString() + "_" + sd.IntAddress];
                                    if (dp != null)
                                    {
                                        byte CC1 = BytesHelper.ShortToBytes((ushort)dp.CC)[1];
                                        byte CC2 = BytesHelper.ShortToBytes((ushort)dp.CC)[0];
                                        sd.DataFromDevice = dp;

                                        sd.ConnectionState = ((byte)(CC2 >> 3) & (byte)0x1) == 0x1;
                                        if (!sd.PrevConnectionState)
                                        {
                                            if (sd.ConnectionState)
                                            {
                                                this.AddNewMessage2Console(new MessageCS(DateTime.Now, "L", "АЛС1." + sd.IntAddress + "." + sd.Driver.ShortName, "Потеря связи с АУ - сработка"));
                                            }
                                        }
                                        else
                                        {
                                            if (!sd.ConnectionState)
                                            {
                                                this.AddNewMessage2Console(new MessageCS(DateTime.Now, "L", "АЛС1." + sd.IntAddress + "." + sd.Driver.ShortName, "Потеря связи с АУ - сброшен"));
                                            }
                                        }
                                        sd.PrevConnectionState = sd.ConnectionState;

                                        sd.TestState = ((byte)(CC1 >> 4) & (byte)0x1) == 0x1;
                                        if (!sd.PrevTestState)
                                        {
                                            if (sd.TestState)
                                            {
                                                this.AddNewMessage2Console(new MessageCS(DateTime.Now, "M", "АЛС1." + sd.IntAddress + "." + sd.Driver.ShortName, "ТЕСТ - сработка"));
                                            }
                                        }
                                        else
                                        {
                                            if (!sd.TestState)
                                            {
                                                this.AddNewMessage2Console(new MessageCS(DateTime.Now, "M", "АЛС1." + sd.IntAddress + "." + sd.Driver.ShortName, "ТЕСТ - сброшен"));
                                            }
                                        }
                                        sd.PrevTestState = sd.TestState;

                                        sd.FailureState = ((byte)(CC1 >> 5) & (byte)0x1) == 0x1;
                                        if (!sd.PrevFailureState)
                                        {
                                            if (sd.FailureState)
                                            {
                                                this.AddNewMessage2Console(new MessageCS(DateTime.Now, "A", "АЛС1." + sd.IntAddress + "." + sd.Driver.ShortName, "НЕИСПРАВНОСТЬ - сработка"));
                                            }
                                        }
                                        else
                                        {
                                            if (!sd.FailureState)
                                            {
                                                this.AddNewMessage2Console(new MessageCS(DateTime.Now, "M", "АЛС1." + sd.IntAddress + "." + sd.Driver.ShortName, "НЕИСПРАВНОСТЬ - сброшен"));
                                            }
                                        }
                                        sd.PrevFailureState = sd.FailureState;

                                        sd.Fire1 = ((byte)(CC1 >> 2) & (byte)0x1) == 0x1;
                                        if (!sd.PrevFire1)
                                        {
                                            if (sd.Fire1)
                                            {
                                                this.AddNewMessage2Console(new MessageCS(DateTime.Now, "A", "АЛС1." + sd.IntAddress + "." + sd.Driver.ShortName, "ПОЖАР1 - сработка"));
                                            }
                                        }
                                        else
                                        {
                                            if (!sd.Fire1)
                                            {
                                                this.AddNewMessage2Console(new MessageCS(DateTime.Now, "M", "АЛС1." + sd.IntAddress + "." + sd.Driver.ShortName, "ПОЖАР1 - сброшен"));
                                            }
                                        }
                                        sd.PrevFire1 = sd.Fire1;

                                        sd.Fire2 = ((byte)(CC1 >> 3) & (byte)0x1) == 0x1;
                                        if (!sd.PrevFire2)
                                        {
                                            if (sd.Fire2)
                                            {
                                                this.AddNewMessage2Console(new MessageCS(DateTime.Now, "A", "АЛС1." + sd.IntAddress + "." + sd.Driver.ShortName, "ПОЖАР2 - сработка"));
                                            }
                                        }
                                        else
                                        {
                                            if (!sd.Fire2)
                                            {
                                                this.AddNewMessage2Console(new MessageCS(DateTime.Now, "M", "АЛС1." + sd.IntAddress + "." + sd.Driver.ShortName, "ПОЖАР2 - сброшен"));
                                            }
                                        }
                                        sd.PrevFire2 = sd.Fire2;

                                        sd.Mode1 = ((byte)(CC1 >> 0) & (byte)0x1) == 0x1;
                                        sd.Mode2 = ((byte)(CC1 >> 6) & (byte)0x1) == 0x1;

                                        sd.OnState = ((byte)(CC1 >> 7) & (byte)0x1) == 0x1;
                                        if (!sd.PrevOnState)
                                        {
                                            if (sd.OnState)
                                            {
                                                this.AddNewMessage2Console(new MessageCS(DateTime.Now, "M", "АЛС1." + sd.IntAddress + "." + sd.Driver.ShortName, "ВКЛЮЧЕНО - сработка"));
                                            }
                                        }
                                        else
                                        {
                                            if (!sd.OnState)
                                            {
                                                this.AddNewMessage2Console(new MessageCS(DateTime.Now, "M", "АЛС1." + sd.IntAddress + "." + sd.Driver.ShortName, "ВКЛЮЧЕНО - сброшен"));
                                            }
                                        }
                                        sd.PrevOnState = sd.OnState;

                                        sd.OffState = ((byte)(CC2 >> 0) & (byte)0x1) == 0x1;
                                        if (!sd.PrevOffState)
                                        {
                                            if (sd.OffState)
                                            {
                                                this.AddNewMessage2Console(new MessageCS(DateTime.Now, "M", "АЛС1." + sd.IntAddress + "." + sd.Driver.ShortName, "ОТКЛЮЧЕНО - сработка"));
                                            }
                                        }
                                        else
                                        {
                                            if (!sd.OffState)
                                            {
                                                this.AddNewMessage2Console(new MessageCS(DateTime.Now, "M", "АЛС1." + sd.IntAddress + "." + sd.Driver.ShortName, "ОТКЛЮЧЕНО - сброшен"));
                                            }
                                        }
                                        sd.PrevOffState = sd.OffState;

                                        sd.AddCCstate = BytesHelper.SubstructShort(dp.data, 2);

                                        sd.OnAction = ((byte)(CC2 >> 1) & (byte)0x1) == 0x1;
                                        if (!sd.PrevOnAction)
                                        {
                                            if (sd.OnAction)
                                            {
                                                this.AddNewMessage2Console(new MessageCS(DateTime.Now, "M", "АЛС1." + sd.IntAddress + "." + sd.Driver.ShortName, "ВКЛЮЧЕНИЕ - сработка"));
                                            }
                                        }
                                        else
                                        {
                                            if (!sd.OnAction)
                                            {
                                                this.AddNewMessage2Console(new MessageCS(DateTime.Now, "M", "АЛС1." + sd.IntAddress + "." + sd.Driver.ShortName, "ВКЛЮЧЕНИЕ - сброшен"));
                                            }
                                        }
                                        sd.PrevOnAction = sd.OnAction;

                                        sd.OffAction = ((byte)(CC2 >> 2) & (byte)0x1) == 0x1;
                                        if (!sd.PrevOffAction)
                                        {
                                            if (sd.OffAction)
                                            {
                                                this.AddNewMessage2Console(new MessageCS(DateTime.Now, "M", "АЛС1." + sd.IntAddress + "." + sd.Driver.ShortName, "ОТКЛЮЧЕНИЕ - сработка"));
                                            }
                                        }
                                        else
                                        {
                                            if (!sd.OffAction)
                                            {
                                                this.AddNewMessage2Console(new MessageCS(DateTime.Now, "M", "АЛС1." + sd.IntAddress + "." + sd.Driver.ShortName, "ОТКЛЮЧЕНИЕ - сброшен"));
                                            }
                                        }
                                        sd.PrevOffAction = sd.OffAction;
                                    }
                                }
                                catch { }
                                sd.deviceMessagesList.Reverse();
                                foreach (MessageCS ms in sd.deviceMessagesList)
                                {
                                    this.AddNewMessage2Console(ms);
                                }
                                sd.deviceMessagesList.Clear();
                            }
                        }
                        d.deviceMessagesList.Reverse();
                        foreach (MessageCS ms in d.deviceMessagesList)
                        {
                            this.AddNewMessage2Console(ms);
                        }
                        d.deviceMessagesList.Clear();
                    }
                    //MessageBox.Show("Параметры от МОПС успешно получены!", "Данные от МОПС");
                }
                #endregion
                #region ALS2
                res = SendManager.RequestDataFromKAU(4, out devList, 2);
                if (!res)
                {
                    //MessageBox.Show("Не удалось получить параметры от МОПС!", "ОШИБКА!");
                }
                else
                {
                    foreach (GKDevice d in this.ALS2LeftSideDevices)
                    {
                        try
                        {
                            //uint sn = uint.Parse();
                            MyData dp = devList.First(x => ((x.ser_no == d.SerialNo) && (x.addr == d.IntAddress))); //devList[d.SerialNo.ToString() + "_" + d.IntAddress];
                            if (dp != null)
                            {
                                byte CC1 = BytesHelper.ShortToBytes((ushort)dp.CC)[1];
                                byte CC2 = BytesHelper.ShortToBytes((ushort)dp.CC)[0];
                                d.DataFromDevice = dp;

                                d.ConnectionState = ((byte)(CC2 >> 3) & (byte)0x1) == 0x1;
                                if (!d.PrevConnectionState)
                                {
                                    if (d.ConnectionState)
                                    {
                                        this.AddNewMessage2Console(new MessageCS(DateTime.Now, "L", "АЛС2." + d.IntAddress + "." + d.Driver.ShortName, "Потеря связи с АУ - сработка"));
                                    }
                                }
                                else
                                {
                                    if (!d.ConnectionState)
                                    {
                                        this.AddNewMessage2Console(new MessageCS(DateTime.Now, "L", "АЛС2." + d.IntAddress + "." + d.Driver.ShortName, "Потеря связи с АУ - сброшен"));
                                    }
                                }
                                d.PrevConnectionState = d.ConnectionState;

                                d.TestState = ((byte)(CC1 >> 4) & (byte)0x1) == 0x1;
                                if (!d.PrevTestState)
                                {
                                    if (d.TestState)
                                    {
                                        this.AddNewMessage2Console(new MessageCS(DateTime.Now, "M", "АЛС2." + d.IntAddress + "." + d.Driver.ShortName, "ТЕСТ - сработка"));
                                    }
                                }
                                else
                                {
                                    if (!d.TestState)
                                    {
                                        this.AddNewMessage2Console(new MessageCS(DateTime.Now, "M", "АЛС2." + d.IntAddress + "." + d.Driver.ShortName, "ТЕСТ - сброшен"));
                                    }
                                }
                                d.PrevTestState = d.TestState;

                                d.FailureState = ((byte)(CC1 >> 5) & (byte)0x1) == 0x1;
                                if (!d.PrevFailureState)
                                {
                                    if (d.FailureState)
                                    {
                                        this.AddNewMessage2Console(new MessageCS(DateTime.Now, "A", "АЛС2." + d.IntAddress + "." + d.Driver.ShortName, "НЕИСПРАВНОСТЬ - сработка"));
                                    }
                                }
                                else
                                {
                                    if (!d.FailureState)
                                    {
                                        this.AddNewMessage2Console(new MessageCS(DateTime.Now, "M", "АЛС2." + d.IntAddress + "." + d.Driver.ShortName, "НЕИСПРАВНОСТЬ - сброшен"));
                                    }
                                }
                                d.PrevFailureState = d.FailureState;

                                d.Fire1 = ((byte)(CC1 >> 2) & (byte)0x1) == 0x1;
                                if (!d.PrevFire1)
                                {
                                    if (d.Fire1)
                                    {
                                        this.AddNewMessage2Console(new MessageCS(DateTime.Now, "A", "АЛС2." + d.IntAddress + "." + d.Driver.ShortName, "ПОЖАР1 - сработка"));
                                    }
                                }
                                else
                                {
                                    if (!d.Fire1)
                                    {
                                        this.AddNewMessage2Console(new MessageCS(DateTime.Now, "M", "АЛС2." + d.IntAddress + "." + d.Driver.ShortName, "ПОЖАР1 - сброшен"));
                                    }
                                }
                                d.PrevFire1 = d.Fire1;

                                d.Fire2 = ((byte)(CC1 >> 3) & (byte)0x1) == 0x1;
                                if (!d.PrevFire2)
                                {
                                    if (d.Fire2)
                                    {
                                        this.AddNewMessage2Console(new MessageCS(DateTime.Now, "A", "АЛС2." + d.IntAddress + "." + d.Driver.ShortName, "ПОЖАР2 - сработка"));
                                    }
                                }
                                else
                                {
                                    if (!d.Fire2)
                                    {
                                        this.AddNewMessage2Console(new MessageCS(DateTime.Now, "M", "АЛС2." + d.IntAddress + "." + d.Driver.ShortName, "ПОЖАР2 - сброшен"));
                                    }
                                }
                                d.PrevFire2 = d.Fire2;

                                d.Mode1 = ((byte)(CC1 >> 0) & (byte)0x1) == 0x1;
                                d.Mode2 = ((byte)(CC1 >> 6) & (byte)0x1) == 0x1;

                                d.OnState = ((byte)(CC1 >> 7) & (byte)0x1) == 0x1;
                                if (!d.PrevOnState)
                                {
                                    if (d.OnState)
                                    {
                                        this.AddNewMessage2Console(new MessageCS(DateTime.Now, "M", "АЛС2." + d.IntAddress + "." + d.Driver.ShortName, "ВКЛЮЧЕНО - сработка"));
                                    }
                                }
                                else
                                {
                                    if (!d.OnState)
                                    {
                                        this.AddNewMessage2Console(new MessageCS(DateTime.Now, "M", "АЛС2." + d.IntAddress + "." + d.Driver.ShortName, "ВКЛЮЧЕНО - сброшен"));
                                    }
                                }
                                d.PrevOnState = d.OnState;

                                d.OffState = ((byte)(CC2 >> 0) & (byte)0x1) == 0x1;
                                if (!d.PrevOffState)
                                {
                                    if (d.OffState)
                                    {
                                        this.AddNewMessage2Console(new MessageCS(DateTime.Now, "M", "АЛС2." + d.IntAddress + "." + d.Driver.ShortName, "ОТКЛЮЧЕНО - сработка"));
                                    }
                                }
                                else
                                {
                                    if (!d.OffState)
                                    {
                                        this.AddNewMessage2Console(new MessageCS(DateTime.Now, "M", "АЛС2." + d.IntAddress + "." + d.Driver.ShortName, "ОТКЛЮЧЕНО - сброшен"));
                                    }
                                }
                                d.PrevOffState = d.OffState;

                                d.AddCCstate = BytesHelper.SubstructShort(dp.data, 2);

                                d.OnAction = ((byte)(CC2 >> 1) & (byte)0x1) == 0x1;
                                if (!d.PrevOnAction)
                                {
                                    if (d.OnAction)
                                    {
                                        this.AddNewMessage2Console(new MessageCS(DateTime.Now, "M", "АЛС2." + d.IntAddress + "." + d.Driver.ShortName, "ВКЛЮЧЕНИЕ - сработка"));
                                    }
                                }
                                else
                                {
                                    if (!d.OnAction)
                                    {
                                        this.AddNewMessage2Console(new MessageCS(DateTime.Now, "M", "АЛС2." + d.IntAddress + "." + d.Driver.ShortName, "ВКЛЮЧЕНИЕ - сброшен"));
                                    }
                                }
                                d.PrevOnAction = d.OnAction;

                                d.OffAction = ((byte)(CC2 >> 2) & (byte)0x1) == 0x1;
                                if (!d.PrevOffAction)
                                {
                                    if (d.OffAction)
                                    {
                                        this.AddNewMessage2Console(new MessageCS(DateTime.Now, "M", "АЛС2." + d.IntAddress + "." + d.Driver.ShortName, "ОТКЛЮЧЕНИЕ - сработка"));
                                    }
                                }
                                else
                                {
                                    if (!d.OffAction)
                                    {
                                        this.AddNewMessage2Console(new MessageCS(DateTime.Now, "M", "АЛС2." + d.IntAddress + "." + d.Driver.ShortName, "ОТКЛЮЧЕНИЕ - сброшен"));
                                    }
                                }
                                d.PrevOffAction = d.OffAction;
                            }
                        }
                        catch { }
                        if ((d.Devices != null) && (d.Devices.Count > 0))
                        {
                            foreach (GKDevice sd in d.Devices)
                            {
                                try
                                {
                                    //uint sn = uint.Parse();
                                    MyData dp = devList.First(x => ((x.ser_no == sd.SerialNo) && (x.addr == sd.IntAddress))); //devList[sd.SerialNo.ToString() + "_" + sd.IntAddress];
                                    if (dp != null)
                                    {
                                        byte CC1 = BytesHelper.ShortToBytes((ushort)dp.CC)[1];
                                        byte CC2 = BytesHelper.ShortToBytes((ushort)dp.CC)[0];
                                        sd.DataFromDevice = dp;

                                        sd.ConnectionState = ((byte)(CC2 >> 3) & (byte)0x1) == 0x1;
                                        if (!sd.PrevConnectionState)
                                        {
                                            if (sd.ConnectionState)
                                            {
                                                this.AddNewMessage2Console(new MessageCS(DateTime.Now, "L", "АЛС2." + sd.IntAddress + "." + sd.Driver.ShortName, "Потеря связи с АУ - сработка"));
                                            }
                                        }
                                        else
                                        {
                                            if (!sd.ConnectionState)
                                            {
                                                this.AddNewMessage2Console(new MessageCS(DateTime.Now, "L", "АЛС2." + sd.IntAddress + "." + sd.Driver.ShortName, "Потеря связи с АУ - сброшен"));
                                            }
                                        }
                                        sd.PrevConnectionState = sd.ConnectionState;

                                        sd.TestState = ((byte)(CC1 >> 4) & (byte)0x1) == 0x1;
                                        if (!sd.PrevTestState)
                                        {
                                            if (sd.TestState)
                                            {
                                                this.AddNewMessage2Console(new MessageCS(DateTime.Now, "M", "АЛС2." + sd.IntAddress + "." + sd.Driver.ShortName, "ТЕСТ - сработка"));
                                            }
                                        }
                                        else
                                        {
                                            if (!sd.TestState)
                                            {
                                                this.AddNewMessage2Console(new MessageCS(DateTime.Now, "M", "АЛС2." + sd.IntAddress + "." + sd.Driver.ShortName, "ТЕСТ - сброшен"));
                                            }
                                        }
                                        sd.PrevTestState = sd.TestState;

                                        sd.FailureState = ((byte)(CC1 >> 5) & (byte)0x1) == 0x1;
                                        if (!sd.PrevFailureState)
                                        {
                                            if (sd.FailureState)
                                            {
                                                this.AddNewMessage2Console(new MessageCS(DateTime.Now, "A", "АЛС2." + sd.IntAddress + "." + sd.Driver.ShortName, "НЕИСПРАВНОСТЬ - сработка"));
                                            }
                                        }
                                        else
                                        {
                                            if (!sd.FailureState)
                                            {
                                                this.AddNewMessage2Console(new MessageCS(DateTime.Now, "M", "АЛС2." + sd.IntAddress + "." + sd.Driver.ShortName, "НЕИСПРАВНОСТЬ - сброшен"));
                                            }
                                        }
                                        sd.PrevFailureState = sd.FailureState;

                                        sd.Fire1 = ((byte)(CC1 >> 2) & (byte)0x1) == 0x1;
                                        if (!sd.PrevFire1)
                                        {
                                            if (sd.Fire1)
                                            {
                                                this.AddNewMessage2Console(new MessageCS(DateTime.Now, "A", "АЛС2." + sd.IntAddress + "." + sd.Driver.ShortName, "ПОЖАР1 - сработка"));
                                            }
                                        }
                                        else
                                        {
                                            if (!sd.Fire1)
                                            {
                                                this.AddNewMessage2Console(new MessageCS(DateTime.Now, "M", "АЛС2." + sd.IntAddress + "." + sd.Driver.ShortName, "ПОЖАР1 - сброшен"));
                                            }
                                        }
                                        sd.PrevFire1 = sd.Fire1;

                                        sd.Fire2 = ((byte)(CC1 >> 3) & (byte)0x1) == 0x1;
                                        if (!sd.PrevFire2)
                                        {
                                            if (sd.Fire2)
                                            {
                                                this.AddNewMessage2Console(new MessageCS(DateTime.Now, "A", "АЛС2." + sd.IntAddress + "." + sd.Driver.ShortName, "ПОЖАР2 - сработка"));
                                            }
                                        }
                                        else
                                        {
                                            if (!sd.Fire2)
                                            {
                                                this.AddNewMessage2Console(new MessageCS(DateTime.Now, "M", "АЛС2." + sd.IntAddress + "." + sd.Driver.ShortName, "ПОЖАР2 - сброшен"));
                                            }
                                        }
                                        sd.PrevFire2 = sd.Fire2;

                                        sd.Mode1 = ((byte)(CC1 >> 0) & (byte)0x1) == 0x1;
                                        sd.Mode2 = ((byte)(CC1 >> 6) & (byte)0x1) == 0x1;

                                        sd.OnState = ((byte)(CC1 >> 7) & (byte)0x1) == 0x1;
                                        if (!sd.PrevOnState)
                                        {
                                            if (sd.OnState)
                                            {
                                                this.AddNewMessage2Console(new MessageCS(DateTime.Now, "M", "АЛС2." + sd.IntAddress + "." + sd.Driver.ShortName, "ВКЛЮЧЕНО - сработка"));
                                            }
                                        }
                                        else
                                        {
                                            if (!sd.OnState)
                                            {
                                                this.AddNewMessage2Console(new MessageCS(DateTime.Now, "M", "АЛС2." + sd.IntAddress + "." + sd.Driver.ShortName, "ВКЛЮЧЕНО - сброшен"));
                                            }
                                        }
                                        sd.PrevOnState = sd.OnState;

                                        sd.OffState = ((byte)(CC2 >> 0) & (byte)0x1) == 0x1;
                                        if (!sd.PrevOffState)
                                        {
                                            if (sd.OffState)
                                            {
                                                this.AddNewMessage2Console(new MessageCS(DateTime.Now, "M", "АЛС2." + sd.IntAddress + "." + sd.Driver.ShortName, "ОТКЛЮЧЕНО - сработка"));
                                            }
                                        }
                                        else
                                        {
                                            if (!sd.OffState)
                                            {
                                                this.AddNewMessage2Console(new MessageCS(DateTime.Now, "M", "АЛС2." + sd.IntAddress + "." + sd.Driver.ShortName, "ОТКЛЮЧЕНО - сброшен"));
                                            }
                                        }
                                        sd.PrevOffState = sd.OffState;

                                        sd.AddCCstate = BytesHelper.SubstructShort(dp.data, 2);

                                        sd.OnAction = ((byte)(CC2 >> 1) & (byte)0x1) == 0x1;
                                        if (!sd.PrevOnAction)
                                        {
                                            if (sd.OnAction)
                                            {
                                                this.AddNewMessage2Console(new MessageCS(DateTime.Now, "M", "АЛС2." + sd.IntAddress + "." + sd.Driver.ShortName, "ВКЛЮЧЕНИЕ - сработка"));
                                            }
                                        }
                                        else
                                        {
                                            if (!sd.OnAction)
                                            {
                                                this.AddNewMessage2Console(new MessageCS(DateTime.Now, "M", "АЛС2." + sd.IntAddress + "." + sd.Driver.ShortName, "ВКЛЮЧЕНИЕ - сброшен"));
                                            }
                                        }
                                        sd.PrevOnAction = sd.OnAction;

                                        sd.OffAction = ((byte)(CC2 >> 2) & (byte)0x1) == 0x1;
                                        if (!sd.PrevOffAction)
                                        {
                                            if (sd.OffAction)
                                            {
                                                this.AddNewMessage2Console(new MessageCS(DateTime.Now, "M", "АЛС2." + sd.IntAddress + "." + sd.Driver.ShortName, "ОТКЛЮЧЕНИЕ - сработка"));
                                            }
                                        }
                                        else
                                        {
                                            if (!sd.OffAction)
                                            {
                                                this.AddNewMessage2Console(new MessageCS(DateTime.Now, "M", "АЛС2." + sd.IntAddress + "." + sd.Driver.ShortName, "ОТКЛЮЧЕНИЕ - сброшен"));
                                            }
                                        }
                                        sd.PrevOffAction = sd.OffAction;
                                    }
                                }
                                catch { }
                                sd.deviceMessagesList.Reverse();
                                foreach (MessageCS ms in sd.deviceMessagesList)
                                {
                                    this.AddNewMessage2Console(ms);
                                }
                                sd.deviceMessagesList.Clear();
                            }
                        }
                        d.deviceMessagesList.Reverse();
                        foreach (MessageCS ms in d.deviceMessagesList)
                        {
                            this.AddNewMessage2Console(ms);
                        }
                        d.deviceMessagesList.Clear();
                    }
                    foreach (GKDevice d in this.ALS2RightSideDevices)
                    {
                        try
                        {
                            MyData dp = devList.First(x => ((x.ser_no == d.SerialNo) && (x.addr == d.IntAddress))); //devList[d.SerialNo.ToString() + "_" + d.IntAddress];
                            if (dp != null)
                            {
                                byte CC1 = BytesHelper.ShortToBytes((ushort)dp.CC)[1];
                                byte CC2 = BytesHelper.ShortToBytes((ushort)dp.CC)[0];
                                d.DataFromDevice = dp;

                                d.ConnectionState = ((byte)(CC2 >> 3) & (byte)0x1) == 0x1;
                                if (!d.PrevConnectionState)
                                {
                                    if (d.ConnectionState)
                                    {
                                        this.AddNewMessage2Console(new MessageCS(DateTime.Now, "L", "АЛС2." + d.IntAddress + "." + d.Driver.ShortName, "Потеря связи с АУ - сработка"));
                                    }
                                }
                                else
                                {
                                    if (!d.ConnectionState)
                                    {
                                        this.AddNewMessage2Console(new MessageCS(DateTime.Now, "L", "АЛС2." + d.IntAddress + "." + d.Driver.ShortName, "Потеря связи с АУ - сброшен"));
                                    }
                                }
                                d.PrevConnectionState = d.ConnectionState;

                                d.TestState = ((byte)(CC1 >> 4) & (byte)0x1) == 0x1;
                                if (!d.PrevTestState)
                                {
                                    if (d.TestState)
                                    {
                                        this.AddNewMessage2Console(new MessageCS(DateTime.Now, "M", "АЛС2." + d.IntAddress + "." + d.Driver.ShortName, "ТЕСТ - сработка"));
                                    }
                                }
                                else
                                {
                                    if (!d.TestState)
                                    {
                                        this.AddNewMessage2Console(new MessageCS(DateTime.Now, "M", "АЛС2." + d.IntAddress + "." + d.Driver.ShortName, "ТЕСТ - сброшен"));
                                    }
                                }
                                d.PrevTestState = d.TestState;

                                d.FailureState = ((byte)(CC1 >> 5) & (byte)0x1) == 0x1;
                                if (!d.PrevFailureState)
                                {
                                    if (d.FailureState)
                                    {
                                        this.AddNewMessage2Console(new MessageCS(DateTime.Now, "A", "АЛС2." + d.IntAddress + "." + d.Driver.ShortName, "НЕИСПРАВНОСТЬ - сработка"));
                                    }
                                }
                                else
                                {
                                    if (!d.FailureState)
                                    {
                                        this.AddNewMessage2Console(new MessageCS(DateTime.Now, "M", "АЛС2." + d.IntAddress + "." + d.Driver.ShortName, "НЕИСПРАВНОСТЬ - сброшен"));
                                    }
                                }
                                d.PrevFailureState = d.FailureState;

                                d.Fire1 = ((byte)(CC1 >> 2) & (byte)0x1) == 0x1;
                                if (!d.PrevFire1)
                                {
                                    if (d.Fire1)
                                    {
                                        this.AddNewMessage2Console(new MessageCS(DateTime.Now, "A", "АЛС2." + d.IntAddress + "." + d.Driver.ShortName, "ПОЖАР1 - сработка"));
                                    }
                                }
                                else
                                {
                                    if (!d.Fire1)
                                    {
                                        this.AddNewMessage2Console(new MessageCS(DateTime.Now, "M", "АЛС2." + d.IntAddress + "." + d.Driver.ShortName, "ПОЖАР1 - сброшен"));
                                    }
                                }
                                d.PrevFire1 = d.Fire1;

                                d.Fire2 = ((byte)(CC1 >> 3) & (byte)0x1) == 0x1;
                                if (!d.PrevFire2)
                                {
                                    if (d.Fire2)
                                    {
                                        this.AddNewMessage2Console(new MessageCS(DateTime.Now, "A", "АЛС2." + d.IntAddress + "." + d.Driver.ShortName, "ПОЖАР2 - сработка"));
                                    }
                                }
                                else
                                {
                                    if (!d.Fire2)
                                    {
                                        this.AddNewMessage2Console(new MessageCS(DateTime.Now, "M", "АЛС2." + d.IntAddress + "." + d.Driver.ShortName, "ПОЖАР2 - сброшен"));
                                    }
                                }
                                d.PrevFire2 = d.Fire2;

                                d.Mode1 = ((byte)(CC1 >> 0) & (byte)0x1) == 0x1;
                                d.Mode2 = ((byte)(CC1 >> 6) & (byte)0x1) == 0x1;

                                d.OnState = ((byte)(CC1 >> 7) & (byte)0x1) == 0x1;
                                if (!d.PrevOnState)
                                {
                                    if (d.OnState)
                                    {
                                        this.AddNewMessage2Console(new MessageCS(DateTime.Now, "M", "АЛС2." + d.IntAddress + "." + d.Driver.ShortName, "ВКЛЮЧЕНО - сработка"));
                                    }
                                }
                                else
                                {
                                    if (!d.OnState)
                                    {
                                        this.AddNewMessage2Console(new MessageCS(DateTime.Now, "M", "АЛС2." + d.IntAddress + "." + d.Driver.ShortName, "ВКЛЮЧЕНО - сброшен"));
                                    }
                                }
                                d.PrevOnState = d.OnState;

                                d.OffState = ((byte)(CC2 >> 0) & (byte)0x1) == 0x1;
                                if (!d.PrevOffState)
                                {
                                    if (d.OffState)
                                    {
                                        this.AddNewMessage2Console(new MessageCS(DateTime.Now, "M", "АЛС2." + d.IntAddress + "." + d.Driver.ShortName, "ОТКЛЮЧЕНО - сработка"));
                                    }
                                }
                                else
                                {
                                    if (!d.OffState)
                                    {
                                        this.AddNewMessage2Console(new MessageCS(DateTime.Now, "M", "АЛС2." + d.IntAddress + "." + d.Driver.ShortName, "ОТКЛЮЧЕНО - сброшен"));
                                    }
                                }
                                d.PrevOffState = d.OffState;

                                d.AddCCstate = BytesHelper.SubstructShort(dp.data, 2);

                                d.OnAction = ((byte)(CC2 >> 1) & (byte)0x1) == 0x1;
                                if (!d.PrevOnAction)
                                {
                                    if (d.OnAction)
                                    {
                                        this.AddNewMessage2Console(new MessageCS(DateTime.Now, "M", "АЛС2." + d.IntAddress + "." + d.Driver.ShortName, "ВКЛЮЧЕНИЕ - сработка"));
                                    }
                                }
                                else
                                {
                                    if (!d.OnAction)
                                    {
                                        this.AddNewMessage2Console(new MessageCS(DateTime.Now, "M", "АЛС2." + d.IntAddress + "." + d.Driver.ShortName, "ВКЛЮЧЕНИЕ - сброшен"));
                                    }
                                }
                                d.PrevOnAction = d.OnAction;

                                d.OffAction = ((byte)(CC2 >> 2) & (byte)0x1) == 0x1;
                                if (!d.PrevOffAction)
                                {
                                    if (d.OffAction)
                                    {
                                        this.AddNewMessage2Console(new MessageCS(DateTime.Now, "M", "АЛС2." + d.IntAddress + "." + d.Driver.ShortName, "ОТКЛЮЧЕНИЕ - сработка"));
                                    }
                                }
                                else
                                {
                                    if (!d.OffAction)
                                    {
                                        this.AddNewMessage2Console(new MessageCS(DateTime.Now, "M", "АЛС2." + d.IntAddress + "." + d.Driver.ShortName, "ОТКЛЮЧЕНИЕ - сброшен"));
                                    }
                                }
                                d.PrevOffAction = d.OffAction;
                            }
                        }
                        catch { }
                        if ((d.Devices != null) && (d.Devices.Count > 0))
                        {
                            foreach (GKDevice sd in d.Devices)
                            {
                                try
                                {
                                    //uint sn = uint.Parse();
                                    MyData dp = devList.First(x => ((x.ser_no == sd.SerialNo) && (x.addr == sd.IntAddress))); //devList[sd.SerialNo.ToString() + "_" + sd.IntAddress];
                                    if (dp != null)
                                    {
                                        byte CC1 = BytesHelper.ShortToBytes((ushort)dp.CC)[1];
                                        byte CC2 = BytesHelper.ShortToBytes((ushort)dp.CC)[0];
                                        sd.DataFromDevice = dp;

                                        sd.ConnectionState = ((byte)(CC2 >> 3) & (byte)0x1) == 0x1;
                                        if (!sd.PrevConnectionState)
                                        {
                                            if (sd.ConnectionState)
                                            {
                                                this.AddNewMessage2Console(new MessageCS(DateTime.Now, "L", "АЛС2." + sd.IntAddress + "." + sd.Driver.ShortName, "Потеря связи с АУ - сработка"));
                                            }
                                        }
                                        else
                                        {
                                            if (!sd.ConnectionState)
                                            {
                                                this.AddNewMessage2Console(new MessageCS(DateTime.Now, "L", "АЛС2." + sd.IntAddress + "." + sd.Driver.ShortName, "Потеря связи с АУ - сброшен"));
                                            }
                                        }
                                        sd.PrevConnectionState = sd.ConnectionState;

                                        sd.TestState = ((byte)(CC1 >> 4) & (byte)0x1) == 0x1;
                                        if (!sd.PrevTestState)
                                        {
                                            if (sd.TestState)
                                            {
                                                this.AddNewMessage2Console(new MessageCS(DateTime.Now, "M", "АЛС2." + sd.IntAddress + "." + sd.Driver.ShortName, "ТЕСТ - сработка"));
                                            }
                                        }
                                        else
                                        {
                                            if (!sd.TestState)
                                            {
                                                this.AddNewMessage2Console(new MessageCS(DateTime.Now, "M", "АЛС2." + sd.IntAddress + "." + sd.Driver.ShortName, "ТЕСТ - сброшен"));
                                            }
                                        }
                                        sd.PrevTestState = sd.TestState;

                                        sd.FailureState = ((byte)(CC1 >> 5) & (byte)0x1) == 0x1;
                                        if (!sd.PrevFailureState)
                                        {
                                            if (sd.FailureState)
                                            {
                                                this.AddNewMessage2Console(new MessageCS(DateTime.Now, "A", "АЛС2." + sd.IntAddress + "." + sd.Driver.ShortName, "НЕИСПРАВНОСТЬ - сработка"));
                                            }
                                        }
                                        else
                                        {
                                            if (!sd.FailureState)
                                            {
                                                this.AddNewMessage2Console(new MessageCS(DateTime.Now, "M", "АЛС2." + sd.IntAddress + "." + sd.Driver.ShortName, "НЕИСПРАВНОСТЬ - сброшен"));
                                            }
                                        }
                                        sd.PrevFailureState = sd.FailureState;

                                        sd.Fire1 = ((byte)(CC1 >> 2) & (byte)0x1) == 0x1;
                                        if (!sd.PrevFire1)
                                        {
                                            if (sd.Fire1)
                                            {
                                                this.AddNewMessage2Console(new MessageCS(DateTime.Now, "A", "АЛС2." + sd.IntAddress + "." + sd.Driver.ShortName, "ПОЖАР1 - сработка"));
                                            }
                                        }
                                        else
                                        {
                                            if (!sd.Fire1)
                                            {
                                                this.AddNewMessage2Console(new MessageCS(DateTime.Now, "M", "АЛС2." + sd.IntAddress + "." + sd.Driver.ShortName, "ПОЖАР1 - сброшен"));
                                            }
                                        }
                                        sd.PrevFire1 = sd.Fire1;

                                        sd.Fire2 = ((byte)(CC1 >> 3) & (byte)0x1) == 0x1;
                                        if (!sd.PrevFire2)
                                        {
                                            if (sd.Fire2)
                                            {
                                                this.AddNewMessage2Console(new MessageCS(DateTime.Now, "A", "АЛС2." + sd.IntAddress + "." + sd.Driver.ShortName, "ПОЖАР2 - сработка"));
                                            }
                                        }
                                        else
                                        {
                                            if (!sd.Fire2)
                                            {
                                                this.AddNewMessage2Console(new MessageCS(DateTime.Now, "M", "АЛС2." + sd.IntAddress + "." + sd.Driver.ShortName, "ПОЖАР2 - сброшен"));
                                            }
                                        }
                                        sd.PrevFire2 = sd.Fire2;

                                        sd.Mode1 = ((byte)(CC1 >> 0) & (byte)0x1) == 0x1;
                                        sd.Mode2 = ((byte)(CC1 >> 6) & (byte)0x1) == 0x1;

                                        sd.OnState = ((byte)(CC1 >> 7) & (byte)0x1) == 0x1;
                                        if (!sd.PrevOnState)
                                        {
                                            if (sd.OnState)
                                            {
                                                this.AddNewMessage2Console(new MessageCS(DateTime.Now, "M", "АЛС2." + sd.IntAddress + "." + sd.Driver.ShortName, "ВКЛЮЧЕНО - сработка"));
                                            }
                                        }
                                        else
                                        {
                                            if (!sd.OnState)
                                            {
                                                this.AddNewMessage2Console(new MessageCS(DateTime.Now, "M", "АЛС2." + sd.IntAddress + "." + sd.Driver.ShortName, "ВКЛЮЧЕНО - сброшен"));
                                            }
                                        }
                                        sd.PrevOnState = sd.OnState;

                                        sd.OffState = ((byte)(CC2 >> 0) & (byte)0x1) == 0x1;
                                        if (!sd.PrevOffState)
                                        {
                                            if (sd.OffState)
                                            {
                                                this.AddNewMessage2Console(new MessageCS(DateTime.Now, "M", "АЛС2." + sd.IntAddress + "." + sd.Driver.ShortName, "ОТКЛЮЧЕНО - сработка"));
                                            }
                                        }
                                        else
                                        {
                                            if (!sd.OffState)
                                            {
                                                this.AddNewMessage2Console(new MessageCS(DateTime.Now, "M", "АЛС2." + sd.IntAddress + "." + sd.Driver.ShortName, "ОТКЛЮЧЕНО - сброшен"));
                                            }
                                        }
                                        sd.PrevOffState = sd.OffState;

                                        sd.AddCCstate = BytesHelper.SubstructShort(dp.data, 2);

                                        sd.OnAction = ((byte)(CC2 >> 1) & (byte)0x1) == 0x1;
                                        if (!sd.PrevOnAction)
                                        {
                                            if (sd.OnAction)
                                            {
                                                this.AddNewMessage2Console(new MessageCS(DateTime.Now, "M", "АЛС2." + sd.IntAddress + "." + sd.Driver.ShortName, "ВКЛЮЧЕНИЕ - сработка"));
                                            }
                                        }
                                        else
                                        {
                                            if (!sd.OnAction)
                                            {
                                                this.AddNewMessage2Console(new MessageCS(DateTime.Now, "M", "АЛС2." + sd.IntAddress + "." + sd.Driver.ShortName, "ВКЛЮЧЕНИЕ - сброшен"));
                                            }
                                        }
                                        sd.PrevOnAction = sd.OnAction;

                                        sd.OffAction = ((byte)(CC2 >> 2) & (byte)0x1) == 0x1;
                                        if (!sd.PrevOffAction)
                                        {
                                            if (sd.OffAction)
                                            {
                                                this.AddNewMessage2Console(new MessageCS(DateTime.Now, "M", "АЛС2." + sd.IntAddress + "." + sd.Driver.ShortName, "ОТКЛЮЧЕНИЕ - сработка"));
                                            }
                                        }
                                        else
                                        {
                                            if (!sd.OffAction)
                                            {
                                                this.AddNewMessage2Console(new MessageCS(DateTime.Now, "M", "АЛС2." + sd.IntAddress + "." + sd.Driver.ShortName, "ОТКЛЮЧЕНИЕ - сброшен"));
                                            }
                                        }
                                        sd.PrevOffAction = sd.OffAction;
                                    }
                                }
                                catch { }
                                sd.deviceMessagesList.Reverse();
                                foreach (MessageCS ms in sd.deviceMessagesList)
                                {
                                    this.AddNewMessage2Console(ms);
                                }
                                sd.deviceMessagesList.Clear();
                            }
                        }
                        d.deviceMessagesList.Reverse();
                        foreach (MessageCS ms in d.deviceMessagesList)
                        {
                            this.AddNewMessage2Console(ms);
                        }
                        d.deviceMessagesList.Clear();
                    }
                    //MessageBox.Show("Параметры от МОПС успешно получены!", "Данные от МОПС");
                }
                #endregion
                Thread.Sleep(500);
            }
        }

        //Передача конфигурации в выбранный АУ
        private void SendConfigForSelectedDeviceBtn_Click(object sender, RoutedEventArgs e)
        {
            byte alsn = 0;
            if (this.ALSTabControl.SelectedIndex == 0)
                alsn = 1;
            else
                alsn = 2;
            SendManager.WriteDeviceParameters(4, SelectedDevice, alsn, 1, 1);
            //SendManager.FinalizeWritingDeviceParameters(4, alsn);
            MessageBox.Show("Запись уставок в АУ успешно завершена.", "Запись параметров в АУ");
        }

        //Команда №0
        private void Cntr0_Btn_Click(object sender, RoutedEventArgs e)
        {
            byte alsn = 0;
            if (this.ALSTabControl.SelectedIndex == 0)
                alsn = 1;
            else
                alsn = 2;

            if (this.SelectedDevice != null)
            {
                SendManager.WriteDeviceCommand(4, this.SelectedDevice, alsn, GKStateBit.Norm);
                this.AddNewMessage2Console(new MessageCS(DateTime.Now, "C", "АЛС" + alsn + "." + this.SelectedDevice.IntAddress + "." + this.SelectedDevice.Driver.ShortName, "Команда <(0) Включить автоматику>"));
            }
        }
        //Команда №1
        private void Cntr1_Btn_Click(object sender, RoutedEventArgs e)
        {
            byte alsn = 0;
            if (this.ALSTabControl.SelectedIndex == 0)
                alsn = 1;
            else
                alsn = 2;

            if (this.SelectedDevice != null)
            {
                SendManager.WriteDeviceCommand(4, this.SelectedDevice, alsn, GKStateBit.Attention);
                this.AddNewMessage2Console(new MessageCS(DateTime.Now, "C", "АЛС" + alsn + "." + this.SelectedDevice.IntAddress + "." + this.SelectedDevice.Driver.ShortName, "Команда <(1) В ручной режим>"));
            }
        }
        //Команда №2
        private void Cntr2_Btn_Click(object sender, RoutedEventArgs e)
        {
            byte alsn = 0;
            if (this.ALSTabControl.SelectedIndex == 0)
                alsn = 1;
            else
                alsn = 2;

            if (this.SelectedDevice != null)
            {
                SendManager.WriteDeviceCommand(4, this.SelectedDevice, alsn, GKStateBit.Fire1);
                this.AddNewMessage2Console(new MessageCS(DateTime.Now, "C", "АЛС" + alsn + "." + this.SelectedDevice.IntAddress + "." + this.SelectedDevice.Driver.ShortName, "Команда <(2) Отключить>"));
            }
        }
        //Команда №3
        private void Cntr3_Btn_Click(object sender, RoutedEventArgs e)
        {
            byte alsn = 0;
            if (this.ALSTabControl.SelectedIndex == 0)
                alsn = 1;
            else
                alsn = 2;

            if (this.SelectedDevice != null)
            {
                SendManager.WriteDeviceCommand(4, this.SelectedDevice, alsn, GKStateBit.Fire2);
                this.AddNewMessage2Console(new MessageCS(DateTime.Now, "C", "АЛС" + alsn + "." + this.SelectedDevice.IntAddress + "." + this.SelectedDevice.Driver.ShortName, "Команда <(3) Снять тест>"));
            }
        }
        //Команда №4
        private void Cntr4_Btn_Click(object sender, RoutedEventArgs e)
        {
            byte alsn = 0;
            if (this.ALSTabControl.SelectedIndex == 0)
                alsn = 1;
            else
                alsn = 2;

            if (this.SelectedDevice != null)
            {
                SendManager.WriteDeviceCommand(4, this.SelectedDevice, alsn, GKStateBit.Test);
                this.AddNewMessage2Console(new MessageCS(DateTime.Now, "C", "АЛС" + alsn + "." + this.SelectedDevice.IntAddress + "." + this.SelectedDevice.Driver.ShortName, "Команда <(4) Сброс>"));
            }
        }
        //Команда №5
        private void Cntr5_Btn_Click(object sender, RoutedEventArgs e)
        {
            byte alsn = 0;
            if (this.ALSTabControl.SelectedIndex == 0)
                alsn = 1;
            else
                alsn = 2;

            if (this.SelectedDevice != null)
            {
                SendManager.WriteDeviceCommand(4, this.SelectedDevice, alsn, GKStateBit.Failure);
                this.AddNewMessage2Console(new MessageCS(DateTime.Now, "C", "АЛС" + alsn + "." + this.SelectedDevice.IntAddress + "." + this.SelectedDevice.Driver.ShortName, "Команда <(5)>"));
            }
        }
        //Команда №6
        private void Cntr6_Btn_Click(object sender, RoutedEventArgs e)
        {
            byte alsn = 0;
            if (this.ALSTabControl.SelectedIndex == 0)
                alsn = 1;
            else
                alsn = 2;

            if (this.SelectedDevice != null)
            {
                SendManager.WriteDeviceCommand(4, this.SelectedDevice, alsn, GKStateBit.Ignore);
                this.AddNewMessage2Console(new MessageCS(DateTime.Now, "C", "АЛС" + alsn + "." + this.SelectedDevice.IntAddress + "." + this.SelectedDevice.Driver.ShortName, "Команда <(6)>"));
            }
        }
        //Команда №7
        private void Cntr7_Btn_Click(object sender, RoutedEventArgs e)
        {
            byte alsn = 0;
            if (this.ALSTabControl.SelectedIndex == 0)
                alsn = 1;
            else
                alsn = 2;

            if (this.SelectedDevice != null)
            {
                SendManager.WriteDeviceCommand(4, this.SelectedDevice, alsn, GKStateBit.On);
                this.AddNewMessage2Console(new MessageCS(DateTime.Now, "C", "АЛС" + alsn + "." + this.SelectedDevice.IntAddress + "." + this.SelectedDevice.Driver.ShortName, "Команда <(7) Остановить (руч. режим)>"));
            }
        }
        //Команда №8
        private void Cntr8_Btn_Click(object sender, RoutedEventArgs e)
        {
            byte alsn = 0;
            if (this.ALSTabControl.SelectedIndex == 0)
                alsn = 1;
            else
                alsn = 2;

            if (this.SelectedDevice != null)
            {
                SendManager.WriteDeviceCommand(4, this.SelectedDevice, alsn, GKStateBit.Off);
                this.AddNewMessage2Console(new MessageCS(DateTime.Now, "C", "АЛС" + alsn + "." + this.SelectedDevice.IntAddress + "." + this.SelectedDevice.Driver.ShortName, "Команда <(8) Включить (авт.включена)>"));
            }
        }
        //Команда №9
        private void Cntr9_Btn_Click(object sender, RoutedEventArgs e)
        {
            byte alsn = 0;
            if (this.ALSTabControl.SelectedIndex == 0)
                alsn = 1;
            else
                alsn = 2;

            if (this.SelectedDevice != null)
            {
                SendManager.WriteDeviceCommand(4, this.SelectedDevice, alsn, GKStateBit.TurningOn);
                this.AddNewMessage2Console(new MessageCS(DateTime.Now, "C", "АЛС" + alsn + "." + this.SelectedDevice.IntAddress + "." + this.SelectedDevice.Driver.ShortName, "Команда <(9) Выключить (авт.включена)>"));
            }
        }
        //Команда №10
        private void Cntr10_Btn_Click(object sender, RoutedEventArgs e)
        {
            byte alsn = 0;
            if (this.ALSTabControl.SelectedIndex == 0)
                alsn = 1;
            else
                alsn = 2;

            if (this.SelectedDevice != null)
            {
                SendManager.WriteDeviceCommand(4, this.SelectedDevice, alsn, GKStateBit.TurningOff);
                this.AddNewMessage2Console(new MessageCS(DateTime.Now, "C", "АЛС" + alsn + "." + this.SelectedDevice.IntAddress + "." + this.SelectedDevice.Driver.ShortName, "Команда <(10) Вкл.немедленно (авт.вкл.)>"));
            }
        }
        //Команда №11
        private void Cntr11_Btn_Click(object sender, RoutedEventArgs e)
        {
            byte alsn = 0;
            if (this.ALSTabControl.SelectedIndex == 0)
                alsn = 1;
            else
                alsn = 2;

            if (this.SelectedDevice != null)
            {
                SendManager.WriteDeviceCommand(4, this.SelectedDevice, alsn, GKStateBit.TurnOn_InAutomatic);
                this.AddNewMessage2Console(new MessageCS(DateTime.Now, "C", "АЛС" + alsn + "." + this.SelectedDevice.IntAddress + "." + this.SelectedDevice.Driver.ShortName, "Команда <(11) Выкл.немедленно (авт.вкл.)>"));
            }
        }
        //Команда №12
        private void Cntr12_Btn_Click(object sender, RoutedEventArgs e)
        {
            byte alsn = 0;
            if (this.ALSTabControl.SelectedIndex == 0)
                alsn = 1;
            else
                alsn = 2;

            if (this.SelectedDevice != null)
            {
                SendManager.WriteDeviceCommand(4, this.SelectedDevice, alsn, GKStateBit.Reserve1);
                this.AddNewMessage2Console(new MessageCS(DateTime.Now, "C", "АЛС" + alsn + "." + this.SelectedDevice.IntAddress + "." + this.SelectedDevice.Driver.ShortName, "Команда <(12) Вкл. (ручное управление)>"));
            }
        }
        //Команда №13
        private void Cntr13_Btn_Click(object sender, RoutedEventArgs e)
        {
            byte alsn = 0;
            if (this.ALSTabControl.SelectedIndex == 0)
                alsn = 1;
            else
                alsn = 2;

            if (this.SelectedDevice != null)
            {
                SendManager.WriteDeviceCommand(4, this.SelectedDevice, alsn, GKStateBit.TurnOff_InAutomatic);
                this.AddNewMessage2Console(new MessageCS(DateTime.Now, "C", "АЛС" + alsn + "." + this.SelectedDevice.IntAddress + "." + this.SelectedDevice.Driver.ShortName, "Команда <(13) Выкл. (ручное управление)>"));
            }
        }
        //Команда №14
        private void Cntr14_Btn_Click(object sender, RoutedEventArgs e)
        {
            byte alsn = 0;
            if (this.ALSTabControl.SelectedIndex == 0)
                alsn = 1;
            else
                alsn = 2;

            if (this.SelectedDevice != null)
            {
                SendManager.WriteDeviceCommand(4, this.SelectedDevice, alsn, GKStateBit.Stop_InManual);
                this.AddNewMessage2Console(new MessageCS(DateTime.Now, "C", "АЛС" + alsn + "." + this.SelectedDevice.IntAddress + "." + this.SelectedDevice.Driver.ShortName, "Команда <(14) Вкл.немедленно (руч.упр.)>"));
            }
        }
        //Команда №15
        private void Cntr15_Btn_Click(object sender, RoutedEventArgs e)
        {
            byte alsn = 0;
            if (this.ALSTabControl.SelectedIndex == 0)
                alsn = 1;
            else
                alsn = 2;

            if (this.SelectedDevice != null)
            {
                SendManager.WriteDeviceCommand(4, this.SelectedDevice, alsn, GKStateBit.ForbidStart_InManual);
                this.AddNewMessage2Console(new MessageCS(DateTime.Now, "C", "АЛС" + alsn + "." + this.SelectedDevice.IntAddress + "." + this.SelectedDevice.Driver.ShortName, "Команда <(15) Откл.немедленно (руч.упр.)>"));
            }
        }

        private void SetVisibilityForCommzndButtons()
        {
            switch (this.SelectedDevice.Driver.DriverTypeNo)
            {
                case 0xD0:
                    #region
                    this.Cntr0_Btn.Visibility = Visibility.Visible;
                    this.Cntr1_Btn.Visibility = Visibility.Visible;
                    this.Cntr2_Btn.Visibility = Visibility.Visible;
                    this.Cntr3_Btn.Visibility = Visibility.Visible;
                    this.Cntr4_Btn.Visibility = Visibility.Visible;
                    this.Cntr7_Btn.Visibility = Visibility.Visible;

                    this.Cntr8_Btn.Visibility = Visibility.Hidden;
                    this.Cntr9_Btn.Visibility = Visibility.Hidden;
                    this.Cntr10_Btn.Visibility = Visibility.Hidden;
                    this.Cntr11_Btn.Visibility = Visibility.Hidden;
                    this.Cntr12_Btn.Visibility = Visibility.Hidden;
                    this.Cntr13_Btn.Visibility = Visibility.Hidden;
                    this.Cntr14_Btn.Visibility = Visibility.Hidden;
                    this.Cntr15_Btn.Visibility = Visibility.Hidden;
                    #endregion
                    break;
                case 0xCE:
                    #region
                    this.Cntr0_Btn.Visibility = Visibility.Visible;
                    this.Cntr1_Btn.Visibility = Visibility.Hidden;
                    this.Cntr2_Btn.Visibility = Visibility.Visible;
                    this.Cntr3_Btn.Visibility = Visibility.Visible;
                    this.Cntr4_Btn.Visibility = Visibility.Visible;
                    this.Cntr7_Btn.Visibility = Visibility.Hidden;

                    this.Cntr8_Btn.Visibility = Visibility.Hidden;
                    this.Cntr9_Btn.Visibility = Visibility.Hidden;
                    this.Cntr10_Btn.Visibility = Visibility.Hidden;
                    this.Cntr11_Btn.Visibility = Visibility.Hidden;
                    this.Cntr12_Btn.Visibility = Visibility.Hidden;
                    this.Cntr13_Btn.Visibility = Visibility.Hidden;
                    this.Cntr14_Btn.Visibility = Visibility.Hidden;
                    this.Cntr15_Btn.Visibility = Visibility.Hidden;
                    #endregion
                    break;
                case 0xCA:
                    #region
                    this.Cntr0_Btn.Visibility = Visibility.Visible;
                    this.Cntr1_Btn.Visibility = Visibility.Hidden;
                    this.Cntr2_Btn.Visibility = Visibility.Visible;
                    this.Cntr3_Btn.Visibility = Visibility.Visible;
                    this.Cntr4_Btn.Visibility = Visibility.Visible;
                    this.Cntr7_Btn.Visibility = Visibility.Hidden;

                    this.Cntr8_Btn.Visibility = Visibility.Hidden;
                    this.Cntr9_Btn.Visibility = Visibility.Hidden;
                    this.Cntr10_Btn.Visibility = Visibility.Hidden;
                    this.Cntr11_Btn.Visibility = Visibility.Hidden;
                    this.Cntr12_Btn.Visibility = Visibility.Hidden;
                    this.Cntr13_Btn.Visibility = Visibility.Hidden;
                    this.Cntr14_Btn.Visibility = Visibility.Hidden;
                    this.Cntr15_Btn.Visibility = Visibility.Hidden;
                    #endregion
                    break;
                case 0xC5:
                    #region
                    this.Cntr0_Btn.Visibility = Visibility.Visible;
                    this.Cntr1_Btn.Visibility = Visibility.Hidden;
                    this.Cntr2_Btn.Visibility = Visibility.Visible;
                    this.Cntr3_Btn.Visibility = Visibility.Visible;
                    this.Cntr4_Btn.Visibility = Visibility.Visible;
                    this.Cntr7_Btn.Visibility = Visibility.Hidden;

                    this.Cntr8_Btn.Visibility = Visibility.Hidden;
                    this.Cntr9_Btn.Visibility = Visibility.Hidden;
                    this.Cntr10_Btn.Visibility = Visibility.Hidden;
                    this.Cntr11_Btn.Visibility = Visibility.Hidden;
                    this.Cntr12_Btn.Visibility = Visibility.Hidden;
                    this.Cntr13_Btn.Visibility = Visibility.Hidden;
                    this.Cntr14_Btn.Visibility = Visibility.Hidden;
                    this.Cntr15_Btn.Visibility = Visibility.Hidden;
                    #endregion
                    break;
                case 0x16:
                    #region
                    this.Cntr0_Btn.Visibility = Visibility.Visible;
                    this.Cntr1_Btn.Visibility = Visibility.Hidden;
                    this.Cntr2_Btn.Visibility = Visibility.Visible;
                    this.Cntr3_Btn.Visibility = Visibility.Hidden;
                    this.Cntr4_Btn.Visibility = Visibility.Hidden;
                    this.Cntr7_Btn.Visibility = Visibility.Hidden;

                    this.Cntr8_Btn.Visibility = Visibility.Hidden;
                    this.Cntr9_Btn.Visibility = Visibility.Hidden;
                    this.Cntr10_Btn.Visibility = Visibility.Hidden;
                    this.Cntr11_Btn.Visibility = Visibility.Hidden;
                    this.Cntr12_Btn.Visibility = Visibility.Hidden;
                    this.Cntr13_Btn.Visibility = Visibility.Hidden;
                    this.Cntr14_Btn.Visibility = Visibility.Hidden;
                    this.Cntr15_Btn.Visibility = Visibility.Hidden;
                    #endregion
                    break;
                case 0xD8:
                    #region
                    this.Cntr0_Btn.Visibility = Visibility.Visible;
                    this.Cntr1_Btn.Visibility = Visibility.Hidden;
                    this.Cntr2_Btn.Visibility = Visibility.Visible;
                    this.Cntr3_Btn.Visibility = Visibility.Hidden;
                    this.Cntr4_Btn.Visibility = Visibility.Hidden;
                    this.Cntr7_Btn.Visibility = Visibility.Hidden;

                    this.Cntr8_Btn.Visibility = Visibility.Hidden;
                    this.Cntr9_Btn.Visibility = Visibility.Hidden;
                    this.Cntr10_Btn.Visibility = Visibility.Hidden;
                    this.Cntr11_Btn.Visibility = Visibility.Hidden;
                    this.Cntr12_Btn.Visibility = Visibility.Hidden;
                    this.Cntr13_Btn.Visibility = Visibility.Hidden;
                    this.Cntr14_Btn.Visibility = Visibility.Hidden;
                    this.Cntr15_Btn.Visibility = Visibility.Hidden;
                    #endregion
                    break;
                case 0xD9:
                    #region
                    this.Cntr0_Btn.Visibility = Visibility.Visible;
                    this.Cntr1_Btn.Visibility = Visibility.Hidden;
                    this.Cntr2_Btn.Visibility = Visibility.Visible;
                    this.Cntr3_Btn.Visibility = Visibility.Visible;
                    this.Cntr4_Btn.Visibility = Visibility.Hidden;
                    this.Cntr7_Btn.Visibility = Visibility.Hidden;

                    this.Cntr8_Btn.Visibility = Visibility.Hidden;
                    this.Cntr9_Btn.Visibility = Visibility.Hidden;
                    this.Cntr10_Btn.Visibility = Visibility.Hidden;
                    this.Cntr11_Btn.Visibility = Visibility.Hidden;
                    this.Cntr12_Btn.Visibility = Visibility.Hidden;
                    this.Cntr13_Btn.Visibility = Visibility.Hidden;
                    this.Cntr14_Btn.Visibility = Visibility.Hidden;
                    this.Cntr15_Btn.Visibility = Visibility.Hidden;
                    #endregion
                    break;
                case 0xDA:
                    #region
                    this.Cntr0_Btn.Visibility = Visibility.Visible;
                    this.Cntr1_Btn.Visibility = Visibility.Visible;
                    this.Cntr2_Btn.Visibility = Visibility.Visible;
                    this.Cntr3_Btn.Visibility = Visibility.Visible;
                    this.Cntr4_Btn.Visibility = Visibility.Hidden;
                    this.Cntr7_Btn.Visibility = Visibility.Hidden;

                    this.Cntr8_Btn.Visibility = Visibility.Visible;
                    this.Cntr9_Btn.Visibility = Visibility.Visible;
                    this.Cntr10_Btn.Visibility = Visibility.Visible;
                    this.Cntr11_Btn.Visibility = Visibility.Visible;
                    this.Cntr12_Btn.Visibility = Visibility.Visible;
                    this.Cntr13_Btn.Visibility = Visibility.Visible;
                    this.Cntr14_Btn.Visibility = Visibility.Visible;
                    this.Cntr15_Btn.Visibility = Visibility.Visible;
                    #endregion
                    break;
                case 0xDB:
                    #region
                    this.Cntr0_Btn.Visibility = Visibility.Visible;
                    this.Cntr1_Btn.Visibility = Visibility.Hidden;
                    this.Cntr2_Btn.Visibility = Visibility.Visible;
                    this.Cntr3_Btn.Visibility = Visibility.Visible;
                    this.Cntr4_Btn.Visibility = Visibility.Hidden;
                    this.Cntr7_Btn.Visibility = Visibility.Hidden;

                    this.Cntr8_Btn.Visibility = Visibility.Hidden;
                    this.Cntr9_Btn.Visibility = Visibility.Hidden;
                    this.Cntr10_Btn.Visibility = Visibility.Hidden;
                    this.Cntr11_Btn.Visibility = Visibility.Hidden;
                    this.Cntr12_Btn.Visibility = Visibility.Hidden;
                    this.Cntr13_Btn.Visibility = Visibility.Hidden;
                    this.Cntr14_Btn.Visibility = Visibility.Hidden;
                    this.Cntr15_Btn.Visibility = Visibility.Hidden;
                    #endregion
                    break;
                case 0xDC:
                    #region
                    this.Cntr0_Btn.Visibility = Visibility.Visible;
                    this.Cntr1_Btn.Visibility = Visibility.Visible;
                    this.Cntr2_Btn.Visibility = Visibility.Visible;
                    this.Cntr3_Btn.Visibility = Visibility.Visible;
                    this.Cntr4_Btn.Visibility = Visibility.Hidden;
                    this.Cntr7_Btn.Visibility = Visibility.Visible;

                    this.Cntr8_Btn.Visibility = Visibility.Visible;
                    this.Cntr9_Btn.Visibility = Visibility.Visible;
                    this.Cntr10_Btn.Visibility = Visibility.Visible;
                    this.Cntr11_Btn.Visibility = Visibility.Visible;
                    this.Cntr12_Btn.Visibility = Visibility.Visible;
                    this.Cntr13_Btn.Visibility = Visibility.Visible;
                    this.Cntr14_Btn.Visibility = Visibility.Visible;
                    this.Cntr15_Btn.Visibility = Visibility.Visible;
                    #endregion
                    break;
                case 0xDD:
                    #region
                    this.Cntr0_Btn.Visibility = Visibility.Visible;
                    this.Cntr1_Btn.Visibility = Visibility.Hidden;
                    this.Cntr2_Btn.Visibility = Visibility.Visible;
                    this.Cntr3_Btn.Visibility = Visibility.Visible;
                    this.Cntr4_Btn.Visibility = Visibility.Visible;
                    this.Cntr7_Btn.Visibility = Visibility.Hidden;

                    this.Cntr8_Btn.Visibility = Visibility.Hidden;
                    this.Cntr9_Btn.Visibility = Visibility.Hidden;
                    this.Cntr10_Btn.Visibility = Visibility.Hidden;
                    this.Cntr11_Btn.Visibility = Visibility.Hidden;
                    this.Cntr12_Btn.Visibility = Visibility.Hidden;
                    this.Cntr13_Btn.Visibility = Visibility.Hidden;
                    this.Cntr14_Btn.Visibility = Visibility.Hidden;
                    this.Cntr15_Btn.Visibility = Visibility.Hidden;
                    #endregion
                    break;
                case 0xDE:
                    #region
                    this.Cntr0_Btn.Visibility = Visibility.Visible;
                    this.Cntr1_Btn.Visibility = Visibility.Hidden;
                    this.Cntr2_Btn.Visibility = Visibility.Visible;
                    this.Cntr3_Btn.Visibility = Visibility.Visible;
                    this.Cntr4_Btn.Visibility = Visibility.Visible;
                    this.Cntr7_Btn.Visibility = Visibility.Hidden;

                    this.Cntr8_Btn.Visibility = Visibility.Hidden;
                    this.Cntr9_Btn.Visibility = Visibility.Hidden;
                    this.Cntr10_Btn.Visibility = Visibility.Hidden;
                    this.Cntr11_Btn.Visibility = Visibility.Hidden;
                    this.Cntr12_Btn.Visibility = Visibility.Hidden;
                    this.Cntr13_Btn.Visibility = Visibility.Hidden;
                    this.Cntr14_Btn.Visibility = Visibility.Hidden;
                    this.Cntr15_Btn.Visibility = Visibility.Hidden;
                    #endregion
                    break;
                case 0xDF:
                    #region
                    this.Cntr0_Btn.Visibility = Visibility.Visible;
                    this.Cntr1_Btn.Visibility = Visibility.Hidden;
                    this.Cntr2_Btn.Visibility = Visibility.Visible;
                    this.Cntr3_Btn.Visibility = Visibility.Visible;
                    this.Cntr4_Btn.Visibility = Visibility.Visible;
                    this.Cntr7_Btn.Visibility = Visibility.Hidden;

                    this.Cntr8_Btn.Visibility = Visibility.Hidden;
                    this.Cntr9_Btn.Visibility = Visibility.Hidden;
                    this.Cntr10_Btn.Visibility = Visibility.Hidden;
                    this.Cntr11_Btn.Visibility = Visibility.Hidden;
                    this.Cntr12_Btn.Visibility = Visibility.Hidden;
                    this.Cntr13_Btn.Visibility = Visibility.Hidden;
                    this.Cntr14_Btn.Visibility = Visibility.Hidden;
                    this.Cntr15_Btn.Visibility = Visibility.Hidden;
                    #endregion
                    break;
                case 0xE0:
                    #region
                    this.Cntr0_Btn.Visibility = Visibility.Visible;
                    this.Cntr1_Btn.Visibility = Visibility.Visible;
                    this.Cntr2_Btn.Visibility = Visibility.Visible;
                    this.Cntr3_Btn.Visibility = Visibility.Visible;
                    this.Cntr4_Btn.Visibility = Visibility.Hidden;
                    this.Cntr7_Btn.Visibility = Visibility.Hidden;

                    this.Cntr8_Btn.Visibility = Visibility.Visible;
                    this.Cntr9_Btn.Visibility = Visibility.Visible;
                    this.Cntr10_Btn.Visibility = Visibility.Visible;
                    this.Cntr11_Btn.Visibility = Visibility.Visible;
                    this.Cntr12_Btn.Visibility = Visibility.Visible;
                    this.Cntr13_Btn.Visibility = Visibility.Visible;
                    this.Cntr14_Btn.Visibility = Visibility.Visible;
                    this.Cntr15_Btn.Visibility = Visibility.Visible;
                    #endregion
                    break;
                case 0xE1:
                    #region
                    this.Cntr0_Btn.Visibility = Visibility.Visible;
                    this.Cntr1_Btn.Visibility = Visibility.Hidden;
                    this.Cntr2_Btn.Visibility = Visibility.Visible;
                    this.Cntr3_Btn.Visibility = Visibility.Visible;
                    this.Cntr4_Btn.Visibility = Visibility.Visible;
                    this.Cntr7_Btn.Visibility = Visibility.Hidden;

                    this.Cntr8_Btn.Visibility = Visibility.Hidden;
                    this.Cntr9_Btn.Visibility = Visibility.Hidden;
                    this.Cntr10_Btn.Visibility = Visibility.Hidden;
                    this.Cntr11_Btn.Visibility = Visibility.Hidden;
                    this.Cntr12_Btn.Visibility = Visibility.Hidden;
                    this.Cntr13_Btn.Visibility = Visibility.Hidden;
                    this.Cntr14_Btn.Visibility = Visibility.Hidden;
                    this.Cntr15_Btn.Visibility = Visibility.Hidden;
                    #endregion
                    break;
                case 0xE2:
                    #region
                    this.Cntr0_Btn.Visibility = Visibility.Visible;
                    this.Cntr1_Btn.Visibility = Visibility.Visible;
                    this.Cntr2_Btn.Visibility = Visibility.Visible;
                    this.Cntr3_Btn.Visibility = Visibility.Visible;
                    this.Cntr4_Btn.Visibility = Visibility.Hidden;
                    this.Cntr7_Btn.Visibility = Visibility.Hidden;

                    this.Cntr8_Btn.Visibility = Visibility.Visible;
                    this.Cntr9_Btn.Visibility = Visibility.Visible;
                    this.Cntr10_Btn.Visibility = Visibility.Visible;
                    this.Cntr11_Btn.Visibility = Visibility.Visible;
                    this.Cntr12_Btn.Visibility = Visibility.Visible;
                    this.Cntr13_Btn.Visibility = Visibility.Visible;
                    this.Cntr14_Btn.Visibility = Visibility.Visible;
                    this.Cntr15_Btn.Visibility = Visibility.Visible;
                    #endregion
                    break;
                case 0xE3:
                    #region
                    this.Cntr0_Btn.Visibility = Visibility.Visible;
                    this.Cntr1_Btn.Visibility = Visibility.Visible;
                    this.Cntr2_Btn.Visibility = Visibility.Visible;
                    this.Cntr3_Btn.Visibility = Visibility.Visible;
                    this.Cntr4_Btn.Visibility = Visibility.Hidden;
                    this.Cntr7_Btn.Visibility = Visibility.Hidden;

                    this.Cntr8_Btn.Visibility = Visibility.Visible;
                    this.Cntr9_Btn.Visibility = Visibility.Visible;
                    this.Cntr10_Btn.Visibility = Visibility.Visible;
                    this.Cntr11_Btn.Visibility = Visibility.Visible;
                    this.Cntr12_Btn.Visibility = Visibility.Visible;
                    this.Cntr13_Btn.Visibility = Visibility.Visible;
                    this.Cntr14_Btn.Visibility = Visibility.Visible;
                    this.Cntr15_Btn.Visibility = Visibility.Visible;
                    #endregion
                    break;
                case 0xEB:
                case 0xEC:
                    #region
                    this.Cntr0_Btn.Visibility = Visibility.Visible;
                    this.Cntr1_Btn.Visibility = Visibility.Visible;
                    this.Cntr2_Btn.Visibility = Visibility.Visible;
                    this.Cntr3_Btn.Visibility = Visibility.Visible;
                    this.Cntr4_Btn.Visibility = Visibility.Hidden;
                    this.Cntr7_Btn.Visibility = Visibility.Hidden;

                    this.Cntr8_Btn.Visibility = Visibility.Visible;
                    this.Cntr9_Btn.Visibility = Visibility.Visible;
                    this.Cntr10_Btn.Visibility = Visibility.Visible;
                    this.Cntr11_Btn.Visibility = Visibility.Visible;
                    this.Cntr12_Btn.Visibility = Visibility.Visible;
                    this.Cntr13_Btn.Visibility = Visibility.Visible;
                    this.Cntr14_Btn.Visibility = Visibility.Visible;
                    this.Cntr15_Btn.Visibility = Visibility.Visible;
                    #endregion
                    break;
                case 0xE5:
                    #region
                    this.Cntr0_Btn.Visibility = Visibility.Visible;
                    this.Cntr1_Btn.Visibility = Visibility.Hidden;
                    this.Cntr2_Btn.Visibility = Visibility.Visible;
                    this.Cntr3_Btn.Visibility = Visibility.Visible;
                    this.Cntr4_Btn.Visibility = Visibility.Hidden;
                    this.Cntr7_Btn.Visibility = Visibility.Hidden;

                    this.Cntr8_Btn.Visibility = Visibility.Visible;
                    this.Cntr9_Btn.Visibility = Visibility.Visible;
                    this.Cntr10_Btn.Visibility = Visibility.Hidden;
                    this.Cntr11_Btn.Visibility = Visibility.Hidden;
                    this.Cntr12_Btn.Visibility = Visibility.Hidden;
                    this.Cntr13_Btn.Visibility = Visibility.Hidden;
                    this.Cntr14_Btn.Visibility = Visibility.Hidden;
                    this.Cntr15_Btn.Visibility = Visibility.Hidden;
                    #endregion
                    break;
                case 0xD4:
                    #region
                    this.Cntr0_Btn.Visibility = Visibility.Visible;
                    this.Cntr1_Btn.Visibility = Visibility.Hidden;
                    this.Cntr2_Btn.Visibility = Visibility.Visible;
                    this.Cntr3_Btn.Visibility = Visibility.Visible;
                    this.Cntr4_Btn.Visibility = Visibility.Hidden;
                    this.Cntr7_Btn.Visibility = Visibility.Hidden;

                    this.Cntr8_Btn.Visibility = Visibility.Hidden;
                    this.Cntr9_Btn.Visibility = Visibility.Hidden;
                    this.Cntr10_Btn.Visibility = Visibility.Hidden;
                    this.Cntr11_Btn.Visibility = Visibility.Hidden;
                    this.Cntr12_Btn.Visibility = Visibility.Hidden;
                    this.Cntr13_Btn.Visibility = Visibility.Hidden;
                    this.Cntr14_Btn.Visibility = Visibility.Hidden;
                    this.Cntr15_Btn.Visibility = Visibility.Hidden;
                    #endregion
                    break;
                case 0xE6:
                    #region
                    this.Cntr0_Btn.Visibility = Visibility.Visible;
                    this.Cntr1_Btn.Visibility = Visibility.Visible;
                    this.Cntr2_Btn.Visibility = Visibility.Visible;
                    this.Cntr3_Btn.Visibility = Visibility.Visible;
                    this.Cntr4_Btn.Visibility = Visibility.Hidden;
                    this.Cntr7_Btn.Visibility = Visibility.Hidden;

                    this.Cntr8_Btn.Visibility = Visibility.Visible;
                    this.Cntr9_Btn.Visibility = Visibility.Visible;
                    this.Cntr10_Btn.Visibility = Visibility.Visible;
                    this.Cntr11_Btn.Visibility = Visibility.Visible;
                    this.Cntr12_Btn.Visibility = Visibility.Visible;
                    this.Cntr13_Btn.Visibility = Visibility.Visible;
                    this.Cntr14_Btn.Visibility = Visibility.Visible;
                    this.Cntr15_Btn.Visibility = Visibility.Visible;
                    #endregion
                    break;
                case 0xE7:
                    #region
                    this.Cntr0_Btn.Visibility = Visibility.Visible;
                    this.Cntr1_Btn.Visibility = Visibility.Visible;
                    this.Cntr2_Btn.Visibility = Visibility.Visible;
                    this.Cntr3_Btn.Visibility = Visibility.Visible;
                    this.Cntr4_Btn.Visibility = Visibility.Hidden;
                    this.Cntr7_Btn.Visibility = Visibility.Visible;

                    this.Cntr8_Btn.Visibility = Visibility.Visible;
                    this.Cntr9_Btn.Visibility = Visibility.Visible;
                    this.Cntr10_Btn.Visibility = Visibility.Hidden;
                    this.Cntr11_Btn.Visibility = Visibility.Hidden;
                    this.Cntr12_Btn.Visibility = Visibility.Hidden;
                    this.Cntr13_Btn.Visibility = Visibility.Hidden;
                    this.Cntr14_Btn.Visibility = Visibility.Hidden;
                    this.Cntr15_Btn.Visibility = Visibility.Hidden;
                    #endregion
                    break;
                case 0x10:
                    #region
                    this.Cntr0_Btn.Visibility = Visibility.Visible;
                    this.Cntr1_Btn.Visibility = Visibility.Visible;
                    this.Cntr2_Btn.Visibility = Visibility.Visible;
                    this.Cntr3_Btn.Visibility = Visibility.Visible;
                    this.Cntr4_Btn.Visibility = Visibility.Hidden;
                    this.Cntr7_Btn.Visibility = Visibility.Hidden;

                    this.Cntr8_Btn.Visibility = Visibility.Visible;
                    this.Cntr9_Btn.Visibility = Visibility.Visible;
                    this.Cntr10_Btn.Visibility = Visibility.Visible;
                    this.Cntr11_Btn.Visibility = Visibility.Visible;
                    this.Cntr12_Btn.Visibility = Visibility.Visible;
                    this.Cntr13_Btn.Visibility = Visibility.Visible;
                    this.Cntr14_Btn.Visibility = Visibility.Visible;
                    this.Cntr15_Btn.Visibility = Visibility.Visible;
                    #endregion
                    break;
                case 0x11:
                    #region
                    this.Cntr0_Btn.Visibility = Visibility.Visible;
                    this.Cntr1_Btn.Visibility = Visibility.Visible;
                    this.Cntr2_Btn.Visibility = Visibility.Visible;
                    this.Cntr3_Btn.Visibility = Visibility.Visible;
                    this.Cntr4_Btn.Visibility = Visibility.Hidden;
                    this.Cntr7_Btn.Visibility = Visibility.Hidden;

                    this.Cntr8_Btn.Visibility = Visibility.Visible;
                    this.Cntr9_Btn.Visibility = Visibility.Visible;
                    this.Cntr10_Btn.Visibility = Visibility.Visible;
                    this.Cntr11_Btn.Visibility = Visibility.Visible;
                    this.Cntr12_Btn.Visibility = Visibility.Visible;
                    this.Cntr13_Btn.Visibility = Visibility.Visible;
                    this.Cntr14_Btn.Visibility = Visibility.Visible;
                    this.Cntr15_Btn.Visibility = Visibility.Visible;
                    #endregion
                    break;
                case 0xF8:
                    #region
                    this.Cntr0_Btn.Visibility = Visibility.Visible;
                    this.Cntr1_Btn.Visibility = Visibility.Visible;
                    this.Cntr2_Btn.Visibility = Visibility.Visible;
                    this.Cntr3_Btn.Visibility = Visibility.Visible;
                    this.Cntr4_Btn.Visibility = Visibility.Visible;
                    this.Cntr7_Btn.Visibility = Visibility.Visible;

                    this.Cntr8_Btn.Visibility = Visibility.Visible;
                    this.Cntr9_Btn.Visibility = Visibility.Visible;
                    this.Cntr10_Btn.Visibility = Visibility.Visible;
                    this.Cntr11_Btn.Visibility = Visibility.Visible;
                    this.Cntr12_Btn.Visibility = Visibility.Visible;
                    this.Cntr13_Btn.Visibility = Visibility.Visible;
                    this.Cntr14_Btn.Visibility = Visibility.Visible;
                    this.Cntr15_Btn.Visibility = Visibility.Hidden;
                    #endregion
                    break;
                case 0xEE:
                    #region
                    this.Cntr0_Btn.Visibility = Visibility.Visible;
                    this.Cntr1_Btn.Visibility = Visibility.Hidden;
                    this.Cntr2_Btn.Visibility = Visibility.Visible;
                    this.Cntr3_Btn.Visibility = Visibility.Visible;
                    this.Cntr4_Btn.Visibility = Visibility.Visible;
                    this.Cntr7_Btn.Visibility = Visibility.Hidden;

                    this.Cntr8_Btn.Visibility = Visibility.Visible;
                    this.Cntr9_Btn.Visibility = Visibility.Visible;
                    this.Cntr10_Btn.Visibility = Visibility.Hidden;
                    this.Cntr11_Btn.Visibility = Visibility.Hidden;
                    this.Cntr12_Btn.Visibility = Visibility.Hidden;
                    this.Cntr13_Btn.Visibility = Visibility.Hidden;
                    this.Cntr14_Btn.Visibility = Visibility.Hidden;
                    this.Cntr15_Btn.Visibility = Visibility.Hidden;
                    #endregion
                    break;
                case 0x25:
                    #region
                    this.Cntr0_Btn.Visibility = Visibility.Visible;
                    this.Cntr1_Btn.Visibility = Visibility.Hidden;
                    this.Cntr2_Btn.Visibility = Visibility.Visible;
                    this.Cntr3_Btn.Visibility = Visibility.Visible;
                    this.Cntr4_Btn.Visibility = Visibility.Hidden;
                    this.Cntr7_Btn.Visibility = Visibility.Hidden;

                    this.Cntr8_Btn.Visibility = Visibility.Hidden;
                    this.Cntr9_Btn.Visibility = Visibility.Hidden;
                    this.Cntr10_Btn.Visibility = Visibility.Hidden;
                    this.Cntr11_Btn.Visibility = Visibility.Hidden;
                    this.Cntr12_Btn.Visibility = Visibility.Hidden;
                    this.Cntr13_Btn.Visibility = Visibility.Hidden;
                    this.Cntr14_Btn.Visibility = Visibility.Hidden;
                    this.Cntr15_Btn.Visibility = Visibility.Hidden;
                    #endregion
                    break;
                case 0xD7:
                    #region
                    this.Cntr0_Btn.Visibility = Visibility.Visible;
                    this.Cntr1_Btn.Visibility = Visibility.Hidden;
                    this.Cntr2_Btn.Visibility = Visibility.Visible;
                    this.Cntr3_Btn.Visibility = Visibility.Visible;
                    this.Cntr4_Btn.Visibility = Visibility.Hidden;
                    this.Cntr7_Btn.Visibility = Visibility.Hidden;

                    this.Cntr8_Btn.Visibility = Visibility.Hidden;
                    this.Cntr9_Btn.Visibility = Visibility.Hidden;
                    this.Cntr10_Btn.Visibility = Visibility.Hidden;
                    this.Cntr11_Btn.Visibility = Visibility.Hidden;
                    this.Cntr12_Btn.Visibility = Visibility.Hidden;
                    this.Cntr13_Btn.Visibility = Visibility.Hidden;
                    this.Cntr14_Btn.Visibility = Visibility.Hidden;
                    this.Cntr15_Btn.Visibility = Visibility.Hidden;
                    #endregion
                    break;
                case 0x15:
                    #region
                    this.Cntr0_Btn.Visibility = Visibility.Visible;
                    this.Cntr1_Btn.Visibility = Visibility.Hidden;
                    this.Cntr2_Btn.Visibility = Visibility.Visible;
                    this.Cntr3_Btn.Visibility = Visibility.Visible;
                    this.Cntr4_Btn.Visibility = Visibility.Hidden;
                    this.Cntr7_Btn.Visibility = Visibility.Hidden;

                    this.Cntr8_Btn.Visibility = Visibility.Hidden;
                    this.Cntr9_Btn.Visibility = Visibility.Hidden;
                    this.Cntr10_Btn.Visibility = Visibility.Hidden;
                    this.Cntr11_Btn.Visibility = Visibility.Hidden;
                    this.Cntr12_Btn.Visibility = Visibility.Hidden;
                    this.Cntr13_Btn.Visibility = Visibility.Hidden;
                    this.Cntr14_Btn.Visibility = Visibility.Hidden;
                    this.Cntr15_Btn.Visibility = Visibility.Hidden;
                    #endregion
                    break;
                case 0xD6:
                    #region
                    this.Cntr0_Btn.Visibility = Visibility.Visible;
                    this.Cntr1_Btn.Visibility = Visibility.Hidden;
                    this.Cntr2_Btn.Visibility = Visibility.Visible;
                    this.Cntr3_Btn.Visibility = Visibility.Visible;
                    this.Cntr4_Btn.Visibility = Visibility.Hidden;
                    this.Cntr7_Btn.Visibility = Visibility.Hidden;

                    this.Cntr8_Btn.Visibility = Visibility.Visible;
                    this.Cntr9_Btn.Visibility = Visibility.Visible;
                    this.Cntr10_Btn.Visibility = Visibility.Visible;
                    this.Cntr11_Btn.Visibility = Visibility.Hidden;
                    this.Cntr12_Btn.Visibility = Visibility.Visible;
                    this.Cntr13_Btn.Visibility = Visibility.Visible;
                    this.Cntr14_Btn.Visibility = Visibility.Visible;
                    this.Cntr15_Btn.Visibility = Visibility.Hidden;
                    #endregion
                    break;
                case 0xD5:
                    #region
                    this.Cntr0_Btn.Visibility = Visibility.Visible;
                    this.Cntr1_Btn.Visibility = Visibility.Hidden;
                    this.Cntr2_Btn.Visibility = Visibility.Visible;
                    this.Cntr3_Btn.Visibility = Visibility.Visible;
                    this.Cntr4_Btn.Visibility = Visibility.Hidden;
                    this.Cntr7_Btn.Visibility = Visibility.Hidden;

                    this.Cntr8_Btn.Visibility = Visibility.Hidden;
                    this.Cntr9_Btn.Visibility = Visibility.Hidden;
                    this.Cntr10_Btn.Visibility = Visibility.Hidden;
                    this.Cntr11_Btn.Visibility = Visibility.Hidden;
                    this.Cntr12_Btn.Visibility = Visibility.Hidden;
                    this.Cntr13_Btn.Visibility = Visibility.Hidden;
                    this.Cntr14_Btn.Visibility = Visibility.Hidden;
                    this.Cntr15_Btn.Visibility = Visibility.Hidden;
                    #endregion
                    break;
                case 0xD1:
                    #region
                    this.Cntr0_Btn.Visibility = Visibility.Visible;
                    this.Cntr1_Btn.Visibility = Visibility.Hidden;
                    this.Cntr2_Btn.Visibility = Visibility.Visible;
                    this.Cntr3_Btn.Visibility = Visibility.Visible;
                    this.Cntr4_Btn.Visibility = Visibility.Hidden;
                    this.Cntr7_Btn.Visibility = Visibility.Hidden;

                    this.Cntr8_Btn.Visibility = Visibility.Hidden;
                    this.Cntr9_Btn.Visibility = Visibility.Hidden;
                    this.Cntr10_Btn.Visibility = Visibility.Hidden;
                    this.Cntr11_Btn.Visibility = Visibility.Hidden;
                    this.Cntr12_Btn.Visibility = Visibility.Hidden;
                    this.Cntr13_Btn.Visibility = Visibility.Hidden;
                    this.Cntr14_Btn.Visibility = Visibility.Hidden;
                    this.Cntr15_Btn.Visibility = Visibility.Hidden;
                    #endregion
                    break;
                case 0xD2:
                    #region
                    this.Cntr0_Btn.Visibility = Visibility.Visible;
                    this.Cntr1_Btn.Visibility = Visibility.Hidden;
                    this.Cntr2_Btn.Visibility = Visibility.Visible;
                    this.Cntr3_Btn.Visibility = Visibility.Visible;
                    this.Cntr4_Btn.Visibility = Visibility.Hidden;
                    this.Cntr7_Btn.Visibility = Visibility.Hidden;

                    this.Cntr8_Btn.Visibility = Visibility.Hidden;
                    this.Cntr9_Btn.Visibility = Visibility.Hidden;
                    this.Cntr10_Btn.Visibility = Visibility.Hidden;
                    this.Cntr11_Btn.Visibility = Visibility.Hidden;
                    this.Cntr12_Btn.Visibility = Visibility.Hidden;
                    this.Cntr13_Btn.Visibility = Visibility.Hidden;
                    this.Cntr14_Btn.Visibility = Visibility.Hidden;
                    this.Cntr15_Btn.Visibility = Visibility.Hidden;
                    #endregion
                    break;
                default:
                    #region
                    this.Cntr0_Btn.Visibility = Visibility.Visible;
                    this.Cntr1_Btn.Visibility = Visibility.Visible;
                    this.Cntr2_Btn.Visibility = Visibility.Visible;
                    this.Cntr3_Btn.Visibility = Visibility.Visible;
                    this.Cntr4_Btn.Visibility = Visibility.Visible;
                    this.Cntr7_Btn.Visibility = Visibility.Visible;

                    this.Cntr8_Btn.Visibility = Visibility.Visible;
                    this.Cntr9_Btn.Visibility = Visibility.Visible;
                    this.Cntr10_Btn.Visibility = Visibility.Visible;
                    this.Cntr11_Btn.Visibility = Visibility.Visible;
                    this.Cntr12_Btn.Visibility = Visibility.Visible;
                    this.Cntr13_Btn.Visibility = Visibility.Visible;
                    this.Cntr14_Btn.Visibility = Visibility.Visible;
                    this.Cntr15_Btn.Visibility = Visibility.Visible;
                    #endregion
                    break;
            }
        }

        private void treeviewListLeft_Scroll(object sender, System.Windows.Controls.Primitives.ScrollEventArgs e)
        {
            int u = 0;
        }
        bool mSuppressRequestBringIntoView;
        private void TreeViewItem_RequestBringIntoView(object sender, RequestBringIntoViewEventArgs e)
        {

            // Ignore re-entrant calls
            if (mSuppressRequestBringIntoView)
                return;

            // Cancel the current scroll attempt
            e.Handled = true;

            // Call BringIntoView using a rectangle that extends into "negative space" to the left of our
            // actual control. This allows the vertical scrolling behaviour to operate without adversely
            // affecting the current horizontal scroll position.
            mSuppressRequestBringIntoView = true;

            TreeViewItem tvi = sender as TreeViewItem;
            if (tvi != null)
            {
                Rect newTargetRect = new Rect(-1000, 0, tvi.ActualWidth + 1000, tvi.ActualHeight);
                tvi.BringIntoView(newTargetRect);
            }

            mSuppressRequestBringIntoView = false;
        }
        //Групповая рассылка команды сняь Тест
        private void SendTestReset4AllALSmodulesBtn_Click(object sender, RoutedEventArgs e)
        {
            this.AddNewMessage2Console(new MessageCS(DateTime.Now, "C", "Пользователь", "Групповая посылка команды №3: снять Тест"));
            byte alsn = 0;
            List<GKDevice> devList = new List<GKDevice>();
            if (this.ALSTabControl.SelectedIndex == 0)
            {
                alsn = 1;
                devList = this.ALS1LeftSideDevices;
            }
            else
            {
                alsn = 2;
                devList = this.ALS2LeftSideDevices;
            }
            List<byte> data = new List<byte>();
            data.Add(alsn);
            foreach (GKDevice d in devList)
            {
                //foreach (CommandBit cmd in d.Driver.AvailableCommandBits)
                {
                    if ((d.Driver.AvailableStateBits.Contains(GKStateBit.Test)) || d.Driver.AvailableStateClasses.Contains(XStateClass.Test))
                    {
                        data.Add((byte)d.IntAddress);
                        data.Add((byte)GKStateBit.Fire2);
                        data.Add((byte)d.Driver.DriverTypeNo);
                        this.AddNewMessage2Console(new MessageCS(DateTime.Now, "C", "АЛС" + alsn + "." + d.IntAddress + "." + d.Driver.ShortName, "Групповая команда <(3) Снять тест>"));
                        if (d.Devices != null)
                        {
                            if (d.Devices.Count != 0)
                            {
                                foreach (GKDevice sd in d.Devices)
                                {
                                    //foreach (CommandBit cmd1 in sd.Driver.AvailableCommandBits)
                                    {
                                        if ((sd.Driver.AvailableStateBits.Contains(GKStateBit.Test)) || sd.Driver.AvailableStateClasses.Contains(XStateClass.Test))
                                        {
                                            data.Add((byte)sd.IntAddress);
                                            data.Add((byte)GKStateBit.Fire2);
                                            data.Add((byte)sd.Driver.DriverTypeNo);
                                            this.AddNewMessage2Console(new MessageCS(DateTime.Now, "C", "АЛС" + alsn + "." + sd.IntAddress + "." + sd.Driver.ShortName, "Групповая команда <(3) Снять тест>"));
                                        }
                                    }
                                }
                            }
                        }
                    }
                }
            }
            SendManager.WriteDeviceGroupCommand(data);

            return;
            #region ALS1
            foreach (GKDevice d in this.ALS1LeftSideDevices)
            {
                if (d.TestState)
                {
                    SendManager.WriteDeviceCommand(4, d, 1, GKStateBit.Fire2);
                    if (d.Devices != null)
                    {
                        if (d.Devices.Count != 0)
                        {
                            foreach (GKDevice sd in d.Devices)
                            {
                                SendManager.WriteDeviceCommand(4, sd, 1, GKStateBit.Fire2);
                            }
                        }
                    }
                }
            }
            #endregion
            #region ALS2
            foreach (GKDevice d in this.ALS2LeftSideDevices)
            {
                if (d.TestState)
                {
                    SendManager.WriteDeviceCommand(4, d, 2, GKStateBit.Fire2);
                    if (d.Devices != null)
                    {
                        if (d.Devices.Count != 0)
                        {
                            foreach (GKDevice sd in d.Devices)
                            {
                                SendManager.WriteDeviceCommand(4, sd, 2, GKStateBit.Fire2);
                            }
                        }
                    }
                }
            }
            #endregion
        }
        private void Send10Cmd4All_Click(object sender, RoutedEventArgs e)
        {
            this.AddNewMessage2Console(new MessageCS(DateTime.Now, "C", "Пользователь", "Групповая посылка команды №10: Включить немедленно"));
            byte alsn = 0;
            List<GKDevice> devList = new List<GKDevice>();
            if (this.ALSTabControl.SelectedIndex == 0)
            {
                alsn = 1;
                devList = this.ALS1LeftSideDevices;
            }
            else
            {
                alsn = 2;
                devList = this.ALS2LeftSideDevices;
            }
            List<byte> data = new List<byte>();
            data.Add(alsn);
            foreach (GKDevice d in devList)
            {
                //foreach (CommandBit cmd in d.Driver.AvailableCommandBits)
                {
                    if ((d.Driver.AvailableStateBits.Contains(GKStateBit.TurningOff)) || d.Driver.AvailableStateClasses.Contains(XStateClass.TurningOff))
                    {
                        data.Add((byte)d.IntAddress);
                        data.Add((byte)GKStateBit.TurningOff);
                        data.Add((byte)d.Driver.DriverTypeNo);
                        this.AddNewMessage2Console(new MessageCS(DateTime.Now, "C", "АЛС" + alsn + "." + d.IntAddress + "." + d.Driver.ShortName, "Групповая команда <(10) Вкл.немедленно (авт.вкл.)>"));
                        if (d.Devices != null)
                        {
                            if (d.Devices.Count != 0)
                            {
                                foreach (GKDevice sd in d.Devices)
                                {
                                    //foreach (CommandBit cmd1 in sd.Driver.AvailableCommandBits)
                                    {
                                        if ((sd.Driver.AvailableStateBits.Contains(GKStateBit.TurningOff)) || sd.Driver.AvailableStateClasses.Contains(XStateClass.TurningOff))
                                        {
                                            data.Add((byte)sd.IntAddress);
                                            data.Add((byte)GKStateBit.TurningOff);
                                            data.Add((byte)sd.Driver.DriverTypeNo);
                                            this.AddNewMessage2Console(new MessageCS(DateTime.Now, "C", "АЛС" + alsn + "." + sd.IntAddress + "." + sd.Driver.ShortName, "Групповая команда <(10) Вкл.немедленно (авт.вкл.)>"));
                                        }
                                    }
                                }
                            }
                        }
                    }
                }
            }
            SendManager.WriteDeviceGroupCommand(data);            
            return;
            #region ALS1
            foreach (GKDevice d in this.ALS1LeftSideDevices)
            {
                if (d.CMDControls.ContainsKey(10))
                {
                    SendManager.WriteDeviceCommand(4, d, 1, GKStateBit.TurningOff);
                    if (d.Devices != null)
                    {
                        if (d.Devices.Count != 0)
                        {
                            foreach (GKDevice sd in d.Devices)
                            {
                                SendManager.WriteDeviceCommand(4, sd, 1, GKStateBit.TurningOff);
                            }
                        }
                    }
                }
            }
            #endregion
            #region ALS2
            foreach (GKDevice d in this.ALS2LeftSideDevices)
            {
                if (d.CMDControls.ContainsKey(10))
                {
                    SendManager.WriteDeviceCommand(4, d, 2, GKStateBit.TurningOff);
                    if (d.Devices != null)
                    {
                        if (d.Devices.Count != 0)
                        {
                            foreach (GKDevice sd in d.Devices)
                            {
                                SendManager.WriteDeviceCommand(4, sd, 2, GKStateBit.TurningOff);
                            }
                        }
                    }
                }
            }
            #endregion
        }

        private void Send11Cmd4All_Click(object sender, RoutedEventArgs e)
        {
            this.AddNewMessage2Console(new MessageCS(DateTime.Now, "C", "Пользователь", "Групповая посылка команды №11: Выключить немедленно"));
            byte alsn = 0;
            List<GKDevice> devList = new List<GKDevice>();
            if (this.ALSTabControl.SelectedIndex == 0)
            {
                alsn = 1;
                devList = this.ALS1LeftSideDevices;
            }
            else
            {
                alsn = 2;
                devList = this.ALS2LeftSideDevices;
            }
            List<byte> data = new List<byte>();
            data.Add(alsn);
            foreach (GKDevice d in devList)
            {
                //foreach (CommandBit cmd in d.Driver.AvailableCommandBits)
                {
                    if ((d.Driver.AvailableStateBits.Contains(GKStateBit.TurningOn)) || d.Driver.AvailableStateClasses.Contains(XStateClass.TurningOn))
                    {
                        data.Add((byte)d.IntAddress);
                        data.Add((byte)GKStateBit.TurningOn);
                        data.Add((byte)d.Driver.DriverTypeNo);
                        this.AddNewMessage2Console(new MessageCS(DateTime.Now, "C", "АЛС" + alsn + "." + d.IntAddress + "." + d.Driver.ShortName, "Групповая команда <(11) Выкл.немедленно (авт.вкл.)>"));
                        if (d.Devices != null)
                        {
                            if (d.Devices.Count != 0)
                            {
                                foreach (GKDevice sd in d.Devices)
                                {
                                    //foreach (CommandBit cmd1 in sd.Driver.AvailableCommandBits)
                                    {
                                        if ((sd.Driver.AvailableStateBits.Contains(GKStateBit.TurningOn)) || sd.Driver.AvailableStateClasses.Contains(XStateClass.TurningOn))
                                        {
                                            data.Add((byte)sd.IntAddress);
                                            data.Add((byte)GKStateBit.TurningOn);
                                            data.Add((byte)sd.Driver.DriverTypeNo);
                                            this.AddNewMessage2Console(new MessageCS(DateTime.Now, "C", "АЛС" + alsn + "." + sd.IntAddress + "." + sd.Driver.ShortName, "Групповая команда <(11) Выкл.немедленно (авт.вкл.)>"));
                                        }
                                    }
                                }
                            }
                        }
                    }
                }
            }
            SendManager.WriteDeviceGroupCommand(data);
            
            return;
            #region ALS1
            foreach (GKDevice d in this.ALS1LeftSideDevices)
            {
                if (d.CMDControls.ContainsKey(11))
                {
                    SendManager.WriteDeviceCommand(4, d, 1, GKStateBit.TurnOn_InAutomatic);
                    if (d.Devices != null)
                    {
                        if (d.Devices.Count != 0)
                        {
                            foreach (GKDevice sd in d.Devices)
                            {
                                SendManager.WriteDeviceCommand(4, sd, 1, GKStateBit.TurnOn_InAutomatic);
                            }
                        }
                    }
                }
            }
            #endregion
            #region ALS2
            foreach (GKDevice d in this.ALS2LeftSideDevices)
            {
                if (d.CMDControls.ContainsKey(11))
                {
                    SendManager.WriteDeviceCommand(4, d, 2, GKStateBit.TurnOn_InAutomatic);
                    if (d.Devices != null)
                    {
                        if (d.Devices.Count != 0)
                        {
                            foreach (GKDevice sd in d.Devices)
                            {
                                SendManager.WriteDeviceCommand(4, sd, 2, GKStateBit.TurnOn_InAutomatic);
                            }
                        }
                    }
                }
            }
            #endregion
        }
        //Сканирование всех линий АЛС
        private void ScanAllALSlines_Click(object sender, RoutedEventArgs e)
        {
            var result = SendManager.Ping(4);
            if (result.HasError)
            {
                MessageBox.Show("Не удается подключиться к устройству с адресом " + this.IPaddressTxtBox.Text, "ОШИБКА ПОДКЛЮЧЕНИЯ!");
                return;
            }
            List<GKDevice> devs = new List<GKDevice>();
            try
            {
                //Отправляем Команду №0 для инициализации сканирования всех устройств на всех линиях
                SendManager.ScanDevices(0, devs);
                //Ждем 5 секунд перед запросом пакета данных по результатам сканирования линий
                Thread.Sleep(5000);

                try
                {
                    this.ALS1LeftSideDevices = new List<GKDevice>();
                    SendManager.ScanDevices(1, this.ALS1LeftSideDevices);
                    this.ALS1treeviewListLeft.Items.Clear();
                    foreach (GKDevice d in this.ALS1LeftSideDevices)
                    {
                        this.ALS1treeviewListLeft.Items.Add(d.GetTree());
                    }
                    List<GKDevice> als2 = new List<GKDevice>();
                    SendManager.ScanDevices(3, als2);
                    foreach (GKDevice d in als2)
                    {
                        this.ALS2treeviewListLeft.Items.Add(d.GetTree());
                    }
                    this.ALS2LeftSideDevices.AddRange(als2);
                }
                catch (Exception ex) { MessageBox.Show(ex.ToString(), "ОШИБКА!"); }
                try
                {
                    this.ALS1RightSideDevices = new List<GKDevice>();
                    SendManager.ScanDevices(2, this.ALS1RightSideDevices);
                    this.ALS1treeviewListRight.Items.Clear();
                    foreach (GKDevice d in this.ALS1RightSideDevices)
                    {
                        this.ALS1treeviewListRight.Items.Add(d.GetTree());
                    }
                    List<GKDevice> als2 = new List<GKDevice>();
                    SendManager.ScanDevices(4, als2);
                    foreach (GKDevice d in als2)
                    {
                        this.ALS2treeviewListRight.Items.Add(d.GetTree());
                    }
                    this.ALS2RightSideDevices.AddRange(als2);
                }
                catch (Exception ex) { MessageBox.Show(ex.ToString(), "ОШИБКА!"); }

                this.CompareDevicesLists();
            }
            catch (Exception ex) { MessageBox.Show(ex.ToString(), "ОШИБКА!"); }
        }

        /// <summary>
        /// Запись Ethernet-параметров в МОПС
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void SaveModbusTCPParams_Click(object sender, RoutedEventArgs e)
        {
            byte alsn = 0;
            GKDriver kauDRV = DriversRepository1.Drivers.FirstOrDefault(x => x.DriverTypeNo == 0x102);
            if (this.IPaddr1.Text == this.IPaddr2.Text)
            {
                MessageBox.Show("Введены одинаковые IP-адреса!", "ОШИБКА!");
                return;
            }
            kauDRV.IPaddress1 = this.IPaddr1.Text;
            kauDRV.Mask1 = this.PortMask1.Text;
            kauDRV.IPaddress2 = this.IPaddr2.Text;
            kauDRV.Mask2 = this.PortMask2.Text;
            kauDRV.Mac1 = this.MacAddress1Txt.Text;
            kauDRV.Mac2 = this.MacAddress2Txt.Text;
            try
            {
                kauDRV.KAUlogicalNum = byte.Parse(this.KAUaddress.Text);
                if (this.ModBusTCP1Flag.IsChecked == true)
                    kauDRV.MDBethernetPortNum = 1;
                else
                    kauDRV.MDBethernetPortNum = 2;
                var kau = new GKDevice();
                kau.Driver = kauDRV;
                kau.DriverUID = kauDRV.UID;
                kau.IntAddress = 0; //TCP parameters

                if (SendManager.WriteDeviceParameters(4, kau, alsn, 1, 1))
                {
                    SendManager.FinalizeWritingDeviceParameters(4, alsn);
                    MessageBox.Show("Успешно завершено.","Запись параметров в МОПС");
                }
                else
                {
                    this.AddNewMessage2Console(new MessageCS(DateTime.Now, "W", "Система", "Не удалось выполнить отправку данных! Неверный формат параметров (IP,маски или MAC)."));
                    MessageBox.Show("Не удалось выполнить отправку данных! Неверный формат параметров (IP,маски или MAC).","ОШИБКА!");
                }
            }
            catch { MessageBox.Show("Неверный адрес контроллера МОПС!", "ОШИБКА!"); }
        }
        //Передать в МОПС параметры настройки для модбасRTU
        private void SaveModbusRTUParams_Click(object sender, RoutedEventArgs e)
        {
            byte alsn = 0;
            GKDriver kauDRV = DriversRepository1.Drivers.FirstOrDefault(x => x.DriverTypeNo == 0x102);
            kauDRV.RSspeed = uint.Parse(((ComboBoxItem)this.BaudRateCombo.SelectedItem).Tag.ToString());
            kauDRV.RSparity = ushort.Parse(((ComboBoxItem)this.ParityCombo.SelectedItem).Tag.ToString());
            kauDRV.RSdatabit = ushort.Parse(((ComboBoxItem)this.DataBitsCombo.SelectedItem).Tag.ToString());
            kauDRV.RSstopbit = ushort.Parse(((ComboBoxItem)this.StopBitCombo.SelectedItem).Tag.ToString());
            kauDRV.RSReadtimeout = ushort.Parse(this.RSReadTimeoutTxt.Text);
            kauDRV.RSWritetimeout = ushort.Parse(this.RSWriteTimeoutTxt.Text);
            try
            {
                kauDRV.KAUlogicalNum = byte.Parse(this.KAUaddress.Text);
                var kau = new GKDevice();
                kau.Driver = kauDRV;
                kau.DriverUID = kauDRV.UID;
                kau.IntAddress = 1; //RTU parameters

                SendManager.WriteDeviceRTUParameters(4, kau, alsn);
                SendManager.FinalizeWritingDeviceParameters(4, alsn);
                MessageBox.Show("Успешно завершено.", "Запись параметров в МОПС");
            }
            catch
            {
                this.AddNewMessage2Console(new MessageCS(DateTime.Now, "W", "Система", "Неверный адрес контроллера МОПС!"));
                MessageBox.Show("Неверный адрес контроллера МОПС!", "ОШИБКА!");
            }
        }
        //Передача прошивки в МОПС
        private void MKROMsendBtn_Click(object sender, RoutedEventArgs e)
        {
            string ROMfile = "";
            Thread.CurrentThread.CurrentCulture = System.Globalization.CultureInfo.InvariantCulture;
            OpenFileDialog openFileDialog1 = new OpenFileDialog();
            openFileDialog1.InitialDirectory = Environment.CurrentDirectory;
            openFileDialog1.Filter = "Файлы прошивок|*.hex";
            try
            {
                if (openFileDialog1.ShowDialog() == true)
                {
                    ROMfile = openFileDialog1.FileName;
                    if ((ROMfile == "") || (ROMfile == "*.hex"))
                        return;
                }
                else
                    return;
                StreamReader hexFile = new StreamReader(ROMfile);
                string hexROMstr = hexFile.ReadToEnd();
                byte[] hexROM = Encoding.ASCII.GetBytes(hexROMstr);

                int packSize = 8000;
                int packNum = (int)(hexROM.Length / packSize);
                GKDriver kauDRV = DriversRepository1.Drivers.FirstOrDefault(x => x.DriverTypeNo == 0x102);
                kauDRV.KAUlogicalNum = byte.Parse(this.KAUaddress.Text);
                var kau = new GKDevice();
                kau.Driver = kauDRV;
                kau.DriverUID = kauDRV.UID;
                this.ROMsendingStarted = true;
                for (int i = 0; i < packNum; i++)
                {
                    byte[] packROM = new byte[packSize];
                    for (int c = 0; c < packSize; c++)
                    {
                        packROM[c] = hexROM[(i * packSize) + c];
                    }
                    SendManager.WriteROMtoDevice(4, kau, packROM, (byte)i, (byte)(packNum + 1));
                }
                byte[] packROMlast = new byte[hexROM.Length - (packSize * packNum)];
                for (int c = 0; c < (hexROM.Length - (packSize * packNum)); c++)
                {
                    packROMlast[c] = hexROM[(packSize * packNum) + c];
                }
                SendManager.WriteROMtoDevice(4, kau, packROMlast, (byte)packNum, (byte)(packNum + 1));
                //MessageBox.Show("Успешно завершено.", "Запись прошивки МК");
            }
            catch (Exception ex)
            { MessageBox.Show(ex.ToString(), "ОШИБКА ОТКРЫТИЯ ФАЙЛА ПРОЕКТА!"); }

        }


        //Подключение к МОПС
        private void ConnectKAUBtn_Click(object sender, RoutedEventArgs e)
        {
            if (this.KAUConnectionCkeckFlag)
            {
                this.KAUConnectionCkeckFlag = false;
                this.ScanAllLines.Visibility = Visibility.Visible;
            }
            else
            {
                this.ScanAllLines.Visibility = Visibility.Collapsed;
                this.KAUConnectionCkeckFlag = true;
                this.KAUinfoLabel.Content = SendManager.GetDeviceInfo(4);
                ThreadPool.QueueUserWorkItem(new WaitCallback(this.BackgroundKAUConnectChecker), null);
            }
        }
        //Получение настроек МОПС
        private void GetKAUSettingsBtn_Click(object sender, RoutedEventArgs e)
        {
            //List<GKDevice> devices = new List<GKDevice>();
            //SendManager.GetDevices(0x87, devices);
            //return;

            KAUConfig cfg = new KAUConfig();
            if (SendManager.RequestSettingsFromKAU(4, out cfg))
            {
                //MessageBox.Show("Ответ: " + SendManager.Res0x22);
                switch (cfg.BaudRate)
                {
                    case 110: this.BaudRateCombo.SelectedIndex = 0; break;
                    case 150: this.BaudRateCombo.SelectedIndex = 1; break;
                    case 300: this.BaudRateCombo.SelectedIndex = 2; break;
                    case 600: this.BaudRateCombo.SelectedIndex = 3; break;
                    case 1200: this.BaudRateCombo.SelectedIndex = 4; break;
                    case 2400: this.BaudRateCombo.SelectedIndex = 5; break;
                    case 4800: this.BaudRateCombo.SelectedIndex = 6; break;
                    case 9600: this.BaudRateCombo.SelectedIndex = 7; break;
                    case 19200: this.BaudRateCombo.SelectedIndex = 8; break;
                    case 38400: this.BaudRateCombo.SelectedIndex = 9; break;
                    case 57600: this.BaudRateCombo.SelectedIndex = 10; break;
                    case 115200: this.BaudRateCombo.SelectedIndex = 11; break;
                    case 144000: this.BaudRateCombo.SelectedIndex = 12; break;
                    case 192000: this.BaudRateCombo.SelectedIndex = 13; break;
                    case 288000: this.BaudRateCombo.SelectedIndex = 14; break;
                    case 576000: this.BaudRateCombo.SelectedIndex = 15; break;
                }
                switch (cfg.Parity)
                {
                    case 1: this.ParityCombo.SelectedIndex = 0; break;
                    case 2: this.ParityCombo.SelectedIndex = 1; break;
                    case 3: this.ParityCombo.SelectedIndex = 2; break;
                    case 4: this.ParityCombo.SelectedIndex = 3; break;
                    case 5: this.ParityCombo.SelectedIndex = 4; break;
                }
                switch (cfg.Data)
                {
                    case 7: this.DataBitsCombo.SelectedIndex = 0; break;
                    case 8: this.DataBitsCombo.SelectedIndex = 1; break;
                }
                switch (cfg.Stop)
                {
                    case 0: this.StopBitCombo.SelectedIndex = 0; break;
                    case 1: this.StopBitCombo.SelectedIndex = 1; break;
                    case 15: this.StopBitCombo.SelectedIndex = 2; break;
                    case 2: this.StopBitCombo.SelectedIndex = 3; break;
                }
                this.RSReadTimeoutTxt.Text = cfg.Timeout.ToString();
                this.KAUaddress.Text = cfg.Address.ToString();
                this.IPaddr1.Text = cfg.IP1;
                this.IPaddr2.Text = cfg.IP2;
                this.PortMask1.Text = cfg.Mask1;
                this.PortMask2.Text = cfg.Mask2;
                this.MacAddress1Txt.Text = cfg.MAC1;
                this.MacAddress2Txt.Text = cfg.MAC2;
                if (cfg.Protocol1 == "modbus")
                {
                    this.ModBusTCP1Flag.IsChecked = true;
                }
                if (cfg.Protocol2 == "modbus")
                {
                    this.ModBusTCP2Flag.IsChecked = true;
                }
                this.ScanAllLines_Click(null, null);
                MessageBox.Show("Процедура подчитывания параметров МОПС успешно завершена.", "Считывание параметров МОПС");
            }
            else
                MessageBox.Show("Ошибка выполнения команды чтения конфигурации МОПС (0х22)", "ОШИБКА!");
            #region Считываем конфигурацию АУ

            #endregion
        }
        //Подключение ModBus-клиента к МОПС
        private void MDBconnetcBtn_Click(object sender, RoutedEventArgs e)
        {
            if (this.AddrNumTxtBox.Text == "0")
            {
                MessageBox.Show("Необходимо задать количество регистров для считывания!","ВНИМАНИЕ!");
                return;
            }
            if (this.MDBtcpRadioBtn.IsChecked == true)
            {
                if (mdbTCPSrv.WorkerFlag)
                {
                    mdbTCPSrv.WorkerFlag = false;
                    mdbTCPSrv.PropertyChanged -= MdbSrv_PropertyChanged;
                    Thread.Sleep(1000);
                    this.MDBconnetcBtn.Foreground = Brushes.White;
                }
                else
                {
                    if (this.MDBfunctionCmbBox.SelectedIndex == -1)
                        this.MDBfunctionCmbBox.SelectedIndex = 0;
                    this.MDBconnetcBtn.Foreground = Brushes.Yellow;
                    mdbTCPSrv.Address = 1;
                    mdbTCPSrv.IPaddress = this.MDBIPaddressTxtBox.Text;
                    mdbTCPSrv.StartAddress = ushort.Parse(this.AddrFromTxtBox.Text);
                    mdbTCPSrv.NumberOfRegisters = ushort.Parse(this.AddrNumTxtBox.Text);
                    switch (this.ExchngSpeedCmbBox.SelectedIndex)
                    {
                        case 0: mdbTCPSrv.ThreadSleepValue = 1000; break; //1 per sec
                        case 1: mdbTCPSrv.ThreadSleepValue = 500; break; //2 per sec
                        case 2: mdbTCPSrv.ThreadSleepValue = 200; break; //5 per sec
                        case 3: mdbTCPSrv.ThreadSleepValue = 100; break; //10 per sec
                    }

                    mdbTCPSrv.Values = new System.Collections.ObjectModel.ObservableCollection<MDBValue>();
                    for (int i = 0; i < mdbTCPSrv.NumberOfRegisters; i++)
                    {
                        mdbTCPSrv.Values.Add(new MDBValue() { Id = i, Address = (ushort)(mdbTCPSrv.StartAddress + i), Value = 0 });
                    }
                    this.RegistersList.ItemsSource = mdbTCPSrv.Values;

                    mdbTCPSrv.FunctionID = this.MDBfunctionCmbBox.SelectedIndex;
                    mdbTCPSrv.WorkerFlag = true;
                    this.ErrorTxtBox.Text = "";
                    mdbTCPSrv.ErrorString = "";
                    mdbTCPSrv.PropertyChanged += MdbSrv_PropertyChanged;
                    mdbTCPSrv.ServiceReg0.PropertyChanged += ServiceReg0_PropertyChanged;
                    mdbTCPSrv.ServiceReg1.PropertyChanged += ServiceReg1_PropertyChanged;
                    ThreadPool.QueueUserWorkItem(new WaitCallback(mdbTCPSrv.TCPworker), null);
                }
            }
            if (this.MDBrtuRadioBtn.IsChecked == true)
            {
                if (mdbRTUSrv.WorkerFlag)
                {
                    mdbRTUSrv.WorkerFlag = false;
                    mdbRTUSrv.PropertyChanged -= MdbRTUSrv_PropertyChanged;
                    Thread.Sleep(1000);
                    this.MDBconnetcBtn.Foreground = Brushes.White;
                }
                else
                {
                    if (this.SerialPortNameCmbBox.SelectedIndex==-1)
                    {
                        MessageBox.Show("Не выбран СОМ-порт!", "ОШИБКА!");
                        return;
                    }
                    if (this.MDBfunctionCmbBox.SelectedIndex == -1)
                        this.MDBfunctionCmbBox.SelectedIndex = 0;
                    this.MDBconnetcBtn.Foreground = Brushes.Yellow;
                    mdbRTUSrv.Address = 1;
                    mdbRTUSrv.COMport = new SerialPort(this.SerialPortNameCmbBox.SelectedItem.ToString(), int.Parse(((ComboBoxItem)this.RSBaudRateCombo.SelectedItem).Tag.ToString()));

                    mdbRTUSrv.StartAddress = ushort.Parse(this.AddrFromTxtBox.Text);
                    mdbRTUSrv.NumberOfRegisters = ushort.Parse(this.AddrNumTxtBox.Text);
                    switch (this.ExchngSpeedCmbBox.SelectedIndex)
                    {
                        case 0: mdbRTUSrv.ThreadSleepValue = 1000; break; //1 per sec
                        case 1: mdbRTUSrv.ThreadSleepValue = 500; break; //2 per sec
                        case 2: mdbRTUSrv.ThreadSleepValue = 200; break; //5 per sec
                        case 3: mdbRTUSrv.ThreadSleepValue = 100; break; //10 per sec
                    }

                    mdbRTUSrv.Values = new System.Collections.ObjectModel.ObservableCollection<MDBValue>();
                    for (int i = 0; i < mdbRTUSrv.NumberOfRegisters; i++)
                    {
                        mdbRTUSrv.Values.Add(new MDBValue() { Id = i, Address = (ushort)(mdbRTUSrv.StartAddress + i), Value = 0 });
                    }
                    this.RegistersList.ItemsSource = mdbRTUSrv.Values;

                    mdbRTUSrv.FunctionID = this.MDBfunctionCmbBox.SelectedIndex;
                    mdbRTUSrv.WorkerFlag = true;
                    this.ErrorTxtBox.Text = "";
                    mdbRTUSrv.ErrorString = "";
                    mdbRTUSrv.PropertyChanged += MdbRTUSrv_PropertyChanged; ;
                    mdbRTUSrv.ServiceReg0.PropertyChanged += ServiceReg0_PropertyChanged1; ;
                    mdbRTUSrv.ServiceReg1.PropertyChanged += ServiceReg1_PropertyChanged1; ;
                    ThreadPool.QueueUserWorkItem(new WaitCallback(mdbRTUSrv.RTUworker), null);
                }
            }
        }

        private void ServiceReg1_PropertyChanged1(object sender, System.ComponentModel.PropertyChangedEventArgs e)
        {
            try
            {
                //Регистр№1 (Системная неисправность)
                if ((mdbRTUSrv.ServiceReg1.PrevValue & 0x1) == 0x0)
                {
                    if ((mdbRTUSrv.ServiceReg1.Value & 0x1) == 0x1)//бит 0 - осн. выше норм
                    {
                        this.AddNewMessage2Console(new MessageCS(DateTime.Now, "W", "Система", "Основное напряжение питания МОПС - выше нормы!"));
                    }
                }
                else
                {
                    if ((mdbRTUSrv.ServiceReg1.Value & 0x1) == 0x0)//бит 0 - осн. выше норм
                    {
                        this.AddNewMessage2Console(new MessageCS(DateTime.Now, "M", "Система", "Основное напряжение питания МОПС - в норме."));
                    }
                }
                if ((mdbRTUSrv.ServiceReg1.PrevValue & 0x2) == 0x0)
                {
                    if ((mdbRTUSrv.ServiceReg1.Value & 0x2) == 0x2)//бит 1 - осн.ниже норм
                    {
                        this.AddNewMessage2Console(new MessageCS(DateTime.Now, "W", "Система", "Основное напряжение питания МОПС - ниже нормы!"));
                    }
                }
                else
                {
                    if ((mdbRTUSrv.ServiceReg1.Value & 0x2) == 0x0)//бит 1 - осн.ниже норм
                    {
                        this.AddNewMessage2Console(new MessageCS(DateTime.Now, "M", "Система", "Основное напряжение питания МОПС - в норме."));
                    }
                }
                if ((mdbRTUSrv.ServiceReg1.PrevValue & 0x4) == 0x0)
                {
                    if ((mdbRTUSrv.ServiceReg1.Value & 0x4) == 0x4)//бит 2 - рез.выше норм
                    {
                        this.AddNewMessage2Console(new MessageCS(DateTime.Now, "W", "Система", "Резервное напряжение питания МОПС - выше нормы!"));
                    }
                }
                else
                {
                    if ((mdbRTUSrv.ServiceReg1.Value & 0x4) == 0x0)//бит 2 - рез.выше норм
                    {
                        this.AddNewMessage2Console(new MessageCS(DateTime.Now, "M", "Система", "Резервное напряжение питания МОПС - в норме."));
                    }
                }
                if ((mdbRTUSrv.ServiceReg1.PrevValue & 0x8) == 0x0)
                {
                    if ((mdbRTUSrv.ServiceReg1.Value & 0x8) == 0x8)//бит 3 - рез.ниже норм	
                    {
                        this.AddNewMessage2Console(new MessageCS(DateTime.Now, "W", "Система", "Резервное напряжение питания МОПС - ниже нормы!"));
                    }
                }
                else
                {
                    if ((mdbRTUSrv.ServiceReg1.Value & 0x8) == 0x0)//бит 3 - рез.ниже норм	
                    {
                        this.AddNewMessage2Console(new MessageCS(DateTime.Now, "M", "Система", "Резервное напряжение питания МОПС - в норме."));
                    }
                }
            }
            catch { }
            mdbRTUSrv.ServiceReg1.PrevValue = mdbRTUSrv.ServiceReg1.Value;
            /*bool_vals[4] = (short_v & 0x10) == 0x10;
            bool_vals[5] = (short_v & 0x20) == 0x20;
            bool_vals[6] = (short_v & 0x40) == 0x40;
            bool_vals[7] = (short_v & 0x80) == 0x80;
            bool_vals[8] = (short_v & 0x100) == 0x100;
            bool_vals[9] = (short_v & 0x200) == 0x200;
            bool_vals[10] = (short_v & 0x400) == 0x400;
            bool_vals[11] = (short_v & 0x800) == 0x800;
            bool_vals[12] = (short_v & 0x1000) == 0x1000;
            bool_vals[13] = (short_v & 0x2000) == 0x2000;
            bool_vals[14] = (short_v & 0x4000) == 0x4000;
            bool_vals[15] = (short_v & 0x8000) == 0x8000;*/
        }
        private void ServiceReg0_PropertyChanged1(object sender, System.ComponentModel.PropertyChangedEventArgs e)
        {
            try
            {
                //Регистр№0 (Состояние АЛС)
                if ((mdbRTUSrv.ServiceReg0.PrevValue & 0x1) == 0x0)
                {
                    if ((mdbRTUSrv.ServiceReg0.Value & 0x1) == 0x1)//бит 0 - разрыв АЛС 1
                    {
                        this.AddNewMessage2ConsoleALS1(new MessageCS(DateTime.Now, "A", "АЛС1", "Разрыв в АЛС1!"), Brushes.Red, Brushes.Red);
                    }
                }
                else
                {
                    if ((mdbRTUSrv.ServiceReg0.Value & 0x1) == 0x0)//бит 0 - разрыв АЛС 1
                    {
                        this.AddNewMessage2ConsoleALS1(new MessageCS(DateTime.Now, "M", "АЛС1", "Разрыв в АЛС1 устранен."), Brushes.WhiteSmoke, Brushes.WhiteSmoke);
                    }
                }
                if ((mdbRTUSrv.ServiceReg0.PrevValue & 0x2) == 0x0)
                {
                    if ((mdbRTUSrv.ServiceReg0.Value & 0x2) == 0x2)//бит 1 - КЗ гл.АСЛ 1
                    {
                        this.AddNewMessage2ConsoleALS1(new MessageCS(DateTime.Now, "A", "АЛС1", "КЗ в основной АЛС1!"), Brushes.Red, null);
                    }
                }
                else
                {
                    if ((mdbRTUSrv.ServiceReg0.Value & 0x2) == 0x0)//бит 1 - КЗ гл.АСЛ 1
                    {
                        this.AddNewMessage2ConsoleALS1(new MessageCS(DateTime.Now, "M", "АЛС1", "КЗ в основной АЛС1 устранено."), Brushes.WhiteSmoke, null);
                    }
                }
                if ((mdbRTUSrv.ServiceReg0.PrevValue & 0x4) == 0x0)
                {
                    if ((mdbRTUSrv.ServiceReg0.Value & 0x4) == 0x4)//бит 2 - КЗ рез.АСЛ 1
                    {
                        this.AddNewMessage2ConsoleALS1(new MessageCS(DateTime.Now, "A", "АЛС1", "КЗ в резервной АЛС1!"), null, Brushes.Red);
                    }
                }
                else
                {
                    if ((mdbRTUSrv.ServiceReg0.Value & 0x4) == 0x0)//бит 2 - КЗ рез.АСЛ 1
                    {
                        this.AddNewMessage2ConsoleALS1(new MessageCS(DateTime.Now, "M", "АЛС1", "КЗ в резервной АЛС1 устранено."), null, Brushes.WhiteSmoke);
                    }
                }
                if ((mdbRTUSrv.ServiceReg0.PrevValue & 0x100) == 0x0)
                {
                    if ((mdbRTUSrv.ServiceReg0.Value & 0x100) == 0x100)//бит 8 - разрыв АЛС 2
                    {
                        this.AddNewMessage2ConsoleALS2(new MessageCS(DateTime.Now, "A", "АЛС2", "Разрыв в АЛС2!"), Brushes.Red, Brushes.Red);
                    }
                }
                else
                {
                    if ((mdbRTUSrv.ServiceReg0.Value & 0x100) == 0x0)//бит 8 - разрыв АЛС 2
                    {
                        this.AddNewMessage2ConsoleALS2(new MessageCS(DateTime.Now, "M", "АЛС2", "Разрыв в АЛС2 устранен."), Brushes.WhiteSmoke, Brushes.WhiteSmoke);
                    }
                }
                if ((mdbRTUSrv.ServiceReg0.PrevValue & 0x200) == 0x0)
                {
                    if ((mdbRTUSrv.ServiceReg0.Value & 0x200) == 0x200)//бит 9 - КЗ гл.АСЛ 2
                    {
                        this.AddNewMessage2ConsoleALS2(new MessageCS(DateTime.Now, "A", "АЛС2", "КЗ в основной АЛС2!"), Brushes.Red, null);
                    }
                }
                else
                {
                    if ((mdbRTUSrv.ServiceReg0.Value & 0x200) == 0x0)//бит 9 - КЗ гл.АСЛ 2
                    {
                        this.AddNewMessage2ConsoleALS2(new MessageCS(DateTime.Now, "M", "АЛС2", "КЗ в основной АЛС2 устранено."), Brushes.WhiteSmoke, null);
                    }
                }
                if ((mdbRTUSrv.ServiceReg0.PrevValue & 0x400) == 0x0)
                {
                    if ((mdbRTUSrv.ServiceReg0.Value & 0x400) == 0x400)//бит 10 - КЗ рез.АСЛ 2
                    {
                        this.AddNewMessage2ConsoleALS2(new MessageCS(DateTime.Now, "A", "АЛС2", "КЗ в резервной АЛС2!"), null, Brushes.Red);
                    }
                }
                else
                {
                    if ((mdbRTUSrv.ServiceReg0.Value & 0x400) == 0x0)//бит 10 - КЗ рез.АСЛ 2
                    {
                        this.AddNewMessage2ConsoleALS2(new MessageCS(DateTime.Now, "M", "АЛС2", "КЗ в резервной АЛС2 устранено."), null, Brushes.WhiteSmoke);
                    }
                }
            }
            catch { }
            mdbRTUSrv.ServiceReg0.PrevValue = mdbRTUSrv.ServiceReg0.Value;
        }
        //Обработка сервисного регистра1 по модбасу
        private void ServiceReg1_PropertyChanged(object sender, System.ComponentModel.PropertyChangedEventArgs e)
        {
            try
            {
                //Регистр№1 (Системная неисправность)
                if ((mdbTCPSrv.ServiceReg1.PrevValue & 0x1) == 0x0)
                {
                    if ((mdbTCPSrv.ServiceReg1.Value & 0x1) == 0x1)//бит 0 - осн. выше норм
                    {
                        this.AddNewMessage2Console(new MessageCS(DateTime.Now, "W", "Система", "Основное напряжение питания МОПС - выше нормы!"));
                    }
                }
                else
                {
                    if ((mdbTCPSrv.ServiceReg1.Value & 0x1) == 0x0)//бит 0 - осн. выше норм
                    {
                        this.AddNewMessage2Console(new MessageCS(DateTime.Now, "M", "Система", "Основное напряжение питания МОПС - в норме."));
                    }
                }
                if ((mdbTCPSrv.ServiceReg1.PrevValue & 0x2) == 0x0)
                {
                    if ((mdbTCPSrv.ServiceReg1.Value & 0x2) == 0x2)//бит 1 - осн.ниже норм
                    {
                        this.AddNewMessage2Console(new MessageCS(DateTime.Now, "W", "Система", "Основное напряжение питания МОПС - ниже нормы!"));
                    }
                }
                else
                {
                    if ((mdbTCPSrv.ServiceReg1.Value & 0x2) == 0x0)//бит 1 - осн.ниже норм
                    {
                        this.AddNewMessage2Console(new MessageCS(DateTime.Now, "M", "Система", "Основное напряжение питания МОПС - в норме."));
                    }
                }
                if ((mdbTCPSrv.ServiceReg1.PrevValue & 0x4) == 0x0)
                {
                    if ((mdbTCPSrv.ServiceReg1.Value & 0x4) == 0x4)//бит 2 - рез.выше норм
                    {
                        this.AddNewMessage2Console(new MessageCS(DateTime.Now, "W", "Система", "Резервное напряжение питания МОПС - выше нормы!"));
                    }
                }
                else
                {
                    if ((mdbTCPSrv.ServiceReg1.Value & 0x4) == 0x0)//бит 2 - рез.выше норм
                    {
                        this.AddNewMessage2Console(new MessageCS(DateTime.Now, "M", "Система", "Резервное напряжение питания МОПС - в норме."));
                    }
                }
                if ((mdbTCPSrv.ServiceReg1.PrevValue & 0x8) == 0x0)
                {
                    if ((mdbTCPSrv.ServiceReg1.Value & 0x8) == 0x8)//бит 3 - рез.ниже норм	
                    {
                        this.AddNewMessage2Console(new MessageCS(DateTime.Now, "W", "Система", "Резервное напряжение питания МОПС - ниже нормы!"));
                    }
                }
                else
                {
                    if ((mdbTCPSrv.ServiceReg1.Value & 0x8) == 0x0)//бит 3 - рез.ниже норм	
                    {
                        this.AddNewMessage2Console(new MessageCS(DateTime.Now, "M", "Система", "Резервное напряжение питания МОПС - в норме."));
                    }
                }
            }
            catch { }
            mdbTCPSrv.ServiceReg1.PrevValue = mdbTCPSrv.ServiceReg1.Value;
            /*bool_vals[4] = (short_v & 0x10) == 0x10;
            bool_vals[5] = (short_v & 0x20) == 0x20;
            bool_vals[6] = (short_v & 0x40) == 0x40;
            bool_vals[7] = (short_v & 0x80) == 0x80;
            bool_vals[8] = (short_v & 0x100) == 0x100;
            bool_vals[9] = (short_v & 0x200) == 0x200;
            bool_vals[10] = (short_v & 0x400) == 0x400;
            bool_vals[11] = (short_v & 0x800) == 0x800;
            bool_vals[12] = (short_v & 0x1000) == 0x1000;
            bool_vals[13] = (short_v & 0x2000) == 0x2000;
            bool_vals[14] = (short_v & 0x4000) == 0x4000;
            bool_vals[15] = (short_v & 0x8000) == 0x8000;*/
        }
        //Обработка сервисного регистра0 по модбасу
        private void ServiceReg0_PropertyChanged(object sender, System.ComponentModel.PropertyChangedEventArgs e)
        {
            try
            {
                //Регистр№0 (Состояние АЛС)
                if ((mdbTCPSrv.ServiceReg0.PrevValue & 0x1) == 0x0)
                {
                    if ((mdbTCPSrv.ServiceReg0.Value & 0x1) == 0x1)//бит 0 - разрыв АЛС 1
                    {
                        this.AddNewMessage2ConsoleALS1(new MessageCS(DateTime.Now, "A", "АЛС1", "Разрыв в АЛС1!"), Brushes.Red, Brushes.Red);
                    }
                }
                else
                {
                    if ((mdbTCPSrv.ServiceReg0.Value & 0x1) == 0x0)//бит 0 - разрыв АЛС 1
                    {
                        this.AddNewMessage2ConsoleALS1(new MessageCS(DateTime.Now, "M", "АЛС1", "Разрыв в АЛС1 устранен."), Brushes.WhiteSmoke, Brushes.WhiteSmoke);
                    }
                }
                if ((mdbTCPSrv.ServiceReg0.PrevValue & 0x2) == 0x0)
                {
                    if ((mdbTCPSrv.ServiceReg0.Value & 0x2) == 0x2)//бит 1 - КЗ гл.АСЛ 1
                    {
                        this.AddNewMessage2ConsoleALS1(new MessageCS(DateTime.Now, "A", "АЛС1", "КЗ в основной АЛС1!"), Brushes.Red, null);
                    }
                }
                else
                {
                    if ((mdbTCPSrv.ServiceReg0.Value & 0x2) == 0x0)//бит 1 - КЗ гл.АСЛ 1
                    {
                        this.AddNewMessage2ConsoleALS1(new MessageCS(DateTime.Now, "M", "АЛС1", "КЗ в основной АЛС1 устранено."), Brushes.WhiteSmoke, null);
                    }
                }
                if ((mdbTCPSrv.ServiceReg0.PrevValue & 0x4) == 0x0)
                {
                    if ((mdbTCPSrv.ServiceReg0.Value & 0x4) == 0x4)//бит 2 - КЗ рез.АСЛ 1
                    {
                        this.AddNewMessage2ConsoleALS1(new MessageCS(DateTime.Now, "A", "АЛС1", "КЗ в резервной АЛС1!"), null, Brushes.Red);
                    }
                }
                else
                {
                    if ((mdbTCPSrv.ServiceReg0.Value & 0x4) == 0x0)//бит 2 - КЗ рез.АСЛ 1
                    {
                        this.AddNewMessage2ConsoleALS1(new MessageCS(DateTime.Now, "M", "АЛС1", "КЗ в резервной АЛС1 устранено."), null, Brushes.WhiteSmoke);
                    }
                }
                if ((mdbTCPSrv.ServiceReg0.PrevValue & 0x100) == 0x0)
                {
                    if ((mdbTCPSrv.ServiceReg0.Value & 0x100) == 0x100)//бит 8 - разрыв АЛС 2
                    {
                        this.AddNewMessage2ConsoleALS2(new MessageCS(DateTime.Now, "A", "АЛС2", "Разрыв в АЛС2!"), Brushes.Red, Brushes.Red);
                    }
                }
                else
                {
                    if ((mdbTCPSrv.ServiceReg0.Value & 0x100) == 0x0)//бит 8 - разрыв АЛС 2
                    {
                        this.AddNewMessage2ConsoleALS2(new MessageCS(DateTime.Now, "M", "АЛС2", "Разрыв в АЛС2 устранен."), Brushes.WhiteSmoke, Brushes.WhiteSmoke);
                    }
                }
                if ((mdbTCPSrv.ServiceReg0.PrevValue & 0x200) == 0x0)
                {
                    if ((mdbTCPSrv.ServiceReg0.Value & 0x200) == 0x200)//бит 9 - КЗ гл.АСЛ 2
                    {
                        this.AddNewMessage2ConsoleALS2(new MessageCS(DateTime.Now, "A", "АЛС2", "КЗ в основной АЛС2!"), Brushes.Red, null);
                    }
                }
                else
                {
                    if ((mdbTCPSrv.ServiceReg0.Value & 0x200) == 0x0)//бит 9 - КЗ гл.АСЛ 2
                    {
                        this.AddNewMessage2ConsoleALS2(new MessageCS(DateTime.Now, "M", "АЛС2", "КЗ в основной АЛС2 устранено."), Brushes.WhiteSmoke, null);
                    }
                }
                if ((mdbTCPSrv.ServiceReg0.PrevValue & 0x400) == 0x0)
                {
                    if ((mdbTCPSrv.ServiceReg0.Value & 0x400) == 0x400)//бит 10 - КЗ рез.АСЛ 2
                    {
                        this.AddNewMessage2ConsoleALS2(new MessageCS(DateTime.Now, "A", "АЛС2", "КЗ в резервной АЛС2!"), null, Brushes.Red);
                    }
                }
                else
                {
                    if ((mdbTCPSrv.ServiceReg0.Value & 0x400) == 0x0)//бит 10 - КЗ рез.АСЛ 2
                    {
                        this.AddNewMessage2ConsoleALS2(new MessageCS(DateTime.Now, "M", "АЛС2", "КЗ в резервной АЛС2 устранено."), null, Brushes.WhiteSmoke);
                    }
                }
            }
            catch { }
            mdbTCPSrv.ServiceReg0.PrevValue = mdbTCPSrv.ServiceReg0.Value;
        }

        private void MdbRTUSrv_PropertyChanged(object sender, System.ComponentModel.PropertyChangedEventArgs e)
        {
            Application.Current.Dispatcher.BeginInvoke(
System.Windows.Threading.DispatcherPriority.Normal,
new Action(() =>
{
    try
    {
        this.ErrorTxtBox.Text = mdbRTUSrv.ErrorString;
    }
    catch { }
}));
        }

        private void AddNewMessage2ConsoleALS1(MessageCS msg, Brush br1, Brush br2)
        {
            this.MessengesConsole.MessagesList.Insert(0, msg);
            Application.Current.Dispatcher.BeginInvoke(System.Windows.Threading.DispatcherPriority.Normal, new Action(() =>
            {
                if (br1 != null)
                    this.ALS1LeftGrpBox.Foreground = br1;
                if (br2 != null)
                    this.ALS1RightGrpBox.Foreground = br2;
            }));
        }
        private void AddNewMessage2ConsoleALS2(MessageCS msg, Brush br1, Brush br2)
        {
            this.MessengesConsole.MessagesList.Insert(0, msg);
            Application.Current.Dispatcher.BeginInvoke(System.Windows.Threading.DispatcherPriority.Normal, new Action(() =>
            {
                if (br1!=null)
                    this.ALS2LeftGrpBox.Foreground = br1;
                if (br2!=null)
                    this.ALS2RightGrpBox.Foreground = br2;
            }));
        }
        //Отображение сообщения об ошибке
        private void MdbSrv_PropertyChanged(object sender, System.ComponentModel.PropertyChangedEventArgs e)
        {
            Application.Current.Dispatcher.BeginInvoke(
System.Windows.Threading.DispatcherPriority.Normal,
new Action(() =>
{
    try
    {
        this.ErrorTxtBox.Text = mdbTCPSrv.ErrorString;
    }
    catch { }
}));
        }
        //Смена выбранного регистра в списке для разбивки на биты
        private void RegistersList_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            if (e.AddedItems.Count != 0)
            {
                if (mdbValue != null)
                    mdbValue.PropertyChanged -= MdbValue_PropertyChanged;
                mdbValue = (MDBValue)e.AddedItems[0];
                mdbValue.PropertyChanged += MdbValue_PropertyChanged;
            }
        }
        //Разбивка на биты значения выбранного регистра в списке
        private void MdbValue_PropertyChanged(object sender, System.ComponentModel.PropertyChangedEventArgs e)
        {
            Application.Current.Dispatcher.BeginInvoke(
System.Windows.Threading.DispatcherPriority.Normal,
new Action(() =>
{
try
{
        this.Bit0ChkBox.IsChecked = (mdbValue.Value & 0x1) == 0x1;
        this.Bit1ChkBox.IsChecked = (mdbValue.Value & 0x2) == 0x2;
        this.Bit2ChkBox.IsChecked = (mdbValue.Value & 0x4) == 0x4;
        this.Bit3ChkBox.IsChecked = (mdbValue.Value & 0x8) == 0x8;
        this.Bit4ChkBox.IsChecked = (mdbValue.Value & 0x10) == 0x10;
        this.Bit5ChkBox.IsChecked = (mdbValue.Value & 0x20) == 0x20;
        this.Bit6ChkBox.IsChecked = (mdbValue.Value & 0x40) == 0x40;
        this.Bit7ChkBox.IsChecked = (mdbValue.Value & 0x80) == 0x80;
        this.Bit8ChkBox.IsChecked = (mdbValue.Value & 0x100) == 0x100;
        this.Bit9ChkBox.IsChecked = (mdbValue.Value & 0x200) == 0x200;
        this.Bit10ChkBox.IsChecked = (mdbValue.Value & 0x400) == 0x400;
        this.Bit11ChkBox.IsChecked = (mdbValue.Value & 0x800) == 0x800;
        this.Bit12ChkBox.IsChecked = (mdbValue.Value & 0x1000) == 0x1000;
        this.Bit13ChkBox.IsChecked = (mdbValue.Value & 0x2000) == 0x2000;
        this.Bit14ChkBox.IsChecked = (mdbValue.Value & 0x4000) == 0x4000;
        this.Bit15ChkBox.IsChecked = (mdbValue.Value & 0x8000) == 0x8000;
    }
    catch { }
}));
        }
        //Выдача значения команды в МОПС по Модбасу
        private void WriteCmdBtn_Click(object sender, RoutedEventArgs e)
        {
            this.mdbTCPSrv.SendValue = ushort.Parse(this.WriteValueTxtBox.Text);
            this.mdbTCPSrv.SendAddress= ushort.Parse(this.AddrWriteRegTxtBox.Text);
            this.mdbTCPSrv.SendCommandFlag = true;
        }

        private void MDBrtuRadioBtn_Checked(object sender, RoutedEventArgs e)
        {
            this.SerialPortNameCmbBox.Items.Clear();
            string[] pnames = SerialPort.GetPortNames();
            foreach (string p in pnames)
                this.SerialPortNameCmbBox.Items.Add(p);
        }

        private void MessagesListView_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.Key == Key.Enter)
            {
                StringBuilder evnts = new StringBuilder();
                foreach (MessageCS msg in this.MessagesListView.Items)
                {
                    evnts.AppendLine(msg.DT.ToString("F") + ";" + msg.Category + ";" + msg.ObjectName + ";" + msg.Message);
                }
                Clipboard.SetText(evnts.ToString());
            }
        }

        private void RebootMOPSBtn_Click(object sender, RoutedEventArgs e)
        {
            MessageBoxResult res = MessageBox.Show("Вы уверены?", "Перезагрузка МОПС-R2", MessageBoxButton.YesNo);
            if (res == MessageBoxResult.Yes)
            {
                SendManager.RebootMOPSCommand();
            }
        }
    }

    public class KAUConfig
    {
        public string IP1 { set; get; }
        public string Mask1 { set; get; }
        public string MAC1 { set; get; }
        public string Protocol1 { set; get; }
        public string IP2 { set; get; }
        public string Mask2 { set; get; }
        public string MAC2 { set; get; }
        public string Protocol2 { set; get; }
        public int BaudRate { set; get; }
        public int Parity { set; get; }
        public int Data { set; get; }
        public int Stop { set; get; }
        public int Timeout { set; get; }
        public int Address { set; get; }
    }

    public class MyData
    {
        public uint ser_no { get; set; }
        public int addr { get; set; }
        public int status { get; set; }
        public int type { get; set; }
        public int CC { get; set; }
        public List<byte> data { get; set; }
    }

    [StructLayout(LayoutKind.Explicit)]
    public struct UDataConvert
    {
        [FieldOffset(0)]
        public byte b0;
        [FieldOffset(1)]
        public byte b1;
        [FieldOffset(2)]
        public byte b2;
        [FieldOffset(3)]
        public byte b3;
        [FieldOffset(4)]
        public byte b4;
        [FieldOffset(5)]
        public byte b5;
        [FieldOffset(6)]
        public byte b6;
        [FieldOffset(7)]
        public byte b7;

        [FieldOffset(0)]
        public ushort shrt0;
        [FieldOffset(2)]
        public ushort shrt1;
        [FieldOffset(4)]
        public ushort shrt2;
        [FieldOffset(6)]
        public ushort shrt3;

        [FieldOffset(0)]
        public uint i0;
        [FieldOffset(4)]
        public uint i1;

        [FieldOffset(0)]
        public ulong l0;

        [FieldOffset(0)]
        public float f0;

        [FieldOffset(0)]
        public double d0;
    }

}
